@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2009 Kevin Funk <krf@electrostorm.net>                                 *
+ * Copyright (c) 2009-2011 Kevin Funk <krf@electrostorm.net>                            *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -16,14 +16,17 @@
 
 #include "KNotificationBackend.h"
 
-#include "Amarok.h"
-#include "Debug.h"
+#include "core/support/Amarok.h"
+#include "core/meta/Meta.h"
+#include "SvgHandler.h"
 #include "EngineController.h"
-#include "meta/Meta.h"
+
+#include <KIcon>
+#include <KLocale>
+#include <KWindowSystem>
 
 #include <QTimer>
 
-
 namespace Amarok
 {
     KNotificationBackend*
@@ -43,25 +46,52 @@ namespace Amarok
 }
 
 Amarok::KNotificationBackend::KNotificationBackend()
-    : EngineObserver( The::engineController() )
-    , m_notify( 0 )
+    : m_notify( 0 )
     , m_enabled( false )
 {
-    DEBUG_BLOCK
-
     m_timer = new QTimer( this );
     m_timer->setSingleShot( true );
     connect( m_timer, SIGNAL( timeout() ), this, SLOT( showCurrentTrack() ) );
+
+    EngineController *engine = The::engineController();
+
+    connect( engine, SIGNAL( trackPlaying( Meta::TrackPtr ) ),
+             this, SLOT( trackPlaying() ) );
+    connect( engine, SIGNAL( trackMetadataChanged( Meta::TrackPtr ) ),
+             this, SLOT( trackPlaying() ) );
+    connect( engine, SIGNAL( albumMetadataChanged( Meta::AlbumPtr ) ),
+             this, SLOT( trackPlaying() ) );
+
 }
 
 Amarok::KNotificationBackend::~KNotificationBackend()
 {
-    DEBUG_BLOCK
-
     if (m_notify)
         m_notify->close();
 }
 
+void
+Amarok::KNotificationBackend::setEnabled(bool enable)
+{
+    m_enabled = enable;
+}
+
+bool
+Amarok::KNotificationBackend::isEnabled() const
+{
+    return m_enabled;
+}
+
+bool
+Amarok::KNotificationBackend::isFullscreenWindowActive() const
+{
+    // Get information of the active window.
+    KWindowInfo activeWindowInfo = KWindowSystem::windowInfo( KWindowSystem::activeWindow(), NET::WMState );
+
+    // Check if it is running in fullscreen mode.
+    return activeWindowInfo.hasState( NET::FullScreen );
+}
+
 void
 Amarok::KNotificationBackend::notificationClosed()
 {
@@ -70,55 +100,52 @@ Amarok::KNotificationBackend::notificationClosed()
 }
 
 void
-Amarok::KNotificationBackend::engineStateChanged( Phonon::State state, Phonon::State /*oldState*/ )
+Amarok::KNotificationBackend::trackPlaying()
 {
-    switch( state )
-    {
-        case Phonon::PlayingState:
-        case Phonon::StoppedState:
-        case Phonon::PausedState:
-        case Phonon::LoadingState:
-            break;
-        case Phonon::ErrorState:
-            if( m_timer->isActive() )
-                m_timer->stop(); // Do not notify if track cannot be played
-        case Phonon::BufferingState:
-            break;
-    }
-}
-
-void
-Amarok::KNotificationBackend::engineNewTrackPlaying()
-{
-    DEBUG_BLOCK
-
-    m_timer->start( 3000 ); // Wait some time to display the correct cover
-}
-
-void
-Amarok::KNotificationBackend::showCurrentTrack() // slot
-{
-    DEBUG_BLOCK
-
     if( !m_enabled )
+        return; // KNotify is disabled, so don't start timer
+
+    m_timer->start( 3000 ); // Wait some time to display the correct cover and also to check if phonon really managed to play the track
+}
+
+void Amarok::KNotificationBackend::show(const QString& title, const QString& body, const QPixmap& pixmap)
+{
+    QPixmap icon;
+    if (pixmap.isNull()) {
+        KIconLoader loader;
+        icon = loader.loadIcon( QString("amarok"), KIconLoader::Desktop );
+    }
+    else
+        icon = pixmap;
+
+    KNotification::event( KNotification::Notification , title, body, icon );
+}
+
+void
+Amarok::KNotificationBackend::showCurrentTrack( bool force ) // slot
+{
+    if( !force && !m_enabled )
         return;
 
-    Meta::TrackPtr track = The::engineController()->currentTrack();
-    if( track )
+    EngineController *engine = The::engineController();
+
+    Meta::TrackPtr track = engine->currentTrack();
+    if( engine->isPlaying() && track )
     {
         if( m_notify ) {
-            m_notify->close(); // Close old notification when switching quickly between tracks
+            m_notify->close(); // Close old notification (when switching quickly between tracks)
         }
 
         m_notify = new KNotification( "trackChange" );
         connect( m_notify, SIGNAL(closed()), this, SLOT(notificationClosed()) );
 
-        if( track->album() )
-            m_notify->setPixmap( track->album()->imageWithBorder( 80 ) );
+        Meta::AlbumPtr album = track->album();
+        if( album )
+            m_notify->setPixmap( The::svgHandler()->imageWithBorder( album, 80 ) );
 
         m_notify->setTitle( i18n( "Now playing" ) );
 
-        m_notify->setText( Amarok::prettyNowPlaying() );
+        m_notify->setText( engine->prettyNowPlaying() );
         m_notify->sendEvent();
     }
 }
