@@ -31,7 +31,6 @@
 #include "KNotificationBackend.h"
 #include "Osd.h"
 #include "PaletteHandler.h"
-#include "ScriptManager.h"
 #include "amarokconfig.h"
 #include "aboutdialog/OcsData.h"
 #include "amarokurls/AmarokUrlHandler.h"
@@ -43,6 +42,7 @@
 #include "context/ContextDock.h"
 #include "core-impl/collections/support/CollectionManager.h"
 #include "covermanager/CoverManager.h" // for actions
+#include "dialogs/DiagnosticDialog.h"
 #include "dialogs/EqualizerDialog.h"
 #include "likeback/LikeBack.h"
 #include "moodbar/MoodbarManager.h"
@@ -59,13 +59,11 @@
 #include "playlistmanager/file/PlaylistFileProvider.h"
 #include "playlistmanager/PlaylistManager.h"
 #include "PodcastCategory.h"
-#include "services/ServicePluginManager.h"
 #include "services/scriptable/ScriptableService.h"
-#include "statusbar/StatusBar.h"
 #include "toolbar/SlimToolbar.h"
 #include "toolbar/MainToolbar.h"
 #include "SvgHandler.h"
-#include "widgets/Splitter.h"
+#include "PluginManager.h"
 //#include "mediabrowser.h"
 
 #include <KAction>          //m_actionCollection
@@ -77,7 +75,6 @@
 #include <KLocale>
 #include <KMenu>
 #include <KMenuBar>
-#include <KPixmapCache>
 #include <KBugReport>
 #include <KStandardAction>
 #include <KStandardDirs>
@@ -91,7 +88,6 @@
 #include <QDesktopWidget>
 #include <QDockWidget>
 #include <QList>
-#include <QSizeGrip>
 #include <QStyle>
 #include <QVBoxLayout>
 
@@ -107,7 +103,6 @@
 
 #define AMAROK_CAPTION I18N_NOOP( "Amarok" )
 
-extern KAboutData aboutData;
 extern OcsData ocsData;
 
 QWeakPointer<MainWindow> MainWindow::s_instance;
@@ -128,7 +123,6 @@ MainWindow::MainWindow()
     s_instance = this;
 
 #ifdef Q_WS_MAC
-    QSizeGrip* grip = new QSizeGrip( this );
     GrowlInterface* growl = new GrowlInterface( qApp->applicationName() );
 #endif
 
@@ -136,30 +130,19 @@ MainWindow::MainWindow()
     CollectionManager::instance();
     PERF_LOG( "Started Collection Manager instance" )
 
-    /* The ServicePluginManager needs to be loaded before the playlist model
-    * (which gets started by "statusBar" below up so that it can handle any
+    /* The PluginManager needs to be loaded before the playlist model
+    * (which gets started by "statusBar::connectPlaylist" below so that it can handle any
     * tracks in the saved playlist that are associated with services. Eg, if
     * the playlist has a Magnatune track in it when Amarok is closed, then the
     * Magnatune service needs to be initialized before the playlist is loaded
     * here. */
+    PERF_LOG( "Instantiate Plugin Manager" )
+    The::pluginManager();
+    PERF_LOG( "Started Plugin Manager instance" )
 
-    PERF_LOG( "Instantiate Service Plugin Manager" )
-    ServicePluginManager::instance();
-    PERF_LOG( "Started Service Plugin Manager instance" )
-
-    PERF_LOG( "Set Status Bar" )
-    StatusBar * statusBar = new StatusBar( this );
-    setStatusBar( statusBar );
-    PERF_LOG( "Created Status Bar" )
-
-    // Sets caption and icon correctly (needed e.g. for GNOME)
-//     kapp->setTopWidget( this );
-    PERF_LOG( "Set Top Widget" )
     createActions();
     PERF_LOG( "Created actions" )
 
-    //new K3bExporter();
-
     The::paletteHandler()->setPalette( palette() );
     setPlainCaption( i18n( AMAROK_CAPTION ) );
 
@@ -194,11 +177,9 @@ MainWindow::~MainWindow()
     //save currently active category
     Amarok::config().writeEntry( "Browser Path", m_browserDock.data()->list()->path() );
 
-    //delete m_splitter;
 #ifdef DEBUG_BUILD_TYPE
     delete m_networkViewer.data();
 #endif // DEBUG_BUILD_TYPE
-    delete The::statusBar();
     delete The::svgHandler();
     delete The::paletteHandler();
 }
@@ -222,7 +203,6 @@ MainWindow::init()
     m_mainToolbar.data()->setAllowedAreas( Qt::TopToolBarArea | Qt::BottomToolBarArea );
     m_mainToolbar.data()->setMovable ( true );
     addToolBar( Qt::TopToolBarArea, m_mainToolbar.data() );
-    m_mainToolbar.data()->hide();
 
     //create slim toolbar
     m_slimToolbar = new SlimToolbar( 0 );
@@ -242,6 +222,9 @@ MainWindow::init()
     PERF_LOG( "Create Playlist" )
     m_playlistDock = new Playlist::Dock( this );
     m_playlistDock.data()->installEventFilter( this );
+    //HACK, need to connect after because of order in MainWindow()
+    connect( Amarok::actionCollection()->action( "playlist_edit_queue" ),
+             SIGNAL( triggered( bool ) ), m_playlistDock.data(), SLOT( slotEditQueue() ) );
     PERF_LOG( "Playlist created" )
 
     PERF_LOG( "Creating ContextWidget" )
@@ -257,7 +240,8 @@ MainWindow::init()
 
     PERF_LOG( "Loaded default contextScene" )
 
-    setDockOptions ( QMainWindow::AllowNestedDocks | QMainWindow::AllowTabbedDocks | QMainWindow::AnimatedDocks );
+    setDockOptions( QMainWindow::AllowNestedDocks | QMainWindow::AllowTabbedDocks
+                    | QMainWindow::AnimatedDocks | QMainWindow::VerticalTabs );
 
     addDockWidget( Qt::LeftDockWidgetArea, m_browserDock.data() );
     addDockWidget( Qt::LeftDockWidgetArea, m_contextDock.data(), Qt::Horizontal );
@@ -269,8 +253,10 @@ MainWindow::init()
     {
         Debug::Block block( "Creating browsers. Please report long start times!" );
 
+        //TODO: parent these browsers?
         PERF_LOG( "Creating CollectionWidget" )
         m_collectionBrowser = new CollectionWidget( "collections", 0 );
+        //TODO: rename "Music Collections
         m_collectionBrowser->setPrettyName( i18n( "Local Music" ) );
         m_collectionBrowser->setIcon( KIcon( "collection-amarok" ) );
         m_collectionBrowser->setShortDescription( i18n( "Local sources of content" ) );
@@ -279,12 +265,12 @@ MainWindow::init()
 
 
         PERF_LOG( "Creating ServiceBrowser" )
-        ServiceBrowser *internetContentServiceBrowser = ServiceBrowser::instance();
-        internetContentServiceBrowser->setParent( 0 );
-        internetContentServiceBrowser->setPrettyName( i18n( "Internet" ) );
-        internetContentServiceBrowser->setIcon( KIcon( "applications-internet" ) );
-        internetContentServiceBrowser->setShortDescription( i18n( "Online sources of content" ) );
-        m_browserDock.data()->list()->addCategory( internetContentServiceBrowser );
+        ServiceBrowser *serviceBrowser = ServiceBrowser::instance();
+        serviceBrowser->setParent( 0 );
+        serviceBrowser->setPrettyName( i18n( "Internet" ) );
+        serviceBrowser->setIcon( KIcon( "applications-internet" ) );
+        serviceBrowser->setShortDescription( i18n( "Online sources of content" ) );
+        m_browserDock.data()->list()->addCategory( serviceBrowser );
         PERF_LOG( "Created ServiceBrowser" )
 
         PERF_LOG( "Creating PlaylistBrowser" )
@@ -295,22 +281,15 @@ MainWindow::init()
         m_browserDock.data()->list()->addCategory( m_playlistBrowser );
         PERF_LOG( "CreatedPlaylsitBrowser" )
 
-
         PERF_LOG( "Creating FileBrowser" )
-        FileBrowser * fileBrowserMkII = new FileBrowser( "files", 0 );
-        fileBrowserMkII->setPrettyName( i18n("Files") );
-        fileBrowserMkII->setIcon( KIcon( "folder-amarok" ) );
-        fileBrowserMkII->setShortDescription( i18n( "Browse local hard drive for content" ) );
-        m_browserDock.data()->list()->addCategory( fileBrowserMkII );
-
-
+        FileBrowser *fileBrowser = new FileBrowser( "files", 0 );
+        fileBrowser->setPrettyName( i18n("Files") );
+        fileBrowser->setIcon( KIcon( "folder-amarok" ) );
+        fileBrowser->setShortDescription( i18n( "Browse local hard drive for content" ) );
+        m_browserDock.data()->list()->addCategory( fileBrowser );
         PERF_LOG( "Created FileBrowser" )
 
-        PERF_LOG( "Initialising ServicePluginManager" )
-        ServicePluginManager::instance()->init();
-        PERF_LOG( "Initialised ServicePluginManager" )
-
-        internetContentServiceBrowser->setScriptableServiceManager( The::scriptableServiceManager() );
+        serviceBrowser->setScriptableServiceManager( The::scriptableServiceManager() );
         PERF_LOG( "ScriptableServiceManager done" )
 
         PERF_LOG( "Creating Podcast Category" )
@@ -513,6 +492,7 @@ MainWindow::exportPlaylist() const //SLOT
 
     KFileDialog fileDialog( KUrl("kfiledialog:///amarok-playlist-export"), QString(), 0 );
     QCheckBox *saveRelativeCheck = new QCheckBox( i18n("Use relative path for &saving") );
+    saveRelativeCheck->setChecked( AmarokConfig::relativePlaylist() );
 
     QStringList supportedMimeTypes;
     supportedMimeTypes << "audio/x-mpegurl"; //M3U
@@ -531,10 +511,7 @@ MainWindow::exportPlaylist() const //SLOT
     QString playlistPath = fileDialog.selectedFile();
 
     if( !playlistPath.isEmpty() )
-    {
-        AmarokConfig::setRelativePlaylist( saveRelativeCheck->isChecked() );
-        The::playlist()->exportPlaylist( playlistPath );
-    }
+        The::playlist()->exportPlaylist( playlistPath, saveRelativeCheck->isChecked() );
 }
 
 void
@@ -543,12 +520,25 @@ MainWindow::slotShowActiveTrack() const
     m_playlistDock.data()->showActiveTrack();
 }
 
+void
+MainWindow::slotEditTrackInfo() const
+{
+    m_playlistDock.data()->editTrackInfo();
+}
+
 void
 MainWindow::slotShowCoverManager() const //SLOT
 {
     CoverManager::showOnce();
 }
 
+void
+MainWindow::slotShowDiagnosticsDialog() const
+{
+    DiagnosticDialog *dialog = new DiagnosticDialog( KGlobal::mainComponent().aboutData() );
+    dialog->show();
+}
+
 void MainWindow::slotShowBookmarkManager() const
 {
     The::bookmarkManager()->showOnce();
@@ -607,18 +597,19 @@ MainWindow::slotAddStream() //SLOT
 }
 
 void
-MainWindow::slotJumpTo() // slot
+MainWindow::slotFocusPlaylistSearch()
 {
-    DEBUG_BLOCK
-
+    showDock( AmarokDockPlaylist );  // ensure that the dock is visible if tabbed
     m_playlistDock.data()->searchWidget()->focusInputLine();
 }
 
 void
-MainWindow::showScriptSelector() //SLOT
+MainWindow::slotFocusCollectionSearch()
 {
-    ScriptManager::instance()->show();
-    ScriptManager::instance()->raise();
+    // ensure collection browser is activated within navigation dock:
+    browserDock()->list()->navigate( QString("collections") );
+    showDock( AmarokDockNavigation );  // ensure that the dock is visible if tabbed
+    m_collectionBrowser->focusInputLine();
 }
 
 #ifdef DEBUG_BUILD_TYPE
@@ -736,8 +727,11 @@ MainWindow::createActions()
     connect( action, SIGNAL( triggered( bool ) ), pc, SLOT( clear() ) );
     ac->addAction( "playlist_clear", action );
 
-    action = new KAction( KIcon( "format-list-ordered" ), i18nc( "edit play queue of playlist", "Edit &Queue" ), this );
-    ac->addAction( "playlist_edit_queue", action );
+    action = new KAction( KIcon( "format-list-ordered" ),
+                          i18nc( "edit play queue of playlist", "Edit &Queue" ), this );
+    //Qt::META+Qt::Key_Q is taken by Plasma as a global
+    action->setShortcut( KShortcut( Qt::META + Qt::Key_U ) );
+    ac->addAction( "playlist_edit_queue", action );;
 
     action = new KAction( i18nc( "Remove duplicate and dead (unplayable) tracks from the playlist", "Re&move Duplicates" ), this );
     connect( action, SIGNAL( triggered( bool ) ), pc, SLOT( removeDeadAndDuplicates() ) );
@@ -783,9 +777,10 @@ MainWindow::createActions()
     action->setShortcut( Qt::CTRL + Qt::Key_O );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( slotPlayMedia() ) );
 
-    action = new KAction( KIcon("preferences-plugin-script-amarok"), i18n("Script Manager"), this );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( showScriptSelector() ) );
-    ac->addAction( "script_manager", action );
+    action = new KAction( KIcon("media-track-edit-amarok"), i18n("Edit Details of Currently Selected Track"), this );
+    ac->addAction( "trackdetails_edit", action );
+    action->setShortcut( Qt::CTRL + Qt::Key_E );
+    connect( action, SIGNAL( triggered( bool ) ), SLOT( slotEditTrackInfo() ) );
 
     action = new KAction( KIcon( "media-seek-forward-amarok" ), i18n("&Seek Forward"), this );
     ac->addAction( "seek_forward", action );
@@ -857,10 +852,15 @@ MainWindow::createActions()
     action->setShortcut( KShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_F ) );
     connect( action, SIGNAL( triggered() ), SLOT( slotFullScreen() ) );
 
-    action = new KAction( i18n( "Jump to" ), this );
-    ac->addAction( "jumpTo", action );
+    action = new KAction( i18n( "Search playlist" ), this );
+    ac->addAction( "searchPlaylist", action );
     action->setShortcut( KShortcut( Qt::CTRL + Qt::Key_J ) );
-    connect( action, SIGNAL( triggered() ), SLOT( slotJumpTo() ) );
+    connect( action, SIGNAL( triggered() ), SLOT( slotFocusPlaylistSearch()) );
+
+    action = new KAction( i18n( "Search collection" ), this );
+    ac->addAction( "searchCollection", action );
+    action->setShortcut( KShortcut( Qt::CTRL + Qt::Key_F ) );
+    connect( action, SIGNAL( triggered() ), SLOT( slotFocusCollectionSearch()) );
 
     action = new KAction( KIcon( "music-amarok" ), i18n("Show active track"), this );
     ac->addAction( "show_active_track", action );
@@ -944,6 +944,10 @@ MainWindow::createActions()
     ac->addAction( "extendedAbout", action );
     connect( action, SIGNAL( triggered() ), SLOT( showAbout() ) );
 
+    action = new KAction ( KIcon( "info-amarok" ), i18n( "&Diagnostics" ), this );
+    ac->addAction( "diagnosticDialog", action );
+    connect( action, SIGNAL( triggered() ), SLOT( slotShowDiagnosticsDialog() ) );
+
     action = new KAction( KIcon( "tools-report-bug" ), i18n("&Report Bug..."), this );
     ac->addAction( "reportBug", action );
     connect( action, SIGNAL( triggered() ), SLOT( showReportBug() ) );
@@ -995,19 +999,17 @@ MainWindow::setRating( int n )
 void
 MainWindow::createMenus()
 {
+    m_menubar = menuBar();
+
     //BEGIN Actions menu
-    KMenu *actionsMenu;
+    KMenu *actionsMenu = new KMenu( m_menubar.data() );
 #ifdef Q_WS_MAC
-    m_menubar = new QMenuBar( 0 );  // Fixes menubar in OS X
-    actionsMenu = new KMenu( m_menubar.data() );
     // Add these functions to the dock icon menu in OS X
     //extern void qt_mac_set_dock_menu(QMenu *);
     //qt_mac_set_dock_menu(actionsMenu);
     // Change to avoid duplicate menu titles in OS X
     actionsMenu->setTitle( i18n("&Music") );
 #else
-    m_menubar = menuBar();
-    actionsMenu = new KMenu( m_menubar.data() );
     actionsMenu->setTitle( i18n("&Amarok") );
 #endif
     actionsMenu->addAction( Amarok::actionCollection()->action("playlist_playmedia") );
@@ -1044,6 +1046,7 @@ MainWindow::createMenus()
     playlistMenu->addAction( Amarok::actionCollection()->action("playlist_clear") );
     playlistMenu->addAction( Amarok::actionCollection()->action("playlist_remove_dead_and_duplicates") );
     playlistMenu->addAction( Amarok::actionCollection()->action("playlist_layout") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_edit_queue") );
     //END Playlist menu
 
     //BEGIN Tools menu
@@ -1053,7 +1056,9 @@ MainWindow::createMenus()
     m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("bookmark_manager") );
     m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("cover_manager") );
     m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("equalizer_dialog") );
+#ifdef QTSCRIPTQTBINDINGS_FOUND
     m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("script_manager") );
+#endif
 #ifdef DEBUG_BUILD_TYPE
     m_toolsMenu.data()->addAction( Amarok::actionCollection()->action("network_request_viewer") );
 #endif // DEBUG_BUILD_TYPE
@@ -1094,11 +1099,14 @@ MainWindow::createMenus()
                             Amarok::actionCollection()->action( "reportBug" ) );
     helpMenu->insertAction( helpMenu->actions().last(),
                             Amarok::actionCollection()->action( "extendedAbout" ) );
+    helpMenu->insertAction( helpMenu->actions().last(),
+                            Amarok::actionCollection()->action( "diagnosticDialog" ) );
     helpMenu->insertAction( helpMenu->actions().at( 4 ),
                             Amarok::actionCollection()->action( "likeBackSendComment" ) );
     helpMenu->insertAction( helpMenu->actions().at( 5 ),
                             Amarok::actionCollection()->action( "likeBackShowIcons" ) );
 
+    m_menubar.data()->addSeparator();
     m_menubar.data()->addMenu( helpMenu );
 }
 
@@ -1124,7 +1132,7 @@ MainWindow::slotShowMenuBar()
 void
 MainWindow::showAbout()
 {
-    ExtendedAboutDialog dialog( &aboutData, &ocsData );
+    ExtendedAboutDialog dialog( KGlobal::mainComponent().aboutData(), &ocsData );
     dialog.exec();
 }
 
@@ -1141,8 +1149,6 @@ MainWindow::paletteChange( const QPalette & oldPalette )
 {
     Q_UNUSED( oldPalette )
 
-    KPixmapCache cache( "Amarok-pixmaps" );
-    cache.discard();
     The::paletteHandler()->setPalette( palette() );
 }
 
@@ -1186,12 +1192,6 @@ MainWindow::activeBrowserName()
         return QString();
 }
 
-PlaylistBrowserNS::PlaylistBrowser *
-MainWindow::playlistBrowser()
-{
-    return m_playlistBrowser;
-}
-
 void
 MainWindow::setLayoutLocked( bool locked )
 {
@@ -1273,7 +1273,7 @@ MainWindow::restoreLayout()
         const int widgetWidth = totalWidgetWidth / 3;
         const int leftover = totalWidgetWidth - 3*widgetWidth;
 
-        //We need to set fixed widths initially, just until the main window has been properly layed out. As soon as this has
+        //We need to set fixed widths initially, just until the main window has been properly laid out. As soon as this has
         //happened, we will unlock these sizes again so that the elements can be resized by the user.
         const int mins[3] = { m_browserDock.data()->minimumWidth(), m_contextDock.data()->minimumWidth(), m_playlistDock.data()->minimumWidth() };
         const int maxs[3] = { m_browserDock.data()->maximumWidth(), m_contextDock.data()->maximumWidth(), m_playlistDock.data()->maximumWidth() };
@@ -1306,6 +1306,7 @@ MainWindow::playAudioCd()
 
     QList<Collections::Collection*> collections = CollectionManager::instance()->viewableCollections();
 
+    // Search a non-empty MemoryCollection with the id: AudioCd
     foreach( Collections::Collection *collection, collections )
     {
         if( collection->collectionId() == "AudioCd" )
@@ -1324,11 +1325,7 @@ MainWindow::playAudioCd()
 
             The::engineController()->stop( true );
             The::playlistController()->clear();
-
-            Collections::QueryMaker * qm = collection->queryMaker();
-            qm->setQueryType( Collections::QueryMaker::Track );
-            The::playlistController()->insertOptioned( qm, Playlist::DirectPlay );
-
+            The::playlistController()->insertOptioned( cdColl->trackMap().values(), Playlist::DirectPlay );
             m_waitingForCd = false;
             return true;
         }
