@@ -102,7 +102,6 @@ class AMAROK_EXPORT MainWindow : public KMainWindow
         QString activeBrowserName();
 
         CollectionWidget * collectionBrowser();
-        PlaylistBrowserNS::PlaylistBrowser * playlistBrowser();
 
         bool isLayoutLocked() const;
 
@@ -137,15 +136,17 @@ class AMAROK_EXPORT MainWindow : public KMainWindow
 
         void exportPlaylist() const;
         void slotShowActiveTrack() const;
+        void slotEditTrackInfo() const;
         void slotShowBookmarkManager() const;
         void slotShowEqualizer() const;
         void slotShowCoverManager() const;
+        void slotShowDiagnosticsDialog() const;
         void slotShowMenuBar();
         void slotPlayMedia();
         void slotAddLocation( bool directPlay = false );
         void slotAddStream();
-        void slotJumpTo();
-        void showScriptSelector();
+        void slotFocusPlaylistSearch();
+        void slotFocusCollectionSearch();
 #ifdef DEBUG_BUILD_TYPE
         void showNetworkRequestViewer();
 #endif // DEBUG_BUILD_TYPE
