@@ -145,11 +145,12 @@ CollectionTreeItem::data( int role ) const
     }
     else if( m_parentCollection )
     {
-        static const QString counting = i18n( "Counting" );
+        static const QString counting = i18n( "Counting..." );
         switch( role )
         {
         case Qt::DisplayRole:
         case CustomRoles::FilterRole:
+        case CustomRoles::SortRole:
             return m_parentCollection->prettyName();
         case Qt::DecorationRole:
             return m_parentCollection->icon();
@@ -161,8 +162,8 @@ CollectionTreeItem::data( int role ) const
                 m_isCounting = true;
 
                 Collections::QueryMaker *qm = m_parentCollection->queryMaker();
-                connect( qm, SIGNAL( newResultReady(QString, QStringList) ),
-                         SLOT( tracksCounted(QString, QStringList) ) );
+                connect( qm, SIGNAL( newResultReady(QStringList) ),
+                         SLOT( tracksCounted(QStringList) ) );
 
                 qm->setAutoDelete( true )
                   ->setQueryType( Collections::QueryMaker::Custom )
@@ -175,9 +176,9 @@ CollectionTreeItem::data( int role ) const
         case CustomRoles::HasCapacityRole:
             return m_parentCollection->hasCapacity();
         case CustomRoles::UsedCapacityRole:
-            if( m_parentCollection->hasCapacity() && m_parentCollection->totalCapacity() > 0 )
-                return m_parentCollection->usedCapacity() * 100 / m_parentCollection->totalCapacity();
-            break;
+            return m_parentCollection->usedCapacity();
+        case CustomRoles::TotalCapacityRole:
+            return m_parentCollection->totalCapacity();
         case CustomRoles::DecoratorRoleCount:
             return decoratorActions().size();
         case CustomRoles::DecoratorRole:
@@ -200,9 +201,8 @@ CollectionTreeItem::decoratorActions() const
 }
 
 void
-CollectionTreeItem::tracksCounted( QString collectionId, QStringList res )
+CollectionTreeItem::tracksCounted( QStringList res )
 {
-    Q_UNUSED( collectionId );
     if( !res.isEmpty() )
         m_trackCount = res.first().toInt();
     else
@@ -221,8 +221,13 @@ int
 CollectionTreeItem::row() const
 {
     if( m_parent )
-        return m_parent->m_childItems.indexOf( const_cast<CollectionTreeItem*>(this) );
-
+    {
+        const QList<CollectionTreeItem*> &children = m_parent->m_childItems;
+        if( !children.isEmpty() && children.contains( const_cast<CollectionTreeItem*>(this) ) )
+            return children.indexOf( const_cast<CollectionTreeItem*>(this) );
+        else
+            return -1;
+    }
     return 0;
 }
 
@@ -274,17 +279,24 @@ CollectionTreeItem::queryMaker() const
 }
 
 void
-CollectionTreeItem::addMatch( Collections::QueryMaker *qm ) const
+CollectionTreeItem::addMatch( Collections::QueryMaker *qm, CategoryId::CatMenuId levelCategory ) const
 {
-    if( !m_data || m_type != Data )
-        return;
     if( !qm )
         return;
 
-    if( Meta::TrackPtr track = Meta::TrackPtr::dynamicCast( m_data ) )
+    if( isVariousArtistItem() )
+        qm->setAlbumQueryMode( Collections::QueryMaker::OnlyCompilations );
+    if( isNoLabelItem() )
+        qm->setLabelQueryMode( Collections::QueryMaker::OnlyWithoutLabels );
+    else if( Meta::TrackPtr track = Meta::TrackPtr::dynamicCast( m_data ) )
         qm->addMatch( track );
     else if( Meta::ArtistPtr artist = Meta::ArtistPtr::dynamicCast( m_data ) )
+    {
+        if( levelCategory == CategoryId::AlbumArtist )
+            // this needs to be BEFORE addMatch(), because addMatch() resets ArtistQueryMode
+            qm->setArtistQueryMode( Collections::QueryMaker::AlbumArtists );
         qm->addMatch( artist );
+    }
     else if( Meta::AlbumPtr album = Meta::AlbumPtr::dynamicCast( m_data ) )
         qm->addMatch( album );
     else if( Meta::ComposerPtr composer = Meta::ComposerPtr::dynamicCast( m_data ) )
