@@ -25,13 +25,13 @@
 #include "core/collections/Collection.h"
 #include "CollectionTreeItem.h"
 #include "core/support/Debug.h"
-#include "Expression.h"
 #include "core/meta/support/MetaConstants.h"
 #include "core/collections/QueryMaker.h"
 #include "amarokconfig.h"
 #include "core/capabilities/EditCapability.h"
 #include "shared/FileType.h"
 #include "SvgHandler.h"
+#include "core-impl/collections/support/TextualQueryFilter.h"
 
 #include <KGlobalSettings>
 #include <KIcon>
@@ -99,7 +99,7 @@ CollectionTreeItemModelBase::setData( const QModelIndex &index, const QVariant &
 
     if( Meta::TrackPtr track = Meta::TrackPtr::dynamicCast( data ) )
     {
-        if( !track->hasCapabilityInterface( Capabilities::Capability::Editable ) )
+        if( !track->has<Capabilities::EditCapability>() )
             return false;
         Capabilities::EditCapability *ec = track->create<Capabilities::EditCapability>();
         if( ec )
@@ -230,7 +230,7 @@ CollectionTreeItemModelBase::dataForItem( CollectionTreeItem *item, int role, in
                 {
                     int trackNum = track->trackNumber();
                     if( trackNum > 0 )
-                        name.prepend( QString("%1. ").arg(trackNum) );
+                        name.prepend( QString("%1 - ").arg(trackNum) );
                 }
 
                 // Check empty after track logic and before album logic
@@ -254,7 +254,8 @@ CollectionTreeItemModelBase::dataForItem( CollectionTreeItem *item, int role, in
         case Qt::ToolTipRole:
             {
                 QString name = album->prettyName();
-                if( AmarokConfig::showYears() )
+                // add years for named albums (if enabled)
+                if( AmarokConfig::showYears() && !album->name().isEmpty() )
                 {
                     Meta::TrackList tracks = album->tracks();
                     if( !tracks.isEmpty() )
@@ -398,14 +399,32 @@ CollectionTreeItemModelBase::mimeTypes() const
 }
 
 QMimeData*
-CollectionTreeItemModelBase::mimeData(const QModelIndexList & indices) const
+CollectionTreeItemModelBase::mimeData( const QModelIndexList &indices ) const
 {
     if ( indices.isEmpty() )
         return 0;
 
-    QList<CollectionTreeItem*> items;
+    // first, filter out duplicate entries that may arise when both parent and child are selected
+    QSet<QModelIndex> indexSet = QSet<QModelIndex>::fromList( indices );
+    QMutableSetIterator<QModelIndex> it( indexSet );
+    while( it.hasNext() )
+    {
+        it.next();
+        // we go up in parent hierarchy searching whether some parent indices are already in set
+        QModelIndex parentIndex = it.value();
+        while( parentIndex.isValid() )  // leave the root (top, invalid) index intact
+        {
+            parentIndex = parentIndex.parent();  // yes, we start from the parent of current index
+            if( indexSet.contains( parentIndex ) )
+            {
+                it.remove(); // parent already in selected set, remove child
+                break; // no need to continue inner loop, already deleted
+            }
+        }
+    }
 
-    foreach( const QModelIndex &index, indices )
+    QList<CollectionTreeItem*> items;
+    foreach( const QModelIndex &index, indexSet )
     {
         if( index.isValid() )
             items << static_cast<CollectionTreeItem*>( index.internalPointer() );
@@ -431,20 +450,9 @@ CollectionTreeItemModelBase::mimeData(const QList<CollectionTreeItem*> & items)
         else
         {
             Collections::QueryMaker *qm = item->queryMaker();
-            CollectionTreeItem *tmpItem = item;
-            while( tmpItem->isDataItem() )
-            {
-                if( tmpItem->data() )
-                {
-                    if( levelCategory( tmpItem->level() - 1 ) == CategoryId::AlbumArtist )
-                        qm->setArtistQueryMode( Collections::QueryMaker::AlbumArtists );
-                    tmpItem->addMatch( qm );
-                }
-                else
-                    qm->setAlbumQueryMode( Collections::QueryMaker::OnlyCompilations );
-                tmpItem = tmpItem->parent();
-            }
-            addFilters( qm );
+            for( CollectionTreeItem *tmp = item; tmp; tmp = tmp->parent() )
+                tmp->addMatch( qm, levelCategory( tmp->level() - 1 ) );
+            Collections::addTextualFilter( qm, m_currentFilter );
             queries.append( qm );
         }
     }
@@ -571,24 +579,11 @@ void CollectionTreeItemModelBase::listForLevel(int level, Collections::QueryMake
             }
         }
 
-        for( CollectionTreeItem *tmpItem = parent; tmpItem->parent(); tmpItem = tmpItem->parent() )
-        {
-            // a various artist item means we are looking for compilations here
-            if( tmpItem->isVariousArtistItem() )
-                qm->setAlbumQueryMode( Collections::QueryMaker::OnlyCompilations );
-            else if( tmpItem->data() )
-            {
-                if( levelCategory( tmpItem->level() - 1 ) == CategoryId::AlbumArtist )
-                    qm->setArtistQueryMode( Collections::QueryMaker::AlbumArtists );
-                 tmpItem->addMatch( qm );
-            }
-        }
-        addFilters( qm );
-        qm->setReturnResultAsDataPtrs( true );
-        connect( qm, SIGNAL( newResultReady( QString, Meta::DataList ) ), SLOT( newResultReady( QString, Meta::DataList ) ), Qt::QueuedConnection );
-        connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ), Qt::QueuedConnection );
+        for( CollectionTreeItem *tmp = parent; tmp; tmp = tmp->parent() )
+            tmp->addMatch( qm, levelCategory( tmp->level() - 1 ) );
+        Collections::addTextualFilter( qm, m_currentFilter );
+        addQueryMaker( parent, qm );
         d->childQueries.insert( qm, parent );
-        d->runningQueries.insert( parent, qm );
         qm->run();
 
         //some very quick queries may be done so fast that the loading
@@ -632,256 +627,26 @@ CollectionTreeItemModelBase::mapCategoryToQueryType( int levelType ) const
     return type;
 }
 
-
-#define ADD_OR_EXCLUDE_FILTER( VALUE, FILTER, MATCHBEGIN, MATCHEND ) \
-            { if( elem.negate ) \
-                qm->excludeFilter( VALUE, FILTER, MATCHBEGIN, MATCHEND ); \
-            else \
-                qm->addFilter( VALUE, FILTER, MATCHBEGIN, MATCHEND ); }
-#define ADD_OR_EXCLUDE_NUMBER_FILTER( VALUE, FILTER, COMPARE ) \
-            { if( elem.negate ) \
-                qm->excludeNumberFilter( VALUE, FILTER, COMPARE ); \
-            else \
-                qm->addNumberFilter( VALUE, FILTER, COMPARE ); }
-
 void
-CollectionTreeItemModelBase::addDateFilter( qint64 field, Collections::QueryMaker::NumberComparison compare, const expression_element &elem, Collections::QueryMaker *qm ) const
+CollectionTreeItemModelBase::addQueryMaker( CollectionTreeItem* item,
+                                            Collections::QueryMaker *qm ) const
 {
-    if( compare == Collections::QueryMaker::Equals )
-    {
-        const uint dateCutOff = semanticDateTimeParser( elem.text ).toTime_t();
-        if( dateCutOff > 0 )
-        {
-            ADD_OR_EXCLUDE_NUMBER_FILTER( field, dateCutOff, Collections::QueryMaker::GreaterThan );
-        }
-    }
-    else
-    {
-        Collections::QueryMaker::NumberComparison compareAlt = Collections::QueryMaker::GreaterThan;
-        if( compare == Collections::QueryMaker::GreaterThan )
-        {
-            qm->endAndOr();
-            qm->beginAnd();
-            compareAlt = Collections::QueryMaker::LessThan;
-            ADD_OR_EXCLUDE_NUMBER_FILTER( field, 0, compare );
-        }
-        const uint time_t = semanticDateTimeParser( elem.text ).toTime_t();
-        ADD_OR_EXCLUDE_NUMBER_FILTER( field, time_t, compareAlt );
-    }
+    connect( qm, SIGNAL( newResultReady( Meta::TrackList ) ), SLOT( newResultReady( Meta::TrackList ) ), Qt::QueuedConnection );
+    connect( qm, SIGNAL( newResultReady( Meta::ArtistList ) ), SLOT( newResultReady( Meta::ArtistList ) ), Qt::QueuedConnection );
+    connect( qm, SIGNAL( newResultReady( Meta::AlbumList ) ), SLOT( newResultReady( Meta::AlbumList ) ), Qt::QueuedConnection );
+    connect( qm, SIGNAL( newResultReady( Meta::GenreList ) ), SLOT( newResultReady( Meta::GenreList ) ), Qt::QueuedConnection );
+    connect( qm, SIGNAL( newResultReady( Meta::ComposerList ) ), SLOT( newResultReady( Meta::ComposerList ) ), Qt::QueuedConnection );
+    connect( qm, SIGNAL( newResultReady( Meta::YearList ) ), SLOT( newResultReady( Meta::YearList ) ), Qt::QueuedConnection );
+    connect( qm, SIGNAL( newResultReady( Meta::LabelList ) ), SLOT( newResultReady( Meta::LabelList ) ), Qt::QueuedConnection );
+    connect( qm, SIGNAL( newResultReady( Meta::DataList ) ), SLOT( newResultReady( Meta::DataList ) ), Qt::QueuedConnection );
+    connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ), Qt::QueuedConnection );
+    d->runningQueries.insert( item, qm );
 }
 
-void
-CollectionTreeItemModelBase::addFilters( Collections::QueryMaker *qm ) const
-{
-    int validFilters = qm->validFilterMask();
-
-    ParsedExpression parsed = ExpressionParser::parse ( m_currentFilter );
-    foreach( const or_list &orList, parsed )
-    {
-        qm->beginOr();
-
-        foreach ( const expression_element &elem, orList )
-        {
-            if( elem.negate )
-                qm->beginAnd();
-            else
-                qm->beginOr();
-
-            if ( elem.field.isEmpty() )
-            {
-                qm->beginOr();
-
-                if( ( validFilters & Collections::QueryMaker::TitleFilter ) )
-                    ADD_OR_EXCLUDE_FILTER( Meta::valTitle, elem.text, false, false );
-                if( ( validFilters & Collections::QueryMaker::UrlFilter ) )
-                    ADD_OR_EXCLUDE_FILTER( Meta::valUrl, elem.text, false, false );
-                if( ( validFilters & Collections::QueryMaker::AlbumFilter ) )
-                    ADD_OR_EXCLUDE_FILTER( Meta::valAlbum, elem.text, false, false );
-                if( ( validFilters & Collections::QueryMaker::ArtistFilter ) )
-                    ADD_OR_EXCLUDE_FILTER( Meta::valArtist, elem.text, false, false );
-                if( ( validFilters & Collections::QueryMaker::AlbumArtistFilter ) )
-                    ADD_OR_EXCLUDE_FILTER( Meta::valAlbumArtist, elem.text, false, false );
-                if( ( validFilters & Collections::QueryMaker::ComposerFilter ) )
-                    ADD_OR_EXCLUDE_FILTER( Meta::valComposer, elem.text, false, false );
-                if( ( validFilters & Collections::QueryMaker::GenreFilter ) )
-                    ADD_OR_EXCLUDE_FILTER( Meta::valGenre, elem.text, false, false );
-                if( ( validFilters & Collections::QueryMaker::YearFilter ) )
-                    ADD_OR_EXCLUDE_FILTER( Meta::valYear, elem.text, false, false );
-
-                ADD_OR_EXCLUDE_FILTER( Meta::valLabel, elem.text, false, false );
-
-                qm->endAndOr();
-            }
-            else
-            {
-                //get field values based on name
-                QString lcField = elem.field.toLower();
-                Collections::QueryMaker::NumberComparison compare = Collections::QueryMaker::Equals;
-                switch( elem.match )
-                {
-                    case expression_element::More:
-                        compare = Collections::QueryMaker::GreaterThan;
-                        break;
-                    case expression_element::Less:
-                        compare = Collections::QueryMaker::LessThan;
-                        break;
-                    case expression_element::Contains:
-                        compare = Collections::QueryMaker::Equals;
-                        break;
-                }
-
-                // TODO: Once we have MetaConstants.cpp use those functions here
-                if ( lcField.compare( "album", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valAlbum ), Qt::CaseInsensitive ) == 0 )
-                {
-                    if ( ( validFilters & Collections::QueryMaker::AlbumFilter ) == 0 ) continue;
-                    ADD_OR_EXCLUDE_FILTER( Meta::valAlbum, elem.text, false, false );
-                }
-                else if ( lcField.compare( "artist", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valArtist ), Qt::CaseInsensitive ) == 0 )
-                {
-                    if ( ( validFilters & Collections::QueryMaker::ArtistFilter ) == 0 ) continue;
-                    ADD_OR_EXCLUDE_FILTER( Meta::valArtist, elem.text, false, false );
-                }
-                else if ( lcField.compare( "albumartist", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valAlbumArtist ), Qt::CaseInsensitive ) == 0 )
-                {
-                    if ( ( validFilters & Collections::QueryMaker::AlbumArtistFilter ) == 0 ) continue;
-                    ADD_OR_EXCLUDE_FILTER( Meta::valAlbumArtist, elem.text, false, false );
-                }
-                else if ( lcField.compare( "genre", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valGenre ), Qt::CaseInsensitive ) == 0)
-                {
-                    if ( ( validFilters & Collections::QueryMaker::GenreFilter ) == 0 ) continue;
-                    ADD_OR_EXCLUDE_FILTER( Meta::valGenre, elem.text, false, false );
-                }
-                else if ( lcField.compare( "title", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valTitle ), Qt::CaseInsensitive ) == 0 )
-                {
-                    if ( ( validFilters & Collections::QueryMaker::TitleFilter ) == 0 ) continue;
-                    ADD_OR_EXCLUDE_FILTER( Meta::valTitle, elem.text, false, false );
-                }
-                else if ( lcField.compare( "composer", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valComposer ), Qt::CaseInsensitive ) == 0 )
-                {
-                    if ( ( validFilters & Collections::QueryMaker::ComposerFilter ) == 0 ) continue;
-                    ADD_OR_EXCLUDE_FILTER( Meta::valComposer, elem.text, false, false );
-                }
-                else if( lcField.compare( "label", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valLabel ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_FILTER( Meta::valLabel, elem.text, false, false );
-                }
-                else if ( lcField.compare( "year", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valYear ), Qt::CaseInsensitive ) == 0)
-                {
-                    if ( ( validFilters & Collections::QueryMaker::YearFilter ) == 0 ) continue;
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valYear, elem.text.toInt(), compare );
-                }
-                else if ( lcField.compare( "bpm", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valBpm ), Qt::CaseInsensitive ) == 0)
-                {
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valBpm, elem.text.toInt(), compare );
-                }
-                else if( lcField.compare( "comment", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valComment ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_FILTER( Meta::valComment, elem.text, false, false );
-                }
-                else if( lcField.compare( "filename", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valUrl ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_FILTER( Meta::valUrl, elem.text, false, false );
-                }
-                else if( lcField.compare( "bitrate", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valBitrate ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valBitrate, elem.text.toInt(), compare );
-                }
-                else if( lcField.compare( "rating", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valRating ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valRating, elem.text.toFloat() * 2, compare );
-                }
-                else if( lcField.compare( "score", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valScore ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valScore, elem.text.toInt(), compare );
-                }
-                else if( lcField.compare( "playcount", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valPlaycount ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valPlaycount, elem.text.toInt(), compare );
-                }
-                else if( lcField.compare( "samplerate", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valSamplerate ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valSamplerate, elem.text.toInt(), compare );
-                }
-                else if( lcField.compare( "length", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valLength ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valLength, elem.text.toInt() * 1000, compare );
-                }
-                else if( lcField.compare( "filesize", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valFilesize ), Qt::CaseInsensitive ) == 0 )
-                {
-                    bool doubleOk( false );
-                    const double mbytes = elem.text.toDouble( &doubleOk ); // input in MBs
-                    if( !doubleOk )
-                    {
-                        qm->endAndOr();
-                        return;
-                    }
-
-                    /*
-                     * A special case is made for Equals (e.g. filesize:100), which actually filters
-                     * for anything beween 100 and 101MBs. Megabytes are used because for audio files
-                     * they are the most reasonable units for the user to deal with.
-                     */
-                    const qreal bytes = mbytes * 1024.0 * 1024.0;
-                    const qint64 mbFloor = qint64( qAbs(mbytes) );
-                    switch( compare )
-                    {
-                    case Collections::QueryMaker::Equals:
-                        qm->endAndOr();
-                        qm->beginAnd();
-                        ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valFilesize, mbFloor * 1024 * 1024, Collections::QueryMaker::GreaterThan );
-                        ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valFilesize, (mbFloor + 1) * 1024 * 1024, Collections::QueryMaker::LessThan );
-                        break;
-                    case Collections::QueryMaker::GreaterThan:
-                    case Collections::QueryMaker::LessThan:
-                        ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valFilesize, bytes, compare );
-                        break;
-                    }
-                }
-                else if( lcField.compare( "format", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valFormat ), Qt::CaseInsensitive ) == 0 )
-                {
-                    // NOTE: possible keywords that could be considered: codec, filetype, etc.
-                    const QString &ftStr = elem.text;
-                    Amarok::FileType ft = Amarok::FileTypeSupport::fileType(ftStr);
-
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valFormat, int(ft), compare );
-                }
-                else if( lcField.compare( "discnumber", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valDiscNr ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valDiscNr, elem.text.toInt(), compare );
-                }
-                else if( lcField.compare( "tracknumber", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valTrackNr ), Qt::CaseInsensitive ) == 0 )
-                {
-                    ADD_OR_EXCLUDE_NUMBER_FILTER( Meta::valTrackNr, elem.text.toInt(), compare );
-                }
-                else if( lcField.compare( "played", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valLastPlayed ), Qt::CaseInsensitive ) == 0 )
-                {
-                    addDateFilter( Meta::valLastPlayed, compare, elem, qm );
-                }
-                else if( lcField.compare( "first", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valFirstPlayed ), Qt::CaseInsensitive ) == 0 )
-                {
-                    addDateFilter( Meta::valFirstPlayed, compare, elem, qm );
-                }
-                else if( lcField.compare( "added", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valCreateDate ), Qt::CaseInsensitive ) == 0 )
-                {
-                    addDateFilter( Meta::valCreateDate, compare, elem, qm );
-                }
-                else if( lcField.compare( "modified", Qt::CaseInsensitive ) == 0 || lcField.compare( shortI18nForField( Meta::valModified ), Qt::CaseInsensitive ) == 0 )
-                {
-                    addDateFilter( Meta::valModified, compare, elem, qm );
-                }
-            }
-            qm->endAndOr();
-#undef ADD_OR_EXCLUDE_FILTER
-#undef ADD_OR_EXCLUDE_NUMBER_FILTER
-        }
-        qm->endAndOr();
-    }
-}
 
 void
 CollectionTreeItemModelBase::queryDone()
 {
-    DEBUG_BLOCK
     Collections::QueryMaker *qm = qobject_cast<Collections::QueryMaker*>( sender() );
     if( !qm )
         return;
@@ -895,7 +660,7 @@ CollectionTreeItemModelBase::queryDone()
         item = d->noLabelsQueries.take( qm );
 
     if( item )
-        d->runningQueries.remove( item );
+        d->runningQueries.remove( item, qm );
 
     //reset icon for this item
     if( item && item != m_rootItem )
@@ -912,11 +677,65 @@ CollectionTreeItemModelBase::queryDone()
     qm->deleteLater();
 }
 
-void
-CollectionTreeItemModelBase::newResultReady(const QString & collectionId, Meta::DataList data)
-{
-    Q_UNUSED( collectionId )
+// TODO
 
+/** Small helper function to convert a list of e.g. tracks to a list of DataPtr */
+template<class PointerType, class ListType>
+static Meta::DataList
+convertToDataList( const ListType& list )
+{
+    Meta::DataList data;
+    foreach( PointerType p, list )
+        data << Meta::DataPtr::staticCast( p );
+
+    return data;
+}
+
+void
+CollectionTreeItemModelBase::newResultReady( Meta::TrackList res )
+{
+    newResultReady( convertToDataList<Meta::TrackPtr, Meta::TrackList>( res ) );
+}
+
+void
+CollectionTreeItemModelBase::newResultReady( Meta::ArtistList res )
+{
+    newResultReady( convertToDataList<Meta::ArtistPtr, Meta::ArtistList>( res ) );
+}
+
+void
+CollectionTreeItemModelBase::newResultReady( Meta::AlbumList res )
+{
+    newResultReady( convertToDataList<Meta::AlbumPtr, Meta::AlbumList>( res ) );
+}
+
+void
+CollectionTreeItemModelBase::newResultReady( Meta::GenreList res )
+{
+    newResultReady( convertToDataList<Meta::GenrePtr, Meta::GenreList>( res ) );
+}
+
+void
+CollectionTreeItemModelBase::newResultReady( Meta::ComposerList res )
+{
+    newResultReady( convertToDataList<Meta::ComposerPtr, Meta::ComposerList>( res ) );
+}
+
+void
+CollectionTreeItemModelBase::newResultReady( Meta::YearList res )
+{
+    newResultReady( convertToDataList<Meta::YearPtr, Meta::YearList>( res ) );
+}
+
+void
+CollectionTreeItemModelBase::newResultReady( Meta::LabelList res )
+{
+    newResultReady( convertToDataList<Meta::LabelPtr, Meta::LabelList>( res ) );
+}
+
+void
+CollectionTreeItemModelBase::newResultReady( Meta::DataList data )
+{
     //if we are expanding an item, we'll find the sender in childQueries
     //otherwise we are filtering all collections
     Collections::QueryMaker *qm = qobject_cast<Collections::QueryMaker*>( sender() );
@@ -1171,23 +990,18 @@ CollectionTreeItemModelBase::nameForLevel(int level) const
 void
 CollectionTreeItemModelBase::handleCompilations( CollectionTreeItem *parent ) const
 {
-    DEBUG_BLOCK
-
     //this method will be called when we retrieve a list of artists from the database.
     //we have to query for all compilations, and then add a "Various Artists" node if at least
     //one compilation exists
     Collections::QueryMaker *qm = parent->queryMaker();
     qm->setAlbumQueryMode( Collections::QueryMaker::OnlyCompilations );
     qm->setQueryType( Collections::QueryMaker::Album );
-    for( CollectionTreeItem *tmpItem = parent; tmpItem->parent(); tmpItem = tmpItem->parent() )
-        tmpItem->addMatch( qm );
+    for( CollectionTreeItem *tmp = parent; tmp; tmp = tmp->parent() )
+        tmp->addMatch( qm, levelCategory( tmp->level() - 1 ) );
 
-    addFilters( qm );
-    qm->setReturnResultAsDataPtrs( true );
-    connect( qm, SIGNAL( newResultReady( QString, Meta::DataList ) ), SLOT( newResultReady( QString, Meta::DataList ) ), Qt::QueuedConnection );
-    connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ), Qt::QueuedConnection );
+    Collections::addTextualFilter( qm, m_currentFilter );
+    addQueryMaker( parent, qm );
     d->compilationQueries.insert( qm, parent );
-    d->runningQueries.insert( parent, qm );
     qm->run();
 }
 
@@ -1198,92 +1012,15 @@ CollectionTreeItemModelBase::handleTracksWithoutLabels( Collections::QueryMaker:
     Collections::QueryMaker *qm = parent->queryMaker();
     qm->setQueryType( queryType );
     qm->setLabelQueryMode( Collections::QueryMaker::OnlyWithoutLabels );
-    qm->setReturnResultAsDataPtrs( true );
-    for( CollectionTreeItem *tmpItem = parent; tmpItem->parent(); tmpItem = tmpItem->parent() )
-        tmpItem->addMatch( qm );
+    for( CollectionTreeItem *tmp = parent; tmp; tmp = tmp->parent() )
+        tmp->addMatch( qm, levelCategory( tmp->level() - 1 ) );
 
-    addFilters( qm );
-    connect( qm, SIGNAL( newResultReady( QString, Meta::DataList ) ), SLOT( newResultReady( QString, Meta::DataList ) ), Qt::QueuedConnection );
-    connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ), Qt::QueuedConnection );
+    Collections::addTextualFilter( qm, m_currentFilter );
+    addQueryMaker( parent, qm );
     d->noLabelsQueries.insert( qm, parent );
-    d->runningQueries.insert( parent, qm );
     qm->run();
 }
 
-QDateTime
-CollectionTreeItemModelBase::semanticDateTimeParser( const QString &text ) const
-{
-    /* TODO: semanticDateTimeParser: has potential to extend and form a class of its own */
-
-    const QString lowerText = text.toLower();
-    const QDateTime curTime = QDateTime::currentDateTime();
-    QDateTime result;
-
-    if( text.at(0).isLetter() )
-    {
-        if( ( lowerText.compare( "today" ) == 0 ) || ( lowerText.compare( i18n( "today" ) ) == 0 ) )
-            result = curTime.addDays( -1 );
-        else if( ( lowerText.compare( "last week" ) == 0 ) || ( lowerText.compare( i18n( "last week" ) ) == 0 ) )
-            result = curTime.addDays( -7 );
-        else if( ( lowerText.compare( "last month" ) == 0 ) || ( lowerText.compare( i18n( "last month" ) ) == 0 ) )
-            result = curTime.addMonths( -1 );
-        else if( ( lowerText.compare( "two months ago" ) == 0 ) || ( lowerText.compare( i18n( "two months ago" ) ) == 0 ) )
-            result = curTime.addMonths( -2 );
-        else if( ( lowerText.compare( "three months ago" ) == 0 ) || ( lowerText.compare( i18n( "three months ago" ) ) == 0 ) )
-            result = curTime.addMonths( -3 );
-    }
-    else // first character is a number
-    {
-        // parse a "#m#d" (discoverability == 0, but without a GUI, how to do it?)
-        int years = 0, months = 0, weeks = 0, days = 0, secs = 0;
-        QString tmp;
-        for( int i = 0; i < text.length(); i++ )
-        {
-            QChar c = text.at( i );
-            if( c.isNumber() )
-            {
-                tmp += c;
-            }
-            else if( c == 'y' )
-            {
-                years = -tmp.toInt();
-                tmp.clear();
-            }
-            else if( c == 'm' )
-            {
-                months = -tmp.toInt();
-                tmp.clear();
-            }
-            else if( c == 'w' )
-            {
-                weeks = -tmp.toInt() * 7;
-                tmp.clear();
-            }
-            else if( c == 'd' )
-            {
-                days = -tmp.toInt();
-                break;
-            }
-            else if( c == 'h' )
-            {
-                secs = -tmp.toInt() * 60 * 60;
-                break;
-            }
-            else if( c == 'M' )
-            {
-                secs = -tmp.toInt() * 60;
-                break;
-            }
-            else if( c == 's' )
-            {
-                secs = -tmp.toInt();
-                break;
-            }
-        }
-        result = QDateTime::currentDateTime().addYears( years ).addMonths( months ).addDays( weeks ).addDays( days ).addSecs( secs );
-    }
-    return result;
-}
 
 void CollectionTreeItemModelBase::startAnimationTick()
 {
@@ -1303,7 +1040,7 @@ void CollectionTreeItemModelBase::loadingAnimationTick()
 
     //trigger an update of all items being populated at the moment;
 
-    QList< CollectionTreeItem * > items = d->runningQueries.keys();
+    QList< CollectionTreeItem * > items = d->runningQueries.uniqueKeys();
     foreach ( CollectionTreeItem* item, items  )
     {
         if( item == m_rootItem )
@@ -1316,6 +1053,7 @@ void
 CollectionTreeItemModelBase::setCurrentFilter( const QString &filter )
 {
     m_currentFilter = filter;
+    slotFilter();
 }
 
 void
@@ -1407,28 +1145,35 @@ void CollectionTreeItemModelBase::markSubTreeAsDirty( CollectionTreeItem *item )
 
 void CollectionTreeItemModelBase::itemAboutToBeDeleted( CollectionTreeItem *item )
 {
+    // also all the children will be deleted
+    foreach( CollectionTreeItem *child, item->children() )
+        itemAboutToBeDeleted( child );
+
     if( !d->runningQueries.contains( item ) )
         return;
     //replace this hack with QWeakPointer as soon as we depend on Qt 4.6
-    Collections::QueryMaker *qm = d->runningQueries.take( item );
-    if( qm )
+    foreach(Collections::QueryMaker *qm, d->runningQueries.values( item ))
     {
-
         d->childQueries.remove( qm );
         d->compilationQueries.remove( qm );
         d->noLabelsQueries.remove( qm );
-        //we still need to disconnect the qm below
-    }
-    if( qm )
-    {
+        d->runningQueries.remove(item, qm);
+
         //Disconnect all signals from the QueryMaker so we do not get notified about the results
         qm->disconnect();
+        qm->abortQuery();
         //Nuke it
         qm->deleteLater();
     }
 }
 
-int
+void
+CollectionTreeItemModelBase::setDragSourceCollections( const QSet<Collections::Collection*> &collections )
+{
+    m_dragSourceCollections = collections;
+}
+
+CategoryId::CatMenuId
 CollectionTreeItemModelBase::levelCategory( const int level ) const
 {
     if( level >= 0 && level + levelModifier() < m_levelType.count() )
