@@ -22,13 +22,16 @@
 #include "FileBrowser_p.h"
 #include "FileBrowser_p.moc"
 
+#include "amarokconfig.h"
+#include "core/support/Components.h"
 #include "core/support/Debug.h"
+#include "core-impl/meta/file/File.h"
 #include "BrowserBreadcrumbItem.h"
 #include "BrowserCategoryList.h"
 #include "EngineController.h"
 #include "FileView.h"
-#include "MimeTypeFilterProxyModel.h"
-#include "playlist/PlaylistModelStack.h"
+#include "DirPlaylistTrackFilterProxyModel.h"
+#include "playlist/PlaylistController.h"
 #include "widgets/SearchWidget.h"
 
 #include <KAction>
@@ -61,7 +64,9 @@ FileBrowser::Private::Private( FileBrowser *parent )
 
     upAction = KStandardAction::up( q, SLOT(up()), topHBox );
     homeAction = KStandardAction::home( q, SLOT(home()), topHBox );
-    placesAction = new KAction( KIcon( "folder-remote" ), i18nc( "Show Dolphin Places the user configured", "Places" ), topHBox );
+    placesAction = new KAction( KIcon( "folder-remote" ),
+                               i18nc( "Show Dolphin Places the user configured", "Places" ),
+                               topHBox );
 
     navigationToolbar->addAction( backAction );
     navigationToolbar->addAction( forwardAction );
@@ -69,10 +74,9 @@ FileBrowser::Private::Private( FileBrowser *parent )
     navigationToolbar->addAction( homeAction );
     navigationToolbar->addAction( placesAction );
 
-    SearchWidget *searchWidget = new SearchWidget( topHBox, q, false );
+    searchWidget = new SearchWidget( topHBox, false );
     searchWidget->setClickMessage( i18n( "Filter Files" ) );
 
-    filterTimer.setSingleShot( true );
     fileView = new FileView( q );
 }
 
@@ -111,7 +115,6 @@ QStringList
 FileBrowser::Private::siblingsForDir( const QString &path )
 {
     // includes the dir itself
-    // debug() << "path: " << path;
     QStringList siblings;
     QDir dir( path );
     if( !dir.isRoot() )
@@ -166,12 +169,21 @@ FileBrowser::Private::slotSaveHeaderState()
     }
 }
 
-FileBrowser::FileBrowser( const char * name, QWidget *parent )
+FileBrowser::FileBrowser( const char *name, QWidget *parent )
     : BrowserCategory( name, parent )
     , d( new FileBrowser::Private( this ) )
 {
-    setLongDescription( i18n( "The file browser lets you browse files anywhere on your system, regardless of whether these files are part of your local collection. You can then add these files to the playlist as well as perform basic file operations." ) );
+    setLongDescription( i18n( "The file browser lets you browse files anywhere on your system, " \
+                        "regardless of whether these files are part of your local collection. " \
+                        "You can then add these files to the playlist as well as perform basic "\
+                        "file operations." )
+                       );
     setImagePath( KStandardDirs::locate( "data", "amarok/images/hover_info_files.png" ) );
+
+    // set background
+    if( AmarokConfig::showBrowserBackgroundImage() )
+        setBackgroundImage( imagePath() );
+
     QTimer::singleShot( 0, this, SLOT(initView()) );
 }
 
@@ -180,11 +192,15 @@ FileBrowser::initView()
 {
     d->kdirModel = new DirBrowserModel( this );
 
-    d->mimeFilterProxyModel = new MimeTypeFilterProxyModel( EngineController::supportedMimeTypes(), this );
+    EngineController *engineController = Amarok::Components::engineController();
+    Q_ASSERT( engineController );
+    d->mimeFilterProxyModel = new DirPlaylistTrackFilterProxyModel( this );
     d->mimeFilterProxyModel->setSourceModel( d->kdirModel );
     d->mimeFilterProxyModel->setSortCaseSensitivity( Qt::CaseInsensitive );
     d->mimeFilterProxyModel->setFilterCaseSensitivity( Qt::CaseInsensitive );
     d->mimeFilterProxyModel->setDynamicSortFilter( true );
+    connect( d->searchWidget, SIGNAL( filterChanged( const QString & ) ),
+             d->mimeFilterProxyModel, SLOT( setFilterFixedString( const QString & ) ) );
 
     d->fileView->setModel( d->mimeFilterProxyModel );
     d->fileView->header()->setContextMenuPolicy( Qt::ActionsContextMenu );
@@ -199,7 +215,10 @@ FileBrowser::initView()
 
     for( int i = 0, columns = d->fileView->model()->columnCount(); i < columns ; ++i )
     {
-        QAction *action = new QAction( d->fileView->model()->headerData( i, Qt::Horizontal ).toString(), d->fileView->header() );
+        QAction *action =
+                new QAction( d->fileView->model()->headerData( i, Qt::Horizontal ).toString(),
+                             d->fileView->header()
+                           );
         d->fileView->header()->addAction( action );
         d->columnActions.append( action );
         action->setCheckable( true );
@@ -209,12 +228,16 @@ FileBrowser::initView()
     }
 
     connect( d->fileView->header(), SIGNAL(geometriesChanged()), this, SLOT(slotSaveHeaderState()) );
-    connect( d->fileView, SIGNAL( activated( const QModelIndex & ) ), this, SLOT( itemActivated( const QModelIndex & ) ) );
+    connect( d->fileView, SIGNAL(activated( const QModelIndex & )),
+                          SLOT(itemActivated( const QModelIndex & )) );
     if( !KGlobalSettings::singleClick() )
-        connect( d->fileView, SIGNAL( doubleClicked( const QModelIndex & ) ), this, SLOT( itemActivated( const QModelIndex & ) ) );
+    {
+        connect( d->fileView, SIGNAL(doubleClicked( const QModelIndex & )),
+                              SLOT(itemActivated( const QModelIndex & ))
+               );
+    }
 
     connect( d->placesAction, SIGNAL( triggered( bool) ), this, SLOT( showPlaces() ) );
-    connect( &d->filterTimer, SIGNAL( timeout() ), this, SLOT( slotFilterNow() ) );
 }
 
 FileBrowser::~FileBrowser()
@@ -238,12 +261,11 @@ FileBrowser::toggleColumn( bool toggled )
 void
 FileBrowser::polish()
 {
-    DEBUG_BLOCK
     setupAddItems();
 }
 
 QString
-FileBrowser::currentDir()
+FileBrowser::currentDir() const
 {
     if( d->showingPlaces )
         return "places:";
@@ -256,11 +278,8 @@ FileBrowser::currentDir()
 void
 FileBrowser::itemActivated( const QModelIndex &index )
 {
-    DEBUG_BLOCK
-
     if( d->showingPlaces )
     {
-        debug() << "place activated!";
         QString placesUrl = index.data( KFilePlacesModel::UrlRole  ).value<QString>();
 
         if( !placesUrl.isEmpty() )
@@ -303,18 +322,14 @@ FileBrowser::itemActivated( const QModelIndex &index )
         KFileItem file = index.data( KDirModel::FileItemRole ).value<KFileItem>();
         KUrl filePath = file.url();
 
-        debug() << "activated url: " << filePath.url();
-        debug() << "filename: " << filePath.fileName();
-
         if( file.isDir() )
         {
-            debug() << "setting root path to: " << filePath.path();
             d->backStack.push( d->currentPath );
             setDir( filePath );
         }
         else
         {
-            if( EngineController::canDecode( filePath ) )
+            if( MetaFile::Track::isTrack( filePath ) )
             {
                 QList<KUrl> urls;
                 urls << filePath;
@@ -324,35 +339,13 @@ FileBrowser::itemActivated( const QModelIndex &index )
     }
 }
 
-void
-FileBrowser::slotSetFilterTimeout()
-{
-    KComboBox *comboBox = qobject_cast<KComboBox*>( sender() );
-    if( comboBox )
-    {
-        d->currentFilter = comboBox->currentText();
-        d->filterTimer.stop();
-        d->filterTimer.start( 500 );
-    }
-}
-
-void
-FileBrowser::slotFilterNow()
-{
-    d->mimeFilterProxyModel->setFilterFixedString( d->currentFilter );
-
-    QStringList filters;
-    filters << d->currentFilter;
-}
 
 void
 FileBrowser::addItemActivated( const QString &callbackString )
 {
-    DEBUG_BLOCK
     if( callbackString.isEmpty() )
         return;
 
-    debug() << "callback: " << callbackString;
     d->kdirModel->dirLister()->openUrl( KUrl( callbackString ) );
     d->backStack.push( d->currentPath );
     d->currentPath = KUrl(callbackString);
@@ -364,15 +357,11 @@ FileBrowser::addItemActivated( const QString &callbackString )
 void
 FileBrowser::setupAddItems()
 {
-    DEBUG_BLOCK
     clearAdditionalItems();
 
-    debug() << "current path" << d->currentPath;
-
     if( d->currentPath.isLocalFile() )
     {
         const QString localPath = d->currentPath.toLocalFile();
-        debug() << "local path" << localPath;
         QStringList parts;
         QString partialPath;
 
@@ -384,15 +373,15 @@ FileBrowser::setupAddItems()
             int idx = localPath.indexOf( QDir::homePath() ) + QDir::homePath().size();
             // everything after the homedir e.g., Music/Prince
             QString everything_else = localPath.mid( idx );
-            debug() << "everything else" << everything_else;
             // replace parts with everything else
             parts = everything_else.split( QDir::separator() ) ;
-            debug() << "parts" << parts;
             partialPath = QDir::homePath();
 
             // Add the [Home]
             QStringList siblings = d->siblingsForDir( QDir::homePath() );
-            addAdditionalItem( new BrowserBreadcrumbItem( i18n( "Home" ), siblings, QDir::homePath(), this ) );
+            addAdditionalItem( new BrowserBreadcrumbItem( i18n( "Home" ), siblings,
+                                                         QDir::homePath(), this )
+                             );
         }
         else
         {
@@ -415,8 +404,13 @@ FileBrowser::setupAddItems()
         const QString proto = d->currentPath.protocol();
         const QString authority = d->currentPath.authority();
         const QString protoAuthority = QString( "%1://%2" ).arg( proto, authority );
-        addAdditionalItem( new BrowserBreadcrumbItem( proto + QLatin1Char(':'), QStringList(), proto + QLatin1String("://"), this ) );
-        addAdditionalItem( new BrowserBreadcrumbItem( authority, QStringList(), protoAuthority, this ) );
+        addAdditionalItem( new BrowserBreadcrumbItem( proto + QLatin1Char(':'),
+                                                      QStringList(), proto + QLatin1String("://"),
+                                                      this )
+                         );
+        addAdditionalItem( new BrowserBreadcrumbItem( authority, QStringList(), protoAuthority,
+                                                     this )
+                         );
         QStringList parts = d->currentPath.path().split( QLatin1Char('/'), QString::SkipEmptyParts );
         QString partialPath = protoAuthority;
         foreach( const QString &part, parts )
@@ -430,8 +424,6 @@ FileBrowser::setupAddItems()
 void
 FileBrowser::reActivate()
 {
-    DEBUG_BLOCK
-
     //go to root:
     d->kdirModel->dirLister()->openUrl( KUrl( QDir::rootPath() ) );
     d->currentPath = KUrl( QDir::rootPath() );
@@ -498,9 +490,6 @@ FileBrowser::forward()
 void
 FileBrowser::up()
 {
-    DEBUG_BLOCK
-    debug() << "current dir: " << d->currentPath;
-
     if( d->showingPlaces )
     {
         //apparently, the root level of "places" counts as a valid dir. If we are here, make the
@@ -527,13 +516,16 @@ FileBrowser::showPlaces()
     {
         d->placesModel = new FilePlacesModel( this );
         d->placesModel->setObjectName( "PLACESMODEL");
-        connect( d->placesModel, SIGNAL( setupDone( const QModelIndex &, bool ) ), this, SLOT( setupDone( const QModelIndex &, bool ) ) );
+        connect( d->placesModel, SIGNAL(setupDone( const QModelIndex &, bool )),
+                                 SLOT(setupDone( const QModelIndex &, bool )) );
     }
 
     clearAdditionalItems();
 
     QStringList siblings;
-    addAdditionalItem( new BrowserBreadcrumbItem( i18n( "Places" ), siblings, QDir::homePath(), this ) );
+    addAdditionalItem( new BrowserBreadcrumbItem( i18n( "Places" ), siblings, QDir::homePath(),
+                                                  this )
+                     );
     d->showingPlaces = true;
     d->fileView->setModel( d->placesModel );
     d->fileView->setSelectionMode( QAbstractItemView::SingleSelection );
@@ -544,7 +536,6 @@ FileBrowser::showPlaces()
 void
 FileBrowser::setupDone( const QModelIndex & index, bool success )
 {
-    DEBUG_BLOCK
     if( success )
     {
         QString placesUrl = index.data( KFilePlacesModel::UrlRole  ).value<QString>();
