@@ -14,10 +14,12 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "ConfigDialog"
+
 #include "ConfigDialog.h"
 
-#include "core/support/Amarok.h"
 #include "amarokconfig.h"
+#include "core/support/Amarok.h"
 #include "core/support/Debug.h"
 
 #include "CollectionConfig.h"
@@ -26,10 +28,10 @@
 //#include "MediadeviceConfig.h"
 #include "NotificationsConfig.h"
 #include "PlaybackConfig.h"
-#include "ServiceConfig.h"
-
-#include <KLocale>
+#include "PluginsConfig.h"
+#include "ScriptsConfig.h"
 
+#include <KLocalizedString>
 
 QString Amarok2ConfigDialog::s_currentPage = "GeneralConfig";
 
@@ -40,31 +42,35 @@ QString Amarok2ConfigDialog::s_currentPage = "GeneralConfig";
 Amarok2ConfigDialog::Amarok2ConfigDialog( QWidget *parent, const char* name, KConfigSkeleton *config )
    : KConfigDialog( parent, name, config )
 {
+    DEBUG_BLOCK
     setAttribute( Qt::WA_DeleteOnClose );
 
     ConfigDialogBase* general     = new GeneralConfig( this );
     ConfigDialogBase* collection  = new CollectionConfig( this );
-    ConfigDialogBase* services    = new ServiceConfig( this );
+    ConfigDialogBase* plugins     = new PluginsConfig( this );
     ConfigDialogBase* playback    = new PlaybackConfig( this );
     ConfigDialogBase* notify      = new NotificationsConfig( this );
     ConfigDialogBase* database    = new DatabaseConfig( this );
+    ConfigDialogBase* scripts     = new ScriptsConfig( this );
 
     //ConfigDialogBase* mediadevice = new MediadeviceConfig( this );
 
     addPage( general,     i18nc( "Miscellaneous settings", "General" ), "preferences-other-amarok", i18n( "Configure General Options" ) );
-    addPage( collection,  i18n( "Collection" ), "collection-amarok", i18n( "Configure Collection" ) );
-    addPage( services,    i18n( "Internet Services" ), "services-amarok", i18n( "Configure Services" ) );
+    addPage( collection,  i18n( "Local Collection" ), "collection-amarok", i18n( "Configure Local Collection" ) );
     addPage( playback,    i18n( "Playback" ), "preferences-media-playback-amarok", i18n( "Configure Playback" ) );
-    addPage( notify,         i18n( "Notifications" ), "preferences-indicator-amarok", i18n( "Configure Notifications" ) );
+    addPage( notify,      i18n( "Notifications" ), "preferences-indicator-amarok", i18n( "Configure Notifications" ) );
     addPage( database,    i18n( "Database" ), "server-database", i18n( "Configure Database" ) );
+    addPage( plugins,     i18n( "Plugins" ), "preferences-plugin", i18n( "Configure Plugins" ) );
+    addPage( scripts,     i18n( "Scripts" ), "preferences-plugin-script", i18n( "Configure Scripts" ) );
     //addPage( mediadevice, i18n( "Media Devices" ), "preferences-multimedia-player-amarok", i18n( "Configure Portable Player Support" ) );
 
     setButtons( Help | Ok | Apply | Cancel );
+    restoreDialogSize( Amarok::config( "ConfigDialog" ) );
 }
 
 Amarok2ConfigDialog::~Amarok2ConfigDialog()
 {
-    DEBUG_FUNC_INFO
+    DEBUG_BLOCK
 
     KPageWidgetItem* pageItem = currentPage();
 
@@ -77,6 +83,8 @@ Amarok2ConfigDialog::~Amarok2ConfigDialog()
         }
     }
 
+    KConfigGroup config = Amarok::config( "ConfigDialog" );
+    saveDialogSize( config );
     AmarokConfig::self()->writeConfig();
 }
 
