@@ -19,6 +19,8 @@
 
 #include "amarok_export.h"
 
+#include <KUrl>
+
 #include <QStringList>
 #include <QList>
 #include <QString>
@@ -26,6 +28,22 @@
 
 class LyricsSubject;
 
+struct LyricsData
+{
+    QString text;
+    QString title;
+    QString artist;
+    KUrl site;
+
+    void clear()
+    {
+        text.clear();
+        title.clear();
+        artist.clear();
+        site.clear();
+    }
+};
+
 class AMAROK_EXPORT LyricsObserver
 {
     public:
@@ -34,13 +52,9 @@ class AMAROK_EXPORT LyricsObserver
         virtual ~LyricsObserver();
 
         /**
-         *  A lyrics script has returned new plaintext lyrics.
+         *  A lyrics script has returned new lyrics.
          */
-        virtual void newLyrics( QStringList& lyrics ) { Q_UNUSED( lyrics ); }
-        /**
-         *  A lyrics script has returned new html lyrics.
-         */
-        virtual void newLyricsHtml( QString& lyrics ) { Q_UNUSED( lyrics ); }
+        virtual void newLyrics( const LyricsData &lyrics ) { Q_UNUSED( lyrics ); }
         /**
          *  A lyrics script has returned a list of suggested URLs for correct lyrics.
          */
@@ -48,7 +62,7 @@ class AMAROK_EXPORT LyricsObserver
         /**
          *  A lyrics script has returned some generic message that they want to be displayed.
          */
-        virtual void lyricsMessage( QString& key, QString &val ) { Q_UNUSED( key ); Q_UNUSED( val ); }
+        virtual void lyricsMessage( const QString& key, const QString &val ) { Q_UNUSED( key ); Q_UNUSED( val ); }
 
     private:
         LyricsSubject *m_subject;
@@ -64,10 +78,9 @@ class LyricsSubject
         LyricsSubject() {}
         virtual ~LyricsSubject() {}
     
-        void sendNewLyrics( QStringList lyrics );
-        void sendNewLyricsHtml( QString lyrics );
+        void sendNewLyrics( const LyricsData &lyrics );
         void sendNewSuggestions( const QVariantList &suggestions );
-        void sendLyricsMessage( QString key, QString val );
+        void sendLyricsMessage( const QString &key, const QString &val );
     
     private:
         QList<LyricsObserver*> m_observers;
@@ -76,8 +89,6 @@ class LyricsSubject
 class AMAROK_EXPORT LyricsManager : public LyricsSubject
 {
     public:
-        LyricsManager() : LyricsSubject() { s_self = this; }
-    
         static LyricsManager* self() 
         { 
             if( !s_self )
@@ -99,15 +110,6 @@ class AMAROK_EXPORT LyricsManager : public LyricsSubject
           */
         void setLyricsForTrack( const QString &trackUrl, const QString &lyrics ) const;
 
-        /**
-         * Tests if the given lyrics are Html or plain text.
-         *
-         * @param lyrics The lyrics which will be tested.
-         *
-         * @return true if the given lyrics are Html, otherwise false.
-         */
-        bool isHtmlLyrics( const QString &lyrics ) const;
-
         /**
          * Tests if the given lyrics are empty.
          *
@@ -118,9 +120,13 @@ class AMAROK_EXPORT LyricsManager : public LyricsSubject
         bool isEmpty( const QString &lyrics ) const;
 
     private:
+        LyricsManager() : LyricsSubject() { s_self = this; }
+
         bool showCached();
         
         static LyricsManager* s_self;
 };
 
+Q_DECLARE_METATYPE( LyricsData );
+
 #endif
