@@ -27,7 +27,7 @@
 #include "core/capabilities/ActionsCapability.h"
 #include "core/meta/support/MetaUtility.h"
 #include "core/support/Debug.h"
-#include "playlist/PlaylistModelStack.h"
+#include "playlist/PlaylistController.h"
 #include "widgets/PrettyTreeView.h"
 
 #include <KAction>
@@ -122,6 +122,7 @@ AlbumsView::AlbumsView( QGraphicsWidget *parent )
     m_proxyModel->setSortLocaleAware( true );
     m_proxyModel->setDynamicSortFilter( true );
     m_proxyModel->setSourceModel( m_model );
+    m_proxyModel->setFilterRole( NameRole );
     m_treeView->setModel( m_proxyModel );
 
     QScrollBar *treeScrollBar = m_treeView->verticalScrollBar();
@@ -169,6 +170,18 @@ AlbumsView::scrollTo( QStandardItem *album )
     m_treeView->scrollTo( proxyIndex, QAbstractItemView::EnsureVisible );
 }
 
+QString
+AlbumsView::filterPattern() const
+{
+    return m_proxyModel->filterRegExp().pattern();
+}
+
+void
+AlbumsView::setFilterPattern( const QString &pattern )
+{
+    m_proxyModel->setFilterRegExp( QRegExp(pattern, Qt::CaseInsensitive) );
+}
+
 void
 AlbumsView::clear()
 {
@@ -467,7 +480,7 @@ AlbumsItemDelegate::drawAlbumText( QPainter *p, const QStyleOptionViewItemV4 &vo
     p->setClipRect( textRect );
     applyCommonStyle( p, vopt );
 
-    QString name = index.data( AlbumNameRole ).toString();
+    QString name = index.data( NameRole ).toString();
     int year     = index.data( AlbumYearRole ).toInt();
 
     QStringList texts;
@@ -495,7 +508,7 @@ AlbumsItemDelegate::drawTrackText( QPainter *p, const QStyleOptionViewItemV4 &vo
 
     int trackDigitCount = index.data( AlbumMaxTrackNumberRole ).toString().length();
     bool isCompilation = index.data( AlbumCompilationRole ).toBool();
-    const QString &name = index.data( TrackNameRole ).toString();
+    const QString &name = index.data( NameRole ).toString();
     const QString &artist = index.data( TrackArtistRole ).toString();
     QString length = " (" + Meta::msToPrettyTime( index.data( TrackLengthRole ).toInt() ) + ')';
     QString number = index.data( TrackNumberRole ).toString() + ". ";
@@ -508,7 +521,9 @@ AlbumsItemDelegate::drawTrackText( QPainter *p, const QStyleOptionViewItemV4 &vo
     QFontMetrics fm( vopt.fontMetrics );
 
     int numberFillWidth = boldFm.width( QChar('0') ) * ( trackDigitCount - number.length() + 2 );
-    int numberRectWidth = numberFillWidth + boldFm.width( number ) + 2;
+    int numberRectWidth = 0;
+    if( number != QLatin1String("0. ") )
+        numberRectWidth = numberFillWidth + boldFm.width( number ) + 2;
     int lengthRectWidth = boldFm.width( length );
     int availableWidth = vopt.rect.width() - numberRectWidth - lengthRectWidth;
     if( availableWidth < fm.width( middle ) )
@@ -545,8 +560,10 @@ AlbumsItemDelegate::drawTrackText( QPainter *p, const QStyleOptionViewItemV4 &vo
         textRect = QRect( numberRect.topRight(), QSize( textRectWidth, vopt.rect.height() ) );
         lengthRect = QRect( textRect.topRight(), QSize( lengthRectWidth, vopt.rect.height() ) );
     }
+
+    if( number != QLatin1String("0. ") )
+        p->drawText( numberRect, Qt::AlignRight | Qt::AlignVCenter, number );
     p->drawText( textRect, Qt::AlignVCenter, middle );
-    p->drawText( numberRect, Qt::AlignRight | Qt::AlignVCenter, number );
     p->drawText( lengthRect, m_lengthAlignment | Qt::AlignVCenter, length );
     p->restore();
 }
