@@ -369,7 +369,6 @@ qreal PopupDropperItem::subitemOpacity() const
 
 void PopupDropperItem::setSubitemOpacity( qreal opacity )
 {
-#if QT_VERSION >= 0x040500
     if( d->svgItem )
         d->svgItem->setOpacity( opacity );
     if( d->textItem )
@@ -380,7 +379,6 @@ void PopupDropperItem::setSubitemOpacity( qreal opacity )
         d->hoverIndicatorRectItem->setOpacity( opacity );
     if( d->hoverIndicatorRectFillItem )
         d->hoverIndicatorRectFillItem->setOpacity( opacity );
-#endif
 }
 
 QGraphicsTextItem* PopupDropperItem::textItem() const
@@ -488,12 +486,6 @@ void PopupDropperItem::reposTextItem()
         return;
     }
 
-    int rightside;
-    if( !d->pd || d->pd->viewSize().width() == 0 )
-        rightside = sceneBoundingRect().width();
-    else
-        rightside = d->pd->viewSize().width();
-    
     int offsetPos = d->horizontalOffset + d->textOffset + d->svgElementRect.width();
     d->textItem->setPos(
             ( d->orientation == PopupDropperItem::Left
@@ -541,18 +533,18 @@ void PopupDropperItem::reposHoverFillRects()
 
     //qDebug() << "\n\nPUDItem boundingRect().width() = " << boundingRect().width();
     qreal startx, starty, endx, endy, item_center;
-    int rightside;
-        if( !d->pd || d->pd->viewSize().width() == 0 )
-            rightside = sceneBoundingRect().width();
-        else
-            rightside = d->pd->viewSize().width();
     //int rightside = d->borderRectItem ? d->borderRectItem->boundingRect().width() : boundingRect().width();
     if( d->orientation == PopupDropperItem::Left )
-        startx = d->horizontalOffset 
-                 - d->hoverIndicatorRectWidth
-                 - ( 2  * d->hoverIndicatorRectItem->pen().width() );
+    {
+        startx = d->horizontalOffset
+                - d->hoverIndicatorRectWidth
+                - ( 2  * d->hoverIndicatorRectItem->pen().width() );
+    }
     else
     {
+        int rightside = (!d->pd || d->pd->viewSize().width() == 0 )
+                ? sceneBoundingRect().width()
+                : d->pd->viewSize().width();
         //qDebug() << "right side = " << rightside;
         startx = rightside - d->horizontalOffset
                  + d->hoverIndicatorRectWidth
