@@ -29,11 +29,12 @@
 #include "core/meta/Meta.h"
 #include "core/meta/support/MetaUtility.h"
 #include "core-impl/collections/db/ScanManager.h"
-#include "core-impl/collections/db/sql/MountPointManager.h"
+#include "MountPointManager.h"
 #include "SqlCollection.h"
 #include "SqlMeta.h"
 #include "transcoding/TranscodingJob.h"
 #include "core/transcoding/TranscodingController.h"
+#include <shared/MetaTagLib.h> // for getting the uid
 
 #include <QDir>
 #include <QFile>
@@ -49,9 +50,9 @@
 
 using namespace Collections;
 
-SqlCollectionLocation::SqlCollectionLocation( SqlCollection const *collection )
+SqlCollectionLocation::SqlCollectionLocation( SqlCollection *collection )
     : CollectionLocation( collection )
-    , m_collection( const_cast<Collections::SqlCollection*>( collection ) )
+    , m_collection( collection )
     , m_delegateFactory( 0 )
     , m_overwriteFiles( false )
     , m_transferjob( 0 )
@@ -80,38 +81,31 @@ SqlCollectionLocation::actualLocation() const
 bool
 SqlCollectionLocation::isWritable() const
 {
-    DEBUG_BLOCK
     // TODO: This function is also called when removing files to check
     //  if the tracks can be removed. In such a case we should not check the space
 
-    // The collection is writeable if there exists a path that has more than
+    // The collection is writable if there exists a path that has more than
     // 500 MB free space.
     bool path_exists_with_space = false;
-    bool path_exists_writeable = false;
+    bool path_exists_writable = false;
     QStringList folders = actualLocation();
     foreach(QString path, folders)
     {
         float used = KDiskFreeSpaceInfo::freeSpaceInfo( path ).used();
         float total = KDiskFreeSpaceInfo::freeSpaceInfo( path ).size();
-	debug() << path;
-	debug() << "\tused: " << used;
-	debug() << "\ttotal: " << total;
 
         if( total <= 0 ) // protect against div by zero
             continue; //How did this happen?
 
         float free_space = total - used;
-        debug() <<"\tfree space: " << free_space;
         if( free_space >= 500*1000*1000 ) // ~500 megabytes
             path_exists_with_space = true;
 
         QFileInfo info( path );
         if( info.isWritable() )
-            path_exists_writeable = true;
-	debug() << "\tpath_exists_writeable" << path_exists_writeable;
-	debug() << "\tpath_exists_with_space" << path_exists_with_space;
+            path_exists_writable = true;
     }
-    return path_exists_with_space && path_exists_writeable;
+    return path_exists_with_space && path_exists_writable;
 }
 
 bool
@@ -171,9 +165,13 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
     QString rpath = m_collection->mountPointManager()->getRelativePath( deviceId, url );
     int directoryId = registry->getDirectory( QFileInfo(url).path() );
 
+    // -- the track uid (we can't use the original one from the old collection)
+    Meta::FieldHash fileTags = Meta::Tag::readTags( url );
+    QString uid = fileTags.value( Meta::valUniqueId ).toString();
+
     // -- the the track from the registry
     KSharedPtr<Meta::SqlTrack> metaTrack;
-    metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrackFromUid( track->uidUrl() ) );
+    metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrackFromUid( uid ) );
 
     if( metaTrack ) {
         warning() << "Location is inserting a file with the same uid as an already existing one.";
@@ -181,7 +179,7 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
         metaTrack->setUrl( deviceId, rpath, directoryId );
 
     } else {
-        metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrack( deviceId, rpath, directoryId, track->uidUrl() ) );
+        metaTrack = KSharedPtr<Meta::SqlTrack>::staticCast( registry->getTrack( deviceId, rpath, directoryId, uid ) );
     }
 
     // -- set the values
@@ -190,79 +188,96 @@ SqlCollectionLocation::insert( const Meta::TrackPtr &track, const QString &url )
 
     if( !track->name().isEmpty() )
         metaTrack->setTitle( track->name() );
-
     if( track->album() )
         metaTrack->setAlbum( track->album()->name() );
-
     if( track->artist() )
         metaTrack->setArtist( track->artist()->name() );
-
     if( track->composer() )
         metaTrack->setComposer( track->composer()->name() );
-
-    if( !track->year() >= 0 )
-        metaTrack->setYear( track->year() );
-
+    if( track->year() && track->year()->year() > 0 )
+        metaTrack->setYear( track->year()->year() );
     if( track->genre() )
         metaTrack->setGenre( track->genre()->name() );
 
-    // the filetype is not set or in the database.
-    // Meta::SqlTrack uses the file extension.
-
-    /* we've already done this
-    if( !track->path().isEmpty() )
-        metaTrack->setUrl( track->path() );
-        */
-
-    if( !track->uidUrl().isEmpty() )
-        metaTrack->setUidUrl( track->uidUrl() );
-
-    if( !track->bpm() >= 0 )
+    if( track->bpm() > 0 )
         metaTrack->setBpm( track->bpm() );
-
     if( !track->comment().isEmpty() )
         metaTrack->setComment( track->comment() );
 
-    if( !track->length() >= 0 )
-        metaTrack->setLength( track->length() );
+    if( track->score() > 0 )
+        metaTrack->setScore( track->score() );
+    if( track->rating() > 0 )
+        metaTrack->setRating( track->rating() );
 
+    /* These tags change when transcoding. Prefer to read those from file */
+    if( fileTags.value( Meta::valLength, 0 ).toLongLong() > 0 )
+        metaTrack->setLength( fileTags.value( Meta::valLength ).value<qint64>() );
+    else if( track->length() > 0 )
+        metaTrack->setLength( track->length() );
     // the filesize is updated every time after the
     // file is changed. Doesn't make sense to set it.
-
-    if( !track->sampleRate() >= 0 )
+    if( fileTags.value( Meta::valSamplerate, 0 ).toInt() > 0 )
+        metaTrack->setSampleRate( fileTags.value( Meta::valSamplerate ).toInt() );
+    else if( track->sampleRate() > 0 )
         metaTrack->setSampleRate( track->sampleRate() );
-
-    if( !track->bitrate() >= 0 )
+    if( fileTags.value( Meta::valBitrate, 0 ).toInt() > 0 )
+        metaTrack->setBitrate( fileTags.value( Meta::valBitrate ).toInt() );
+    else if( track->bitrate() > 0 )
         metaTrack->setBitrate( track->bitrate() );
 
-    if( !track->trackNumber() >= 0 )
-        metaTrack->setTrackNumber( track->trackNumber() );
+    // createDate is already set in Track constructor
+    if( track->modifyDate().isValid() )
+        metaTrack->setModifyDate( track->modifyDate() );
 
-    if( !track->discNumber() >= 0 )
+    if( track->trackNumber() > 0 )
+        metaTrack->setTrackNumber( track->trackNumber() );
+    if( track->discNumber() > 0 )
         metaTrack->setDiscNumber( track->discNumber() );
 
+    if( track->lastPlayed().isValid() )
+        metaTrack->setLastPlayed( track->lastPlayed() );
+    if( track->firstPlayed().isValid() )
+        metaTrack->setFirstPlayed( track->firstPlayed() );
+    if( track->playCount() > 0 )
+        metaTrack->setPlayCount( track->playCount() );
+
     Meta::ReplayGainTag modes[] = { Meta::ReplayGain_Track_Gain,
         Meta::ReplayGain_Track_Peak,
         Meta::ReplayGain_Album_Gain,
         Meta::ReplayGain_Album_Peak };
-
     for( int i=0; i<4; i++ )
-        if( !track->replayGain( modes[i] ) != 0 )
+        if( track->replayGain( modes[i] ) != 0 )
             metaTrack->setReplayGain( modes[i], track->replayGain( modes[i] ) );
 
     Meta::LabelList labels = track->labels();
     foreach( Meta::LabelPtr label, labels )
         metaTrack->addLabel( label );
 
-    metaTrack->endMetaDataUpdate();
+    if( fileTags.value( Meta::valFormat, int(Amarok::Unknown) ).toInt() != int(Amarok::Unknown) )
+        metaTrack->setType( Amarok::FileType( fileTags.value( Meta::valFormat ).toInt() ) );
+    else if( Amarok::FileTypeSupport::fileType( track->type() ) != Amarok::Unknown )
+        metaTrack->setType( Amarok::FileTypeSupport::fileType( track->type() ) );
 
     // Used to be updated after changes commit to prevent crash on NULL pointer access
     // if metaTrack had no album.
-    if( track->album() && track->album()->hasImage() && !metaTrack->album()->hasImage() )
-        metaTrack->album()->setImage( track->album()->image().toImage() );
+    if( track->album() && metaTrack->album() )
+    {
+        if( track->album()->hasAlbumArtist() && !metaTrack->album()->hasAlbumArtist() )
+            metaTrack->setAlbumArtist( track->album()->albumArtist()->name() );
 
+        if( track->album()->hasImage() && !metaTrack->album()->hasImage() )
+            metaTrack->album()->setImage( track->album()->image() );
+
+    }
+
+    metaTrack->endMetaDataUpdate();
     metaTrack->setWriteFile( true );
 
+    // we have a first shot at the meta data (expecially ratings and playcounts from media
+    // collections) but we still need to trigger the collection scanner
+    // to get the album and other meta data correct.
+    m_collection->scanManager()->delayedIncrementalScan( QFileInfo(url).path() );
+
     return true;
 }
 
@@ -306,7 +321,7 @@ SqlCollectionLocation::showDestinationDialog( const Meta::TrackList &tracks,
 
         // since bad things happen when drives become totally full
 	// we make sure there is at least ~500MB left
-        // finally, ensure the path is writeable
+        // finally, ensure the path is writable
         debug() << ( freeSpace - transferSize );
         if( ( freeSpace - transferSize ) > 1024*1024*500 && info.isWritable() )
             available_folders << path;
@@ -326,6 +341,7 @@ SqlCollectionLocation::showDestinationDialog( const Meta::TrackList &tracks,
     delegate->setFolders( available_folders );
     delegate->setIsOrganizing( ( collection() == source()->collection() ) );
     delegate->setTranscodingConfiguration( configuration );
+    delegate->setCaption( operationText( configuration ) );
 
     connect( delegate, SIGNAL( accepted() ), SLOT( slotDialogAccepted() ) );
     connect( delegate, SIGNAL( rejected() ), SLOT( slotDialogRejected() ) );
@@ -391,9 +407,10 @@ SqlCollectionLocation::slotRemoveJobFinished( KJob *job )
     {
         //TODO: proper error handling
         warning() << "An error occurred when removing a file: " << job->errorString();
-        transferError( track, KIO::buildErrorString( job->error(), job->errorString() ) );
     }
-    else
+
+    // -- remove the track from the database if it's gone
+    if( !QFile(track->playableUrl().path()).exists() )
     {
         // Remove the track from the database
         remove( track );
@@ -401,6 +418,10 @@ SqlCollectionLocation::slotRemoveJobFinished( KJob *job )
         //we  assume that KIO works correctly...
         transferSuccessful( track );
     }
+    else
+    {
+        transferError( track, KIO::buildErrorString( job->error(), job->errorString() ) );
+    }
 
     m_removejobs.remove( job );
     job->deleteLater();
@@ -461,23 +482,11 @@ SqlCollectionLocation::copyUrlsToCollection( const QMap<Meta::TrackPtr, KUrl> &s
 
     m_sources = sources;
 
-    QString statusBarTxt;
-
-    if( destination() == source() )
-        statusBarTxt = i18n( "Organizing tracks" );
-    else if ( isGoingToRemoveSources() )
-        statusBarTxt = i18n( "Moving tracks" );
-    else
-    {
-        if( configuration.encoder() == Transcoding::NULL_CODEC )
-            statusBarTxt = i18n( "Copying tracks" );
-        else
-            statusBarTxt = i18n( "Transcoding tracks" );
-    }
-
+    QString statusBarTxt = operationInProgressText( configuration, sources.count() );
     m_transferjob = new TransferJob( this, configuration );
-    Amarok::Components::logger()->newProgressOperation( m_transferjob, statusBarTxt, this, SLOT( slotTransferJobAborted() ) );
-    connect( m_transferjob, SIGNAL( result( KJob * ) ), this, SLOT( slotTransferJobFinished( KJob * ) ) );
+    Amarok::Components::logger()->newProgressOperation( m_transferjob, statusBarTxt, this,
+                                                        SLOT(slotTransferJobAborted()) );
+    connect( m_transferjob, SIGNAL(result( KJob * )), SLOT(slotTransferJobFinished( KJob * )) );
     m_transferjob->start();
 }
 
@@ -513,9 +522,11 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
 
         KUrl dest = m_destinations[ track ];
         dest.cleanPath();
-
         src.cleanPath();
-        if( configuration.encoder() == Transcoding::NULL_CODEC )
+
+        bool hasMoodFile = QFile::exists( moodFile( src ).toLocalFile() );
+
+        if( configuration.isJustCopy() )
             debug() << "copying from " << src << " to " << dest;
         else
             debug() << "transcoding from " << src << " to " << dest;
@@ -533,7 +544,7 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
         }
 
         KIO::JobFlags flags;
-        if( configuration.encoder() == Transcoding::NULL_CODEC )
+        if( configuration.isJustCopy() )
         {
             flags = KIO::HideProgressInfo;
             if( m_overwriteFiles )
@@ -543,6 +554,7 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
         }
 
         KJob *job = 0;
+        KJob *moodJob = 0;
 
         if( src.equals( dest ) )
         {
@@ -556,11 +568,17 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
         {
             debug() << "moving!";
             job = KIO::file_move( src, dest, -1, flags );
+            if( hasMoodFile )
+            {
+                KUrl moodSrc = moodFile( src );
+                KUrl moodDest = moodFile( dest );
+                moodJob = KIO::file_move( moodSrc, moodDest, -1, flags );
+            }
         }
         else
         {
             //later on in the case that remove is called, the file will be deleted because we didn't apply moveByDestination to the track
-            if( configuration.encoder() == Transcoding::NULL_CODEC )
+            if( configuration.isJustCopy() )
                 job = KIO::file_copy( src, dest, -1, flags );
             else
             {
@@ -572,17 +590,31 @@ bool SqlCollectionLocation::startNextJob( const Transcoding::Configuration confi
                 job = new Transcoding::Job( src, dest, configuration, this );
                 job->start();
             }
+
+            if( hasMoodFile )
+            {
+                KUrl moodSrc = moodFile( src );
+                KUrl moodDest = moodFile( dest );
+                moodJob = KIO::file_copy( moodSrc, moodDest, -1, flags );
+            }
         }
         if( job )   //just to be safe
         {
             connect( job, SIGNAL( result( KJob* ) ), SLOT( slotJobFinished( KJob* ) ) );
             connect( job, SIGNAL( result( KJob* ) ), m_transferjob, SLOT( slotJobFinished( KJob* ) ) );
             m_transferjob->addSubjob( job );
+
+            if( moodJob )
+            {
+                connect( moodJob, SIGNAL( result( KJob* ) ), m_transferjob, SLOT( slotJobFinished( KJob* ) ) );
+                m_transferjob->addSubjob( moodJob );
+            }
+
             QString name = track->prettyName();
             if( track->artist() )
                 name = QString( "%1 - %2" ).arg( track->artist()->name(), track->prettyName() );
 
-            if( configuration.encoder() == Transcoding::NULL_CODEC )
+            if( configuration.isJustCopy() )
                 m_transferjob->emitInfo( i18n( "Transferring: %1", name ) );
             else
                 m_transferjob->emitInfo( i18n( "Transcoding: %1", name ) );
@@ -602,6 +634,7 @@ bool SqlCollectionLocation::startNextRemoveJob()
         Meta::TrackPtr track = m_removetracks.takeFirst();
         // KUrl src = track->playableUrl();
         KUrl src = track->playableUrl();
+        KUrl srcMoodFile = moodFile( src );
 
         debug() << "isGoingToRemoveSources() " << isGoingToRemoveSources();
         if( isGoingToRemoveSources() && destination() ) // is organize operation?
@@ -620,6 +653,9 @@ bool SqlCollectionLocation::startNextRemoveJob()
         KIO::DeleteJob *job = KIO::del( src, KIO::HideProgressInfo );
         if( job )   //just to be safe
         {
+            if( QFile::exists( srcMoodFile.toLocalFile() ) )
+                KIO::del( srcMoodFile, KIO::HideProgressInfo );
+           
             connect( job, SIGNAL( result( KJob* ) ), SLOT( slotRemoveJobFinished( KJob* ) ) );
             QString name = track->prettyName();
             if( track->artist() )
@@ -634,6 +670,13 @@ bool SqlCollectionLocation::startNextRemoveJob()
     return false;
 }
 
+KUrl 
+SqlCollectionLocation::moodFile( const KUrl &track ) const
+{
+    KUrl moodPath = track;
+    moodPath.setFileName( "." + moodPath.fileName().replace( QRegExp( "(\\.\\w{2,5})$" ), ".mood" ) );
+    return moodPath;
+}
 
 TransferJob::TransferJob( SqlCollectionLocation * location, const Transcoding::Configuration & configuration )
     : KCompositeJob( 0 )
