@@ -30,8 +30,6 @@
 #include <QString>
 #include <QStringList>
 #include <QVariant>
-#include <QPixmap>
-#include <QPixmap>
 
 namespace Capabilities {
     class AlbumCapabilityDelegate;
@@ -95,7 +93,7 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
 
         /** true if there is a collection and the file exists on disk */
         virtual bool isPlayable() const;
-        /** true if there is a collection, the file exists on disk and is writeable */
+        /** true if there is a collection, the file exists on disk and is writable */
         virtual bool isEditable() const;
 
         virtual Meta::AlbumPtr album() const;
@@ -194,8 +192,6 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
 
         // SqlTrack specific methods
 
-        int deviceId() const;
-        QString rpath() const;
         int id() const;
         int urlId() const;
         Collections::SqlCollection* sqlCollection() const { return m_collection; }
@@ -205,8 +201,6 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
          *  this function still tries it's best to notify everyone
          *  That the track is now removed, plus it will also delete it from
          *  the database.
-         *  Do not call this directly as it does not clean up the registy.
-         *  Call m_registry->deleteTrack() instead
          */
         void remove();
 
@@ -248,7 +242,6 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlTrack : public Meta::Track
             if( !m_batchUpdate )
                 commitMetaDataChanges();
         }
-        void writeMetaDataToFile();
         void updatePlaylistsToDb( const FieldHash &fields, const QString &oldUid );
         void updateEmbeddedCoversToDb( const FieldHash &fields, const QString &oldUid );
 
@@ -332,8 +325,6 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlArtist : public Meta::Artist
 
         virtual Meta::TrackList tracks();
 
-        virtual Meta::AlbumList albums();
-
         virtual bool hasCapabilityInterface( Capabilities::Capability::Type type ) const;
 
         virtual Capabilities::Capability* createCapabilityInterface( Capabilities::Capability::Type type );
@@ -347,8 +338,6 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlArtist : public Meta::Artist
 
         bool m_tracksLoaded;
         Meta::TrackList m_tracks;
-        bool m_albumsLoaded;
-        Meta::AlbumList m_albums;
         QMutex m_mutex;
 
         friend class ::SqlRegistry; // needs to call notifyObservers
@@ -371,9 +360,9 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlAlbum : public Meta::Album
 
         virtual Meta::TrackList tracks();
 
-        /** Returns true if this album is a compilation, meaning that the included songs are from different artists.
-        */
         virtual bool isCompilation() const;
+        virtual bool canUpdateCompilation() const { return true; }
+        void setCompilation( bool compilation );
 
         /** Returns true if this album has an artist.
          *  The following equation is always true: isCompilation() != hasAlbumArtist()
@@ -399,13 +388,10 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlAlbum : public Meta::Album
          *  Returns a default image if no specific album image could be found.
          *  In such a case it will start the cover fetcher.
          *
-         *  Note: as this function can create a pixmap it is not recommended to
-         *  call this function from outside the UI thread.
-         *
          *  @param size is the maximum width or height of the resulting image.
          *  when size is <= 1, return the full size image
          */
-        virtual QPixmap image( int size = 0 );
+        virtual QImage image( int size = 0 ) const;
 
         virtual KUrl imageLocation( int size = 0 );
         virtual void setImage( const QImage &image );
@@ -420,21 +406,9 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlAlbum : public Meta::Album
         //SQL specific methods
         int id() const { return m_id; }
 
-        /** Set the compilation flag.
-         *  Actually it does not cange this album but instead moves
-         *  the tracks to other albums (e.g. one with the same name which is a
-         *  compilation)
-         *  If the compilation flag is set to "false" then all songs
-         *  with different artists will be moved to other albums, possibly even
-         *  creating them.
-         */
-        void setCompilation( bool compilation );
         Collections::SqlCollection *sqlCollection() const { return m_collection; }
 
     private:
-        /** Cleans all images from this album cached in the QPixmapCache */
-        void clearPixmapCache();
-
         QByteArray md5sum( const QString& artist, const QString& album, const QString& file ) const;
 
         /** Returns a unique key for the album cover. */
@@ -484,8 +458,6 @@ class AMAROK_SQLCOLLECTION_EXPORT_TESTS SqlAlbum : public Meta::Album
         mutable QString m_imagePath; // path read from the database
         mutable bool m_hasImage; // true if we have an original image
         mutable bool m_hasImageChecked; // true if hasImage was checked
-        QSet<QString> m_pixmapCacheIds; // all our image keys
-        bool m_pixmapCacheDirty; // true if we should remove our images from the cache
 
         mutable int m_unsetImageId; // this is the id of the unset magic value in the image sql database
         static const QString AMAROK_UNSET_MAGIC;
