@@ -18,23 +18,27 @@
 #include "File.h"
 #include "File_p.h"
 
-#include "core/support/Amarok.h"
-#include "amarokurls/BookmarkMetaActions.h"
-#include "browsers/filebrowser/FileBrowser.h"
 #include <config-amarok.h>
 #include "MainWindow.h"
-#include "core/meta/Meta.h"
+#include "amarokurls/BookmarkMetaActions.h"
+#include "amarokurls/PlayUrlRunner.h"
+#include "browsers/filebrowser/FileBrowser.h"
 #include "core/capabilities/BookmarkThisCapability.h"
 #include "core/capabilities/EditCapability.h"
 #include "core/capabilities/FindInSourceCapability.h"
 #include "core/capabilities/StatisticsCapability.h"
+#include "core/meta/Meta.h"
+#include "core/meta/support/MetaUtility.h"
+#include "core/playlists/PlaylistFormat.h"
+#include "core/support/Amarok.h"
 #include "core-impl/capabilities/timecode/TimecodeWriteCapability.h"
 #include "core-impl/capabilities/timecode/TimecodeLoadCapability.h"
 #include "core-impl/statistics/providers/url/PermanentUrlStatisticsProvider.h"
-#include "core/meta/support/MetaUtility.h"
-#include "amarokurls/PlayUrlRunner.h"
+
+#include <KMimeType>
 
 #include <QAction>
+#include <QFileInfo>
 #include <QList>
 #include <QWeakPointer>
 #include <QString>
@@ -65,7 +69,6 @@ class EditCapabilityImpl : public Capabilities::EditCapability
         virtual void setComment( const QString &newComment ) { m_track->setComment( newComment ); }
         virtual void setTrackNumber( int newTrackNumber ) { m_track->setTrackNumber( newTrackNumber ); }
         virtual void setDiscNumber( int newDiscNumber ) { m_track->setDiscNumber( newDiscNumber ); }
-        virtual void setUidUrl( const QString &newUidUrl ) { m_track->setUidUrl( newUidUrl ); }
         virtual void beginMetaDataUpdate() { m_track->beginMetaDataUpdate(); }
         virtual void endMetaDataUpdate() { m_track->endMetaDataUpdate(); }
 
@@ -188,6 +191,7 @@ Track::Track( const KUrl &url )
     d->readMetaData();
     d->album = Meta::AlbumPtr( new MetaFile::FileAlbum( d ) );
     d->artist = Meta::ArtistPtr( new MetaFile::FileArtist( d ) );
+    d->albumArtist = Meta::ArtistPtr( new MetaFile::FileArtist( d, true ) );
     d->genre = Meta::GenrePtr( new MetaFile::FileGenre( d ) );
     d->composer = Meta::ComposerPtr( new MetaFile::FileComposer( d ) );
     d->year = Meta::YearPtr( new MetaFile::FileYear( d ) );
@@ -235,46 +239,18 @@ Track::uidUrl() const
     return d->url.url();
 }
 
-void
-Track::setUidUrl( const QString &newUid ) const
-{
-    if( newUid.isEmpty() )
-        return;
-
-    d->changes.insert( Meta::valUniqueId, QVariant( newUid ) );
-    if( !d->batchUpdate )
-    {
-        d->writeMetaData();
-        notifyObservers();
-    }
-}
-
 bool
 Track::isPlayable() const
 {
-    //simple implementation, check Internet connectivity or ping server?
-    return true;
+    QFileInfo info = QFileInfo( playableUrl().pathOrUrl() );
+    return info.isFile() && info.isReadable();
 }
 
 bool
 Track::isEditable() const
 {
-    DEBUG_BLOCK
-
-    //note this probably needs more work on *nix
-    QFile::Permissions p;
-    if(d->url.isLocalFile())
-    {
-        p = QFile::permissions( d->url.toLocalFile() );
-    }
-    else
-    {
-        p = QFile::permissions( d->url.path() );
-    }
-    const bool editable = ( p & QFile::WriteUser ) || ( p & QFile::WriteGroup ) || ( p & QFile::WriteOther );
-
-    debug() << d->url.path() << " editable: " << editable;
-    return editable;
+    QFileInfo info = QFileInfo( playableUrl().pathOrUrl() );
+    return info.isFile() && info.isWritable();
 }
 
 Meta::AlbumPtr
@@ -615,6 +591,30 @@ Track::type() const
     return Amarok::extension( d->url.fileName() );
 }
 
+bool
+Track::isTrack( const KUrl &url )
+{
+    // some playlists lay under audio/ mime category, filter them
+    if( Playlists::isPlaylist( url ) )
+        return false;
+
+    // accept remote files, it's too slow to check them at this point
+    if( !url.isLocalFile() )
+        return true;
+
+    QFileInfo  fileInfo( url.toLocalFile() );
+    if( fileInfo.size() <= 0 )
+        return false;
+
+    // We can't play directories
+    if( fileInfo.isDir() )
+        return false;
+
+    const KMimeType::Ptr mimeType = KMimeType::findByPath( url.toLocalFile() );
+    const QString name = mimeType->name();
+    return name.startsWith( "audio/" ) || name.startsWith( "video/" );
+}
+
 void
 Track::beginMetaDataUpdate()
 {
@@ -641,13 +641,19 @@ Track::finishedPlaying( double playedFraction )
 bool
 Track::inCollection() const
 {
-    return false;
+    return d->collection; // calls QWeakPointer's (bool) operator
 }
 
 Collections::Collection*
 Track::collection() const
 {
-    return 0;
+    return d->collection.data();
+}
+
+void
+Track::setCollection( Collections::Collection *newCollection )
+{
+    d->collection = newCollection;
 }
 
 bool
@@ -657,7 +663,7 @@ Track::hasCapabilityInterface( Capabilities::Capability::Type type ) const
 #ifdef HAVE_LIBLASTFM
     readlabel = true;
 #endif
-    return type == Capabilities::Capability::Editable ||
+    return ( type == Capabilities::Capability::Editable && isEditable() ) ||
            type == Capabilities::Capability::Importable ||
            type == Capabilities::Capability::BookmarkThis ||
            type == Capabilities::Capability::WriteTimecode ||
@@ -672,7 +678,10 @@ Track::createCapabilityInterface( Capabilities::Capability::Type type )
     switch( type )
     {
         case Capabilities::Capability::Editable:
-            return new EditCapabilityImpl( this );
+            if( isEditable() )
+                return new EditCapabilityImpl( this );
+            else
+                return 0;
 
         case Capabilities::Capability::Importable:
             return new StatisticsCapabilityImpl( this );
