@@ -17,7 +17,7 @@
 #ifndef DBUSQUERYHELPER_H
 #define DBUSQUERYHELPER_H
 
-#include "meta/Meta.h"
+#include "core/meta/Meta.h"
 
 #include <QDBusConnection>
 #include <QDBusMessage>
@@ -26,7 +26,10 @@
 #include <QObject>
 #include <QString>
 #include <QVariant>
-class QueryMaker;
+
+namespace Collections {
+    class QueryMaker;
+}
 
 typedef QList<QVariantMap> VariantMapList;
 
@@ -35,11 +38,11 @@ class DBusQueryHelper : public QObject
     Q_OBJECT
     
     public:
-        DBusQueryHelper( QObject *parent, QueryMaker *qm, const QDBusConnection &conn, const QDBusMessage &msg, bool mprisCompatible );
+        DBusQueryHelper( QObject *parent, Collections::QueryMaker *qm, const QDBusConnection &conn, const QDBusMessage &msg, bool mprisCompatible );
         
     private slots:
-        void slotResultReady( const QString &collectionId, const Meta::TrackList &tracks );
-        
+        void slotResultReady( const Meta::TrackList &tracks );
+
         void slotQueryDone();
 
         void abortQuery();
