@@ -43,8 +43,38 @@
 #include <QAction>
 #include <typeinfo>
 
-Playlist::Controller::Controller( AbstractModel* bottomModel, AbstractModel* topModel, QObject* parent )
-        : QObject( parent )
+
+namespace The
+{
+    AMAROK_EXPORT Playlist::Controller* playlistController()
+    {
+        return Playlist::Controller::instance();
+    }
+}
+
+
+Playlist::Controller* Playlist::Controller::s_instance = 0;
+
+Playlist::Controller*
+Playlist::Controller::instance()
+{
+    if( s_instance == 0 )
+        s_instance = new Controller();
+    return s_instance;
+}
+
+void
+Playlist::Controller::destroy()
+{
+    if( s_instance )
+    {
+        delete s_instance;
+        s_instance = 0;
+    }
+}
+
+Playlist::Controller::Controller()
+        : QObject()
         , m_undoStack( new QUndoStack( this ) )
 {
     DEBUG_BLOCK
@@ -53,8 +83,10 @@ Playlist::Controller::Controller( AbstractModel* bottomModel, AbstractModel* top
     //Playlist::ModelStack::instance->top() or simply The::playlist().
     //This is an exception, because we handle the presence of tracks in the bottom model,
     //so we get a pointer to the bottom model and use it with great care.
-    m_bottomModel = bottomModel;
-    m_topModel = topModel;
+    // TODO: get these values only when we really need them to loosen up the
+    // coupling between Controller and Model
+    m_bottomModel = Playlist::ModelStack::instance()->bottom();
+    m_topModel = The::playlist();
 
     m_undoStack->setUndoLimit( 20 );
     connect( m_undoStack, SIGNAL( canRedoChanged( bool ) ), this, SIGNAL( canRedoChanged( bool ) ) );
@@ -96,6 +128,7 @@ Playlist::Controller::insertOptioned( Meta::TrackList list, int options )
     }
 
     int topModelInsertRow;
+    int visibleInsertedRowCount;
     if( options & Replace )
     {
         debug()<<"Replace";
@@ -107,14 +140,17 @@ Playlist::Controller::insertOptioned( Meta::TrackList list, int options )
         //make sure that we turn off dynamic mode.
         Amarok::actionCollection()->action( "disable_dynamic" )->trigger();
 
-        topModelInsertRow = 0;
-        insertionHelper( insertionTopRowToBottom( topModelInsertRow ), list );
+        int bottomModelInsertRow = insertionTopRowToBottom( 0 );
+        insertionHelper( bottomModelInsertRow, list );
+        topModelInsertRow = m_topModel->rowFromBottomModel( bottomModelInsertRow );
         m_undoStack->endMacro();
+        visibleInsertedRowCount = m_topModel->qaim()->rowCount(); // simple
     }
     else if( options & Queue )
     {
         debug()<<"Queue";
 
+        int oldVisibleRowCount = m_topModel->qaim()->rowCount();
         topModelInsertRow = m_topModel->activeRow() + 1;
 
         while( m_topModel->queuePositionOfRow( topModelInsertRow ) )
@@ -122,6 +158,8 @@ Playlist::Controller::insertOptioned( Meta::TrackList list, int options )
 
         int bottomModelInsertRow = insertionTopRowToBottom( topModelInsertRow );
         insertionHelper( bottomModelInsertRow, list );
+        topModelInsertRow = m_topModel->rowFromBottomModel( bottomModelInsertRow );
+        visibleInsertedRowCount = m_topModel->qaim()->rowCount() - oldVisibleRowCount;
 
         // Construct list of rows to be queued
         QList<int> topModelRows;
@@ -137,8 +175,14 @@ Playlist::Controller::insertOptioned( Meta::TrackList list, int options )
     else
     {
         debug()<<"Append";
+
+        int oldVisibleRowCount = m_topModel->qaim()->rowCount();
         topModelInsertRow = m_topModel->qaim()->rowCount();
-        insertionHelper( insertionTopRowToBottom( topModelInsertRow ), list );
+
+        int bottomModelInsertRow = insertionTopRowToBottom( topModelInsertRow );
+        insertionHelper( bottomModelInsertRow, list );
+        topModelInsertRow = m_topModel->rowFromBottomModel( bottomModelInsertRow );
+        visibleInsertedRowCount = m_topModel->qaim()->rowCount() - oldVisibleRowCount;
     }
 
     debug() << "engine playing?: " << The::engineController()->isPlaying();
@@ -146,16 +190,26 @@ Playlist::Controller::insertOptioned( Meta::TrackList list, int options )
     bool playNow = false;
     if( options & DirectPlay )
         playNow = true;
-    if( options & StartPlay )
-        if( !The::engineController()->isPlaying() )
-            playNow = true;
 
-    if ( playNow ) {
+    if( options & StartPlay )
+    {
+        bool isDynamic = AmarokConfig::dynamicMode();
+        bool isPaused  = The::engineController()->isPaused();
+        bool isPlaying = The::engineController()->isPlaying();
+
+        // when not in dyanmic mode: start playing when paused
+        // when in dyanmic mode: do not start playing when paused
+        if( !isPlaying && (!isDynamic || (isDynamic && !isPaused)) )
+            playNow = true;
+    }
+
+    if( playNow && visibleInsertedRowCount > 0 )
+    {
+        int fuzz = 0;
         if ( AmarokConfig::trackProgression() == AmarokConfig::EnumTrackProgression::RandomTrack ||
              AmarokConfig::trackProgression() == AmarokConfig::EnumTrackProgression::RandomAlbum )
-            Actions::instance()->play();
-        else
-            Actions::instance()->play( topModelInsertRow );
+            fuzz = qrand() % visibleInsertedRowCount;
+        Actions::instance()->play( topModelInsertRow + fuzz );
     }
 
     emit changed();
@@ -181,17 +235,6 @@ Playlist::Controller::insertOptioned( Playlists::PlaylistList list, int options
         insertOptioned( playlist, options );
 }
 
-void
-Playlist::Controller::insertOptioned( Collections::QueryMaker *qm, int options )
-{
-    DEBUG_BLOCK
-    qm->setQueryType( Collections::QueryMaker::Track );
-    connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ) );
-    connect( qm, SIGNAL( newResultReady( QString, Meta::TrackList ) ), SLOT( newResultReady( QString, Meta::TrackList ) ) );
-    m_optionedQueryMap.insert( qm, options );
-    qm->run();
-}
-
 void
 Playlist::Controller::insertOptioned( QList<KUrl>& urls, int options )
 {
@@ -244,17 +287,6 @@ Playlist::Controller::insertPlaylists( int topModelRow, Playlists::PlaylistList
     insertTracks( topModelRow, tl );
 }
 
-void
-Playlist::Controller::insertTracks( int row, Collections::QueryMaker *qm )
-{
-    DEBUG_BLOCK
-    qm->setQueryType( Collections::QueryMaker::Track );
-    connect( qm, SIGNAL( queryDone() ), SLOT( queryDone() ) );
-    connect( qm, SIGNAL( newResultReady( QString, Meta::TrackList ) ), SLOT( newResultReady( QString, Meta::TrackList ) ) );
-    m_queryMap.insert( qm, row );
-    qm->run();
-}
-
 void
 Playlist::Controller::insertUrls( int row, const QList<KUrl>& urls )
 {
@@ -495,42 +527,6 @@ Playlist::Controller::clear()
  * Private Functions
  **************************************************/
 
-void
-Playlist::Controller::newResultReady( const QString&, const Meta::TrackList& tracks )
-{
-    DEBUG_BLOCK
-
-    Collections::QueryMaker *qm = dynamic_cast<Collections::QueryMaker*>( sender() );
-    if( qm )
-    {
-        m_queryMakerTrackResults[qm] += tracks;
-    }
-}
-
-void
-Playlist::Controller::queryDone()
-{
-    DEBUG_BLOCK
-
-    Collections::QueryMaker *qm = dynamic_cast<Collections::QueryMaker*>( sender() );
-    if( qm )
-    {
-        qStableSort( m_queryMakerTrackResults[qm].begin(), m_queryMakerTrackResults[qm].end(), Meta::Track::lessThan );
-        if( m_queryMap.contains( qm ) )
-        {
-            insertTracks( m_queryMap.value( qm ), m_queryMakerTrackResults.value( qm ) );
-            m_queryMap.remove( qm );
-        }
-        else if( m_optionedQueryMap.contains( qm ) )
-        {
-            insertOptioned( m_queryMakerTrackResults.value( qm ), m_optionedQueryMap.value( qm ) );
-            m_optionedQueryMap.remove( qm );
-        }
-        m_queryMakerTrackResults.remove( qm );
-        qm->deleteLater();
-    }
-}
-
 void
 Playlist::Controller::slotFinishDirectoryLoader( const Meta::TrackList& tracks )
 {
@@ -602,7 +598,7 @@ Playlist::Controller::insertionHelper( int bottomModelRow, Meta::TrackList& tl )
             KUrl cuesheet = MetaCue::CueFileSupport::locateCueSheet( track->playableUrl() );
             if( !cuesheet.isEmpty() )
             {
-                MetaCue::CueFileItemMap cueMap = MetaCue::CueFileSupport::loadCueFile( track );
+                MetaCue::CueFileItemMap cueMap = MetaCue::CueFileSupport::loadCueFile( cuesheet, track );
                 if( !cueMap.isEmpty() )
                 {
                     Meta::TrackList cueTracks = MetaCue::CueFileSupport::generateTimeCodeTracks( track, cueMap );
