@@ -25,27 +25,40 @@
 #include "amarokconfig.h"
 #include "App.h"
 #include "core/support/Debug.h"
-#include "DynamicModel.h"
 #include "layouts/LayoutManager.h"
 #include "MainWindow.h"
 #include "navigators/NavigatorConfigAction.h"
 #include "PaletteHandler.h"
 #include "PlaylistController.h"
 #include "PlaylistDefines.h"
+#include "PlaylistInfoWidget.h"
 #include "PlaylistManager.h"
 #include "PlaylistModelStack.h"
 #include "PlaylistQueueEditor.h"
+#include "PlaylistToolBar.h"
 #include "ProgressiveSearchWidget.h"
+#include "playlist/PlaylistActions.h"
 #include "core-impl/playlists/providers/user/UserPlaylistProvider.h"
 #include "widgets/HorizontalDivider.h"
+#include "amarokurls/AmarokUrl.h"
 
 #include <KActionMenu>
+#include <KStandardDirs>
 #include <KToolBarSpacerAction>
+#include <KVBox>
 
+#include <QLabel>
+#include <QToolBar>
 #include <QHBoxLayout>
 
+
+static const QString s_dynMode( "dynamic_mode" );
+static const QString s_repopulate( "repopulate" );
+static const QString s_turnOff( "turn_off" );
+
 Playlist::Dock::Dock( QWidget* parent )
     : AmarokDockWidget( i18n( "&Playlist" ), parent )
+    , m_barBox( 0 )
 {
     DEBUG_BLOCK
 
@@ -78,6 +91,7 @@ void
 Playlist::Dock::polish()
 {
     DEBUG_BLOCK
+
     m_mainWidget = new KVBox( this );
     setWidget( m_mainWidget );
     m_mainWidget->setContentsMargins( 0, 0, 0, 0 );
@@ -92,24 +106,25 @@ Playlist::Dock::polish()
     m_searchWidget = new Playlist::ProgressiveSearchWidget( m_mainWidget );
 
     // show visual indication of dynamic playlists  being enabled
-    connect( PlaylistBrowserNS::DynamicModel::instance(), SIGNAL( enableDynamicMode( bool ) ),
-             SLOT( showDynamicHint( bool ) ) );
-    m_dynamicHintWidget = new QLabel( i18n( "Dynamic Mode Enabled" ), m_mainWidget );
+    connect( The::playlistActions(), SIGNAL( navigatorChanged() ),
+             SLOT( showDynamicHint() ) );
+    m_dynamicHintWidget = new QLabel( i18n( "<a href='%1'>Dynamic Mode</a> Enabled. "
+        "<a href='%2'>Repopulate</a> | <a href='%3'>Turn off</a>", s_dynMode,
+        s_repopulate, s_turnOff ), m_mainWidget );
     m_dynamicHintWidget->setAlignment( Qt::AlignCenter );
-    m_dynamicHintWidget->setStyleSheet(
-            QString( "QLabel { background-color: %1; color: %2; border-radius: 3px; } " )
-                    .arg( PaletteHandler::highlightColor().name() )
-                    .arg( The::paletteHandler()->palette().highlightedText().color().name() )
-            );
+    m_dynamicHintWidget->setTextInteractionFlags( Qt::LinksAccessibleByKeyboard | Qt::LinksAccessibleByMouse );
+    m_dynamicHintWidget->setMinimumSize( 1, 1 ); // so that it doesn't prevent playlist from shrinking
+    connect( m_dynamicHintWidget, SIGNAL(linkActivated(QString)), SLOT(slotDynamicHintLinkActivated(QString)) );
+
     QFont dynamicHintWidgetFont = m_dynamicHintWidget->font();
     dynamicHintWidgetFont.setPointSize( dynamicHintWidgetFont.pointSize() + 1 );
     m_dynamicHintWidget->setFont( dynamicHintWidgetFont );
 
-    showDynamicHint( AmarokConfig::dynamicMode() );
+    showDynamicHint();
 
     paletteChanged( App::instance()->palette() );
     connect( The::paletteHandler(), SIGNAL( newPalette( const QPalette& ) ),
-             SLOT(  paletteChanged( const QPalette &  ) ) );
+             SLOT( paletteChanged( const QPalette &  ) ) );
 
     QWidget * layoutHolder = new QWidget( m_mainWidget );
 
@@ -149,29 +164,29 @@ Playlist::Dock::polish()
     mainPlaylistlayout->setSpacing( 0 );
     mainPlaylistlayout->addWidget( m_playlistView );
 
-    KHBox *barBox = new KHBox( m_mainWidget );
-    barBox->setMargin( 0 );
-    barBox->setContentsMargins( 0, 0, 0, 0 );
-
-    // Use QToolBar instead of KToolBar, see bug 228390
-    QToolBar *plBar = new QToolBar( barBox );
-    plBar->setFixedHeight( 30 );
-    plBar->setObjectName( "PlaylistToolBar" );
-
     ModelStack::instance(); //This also creates the Controller.
 
-    { // START Playlist toolbar
+    { // START: Playlist toolbar
+        // action toolbar
+        m_barBox = new KHBox( m_mainWidget );
+        m_barBox->setObjectName( "PlaylistBarBox" );
+        m_barBox->setContentsMargins( 0, 0, 0, 0 );
+        m_barBox->setFixedHeight( 36 );
+
+        // Use QToolBar instead of KToolBar, see bug 228390
+        Playlist::ToolBar *plBar = new Playlist::ToolBar( m_barBox );
         plBar->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Preferred );
         plBar->setMovable( false );
-        plBar->addAction( new KToolBarSpacerAction( m_mainWidget ) );
 
-        plBar->addAction( Amarok::actionCollection()->action( "playlist_clear" ) );
 
-        plBar->addSeparator();
+        QActionGroup *playlistActions = new QActionGroup( m_mainWidget );
+        playlistActions->addAction( Amarok::actionCollection()->action( "playlist_clear" ) );
 
         m_savePlaylistMenu = new KActionMenu( KIcon( "document-save-amarok" ),
                                               i18n("&Save Current Playlist"), m_mainWidget );
+
         m_saveActions = new KActionCollection( m_mainWidget );
+
         connect( m_savePlaylistMenu, SIGNAL( triggered( bool ) ),
                  SLOT( slotSaveCurrentPlaylist() ) );
         foreach( Playlists::PlaylistProvider *provider, The::playlistManager()->providersForCategory(
@@ -189,32 +204,32 @@ Playlist::Dock::polish()
                  SLOT( playlistProviderRemoved( Playlists::PlaylistProvider *, int ) )
                  );
 
-        plBar->addAction( m_savePlaylistMenu );
+        playlistActions->addAction( m_savePlaylistMenu );
 
-        plBar->addSeparator();
-        plBar->addAction( Amarok::actionCollection()->action( "playlist_undo" ) );
-        plBar->addAction( Amarok::actionCollection()->action( "playlist_redo" ) );
-        plBar->addSeparator();
+        playlistActions->addAction( Amarok::actionCollection()->action( "playlist_undo" ) );
+        //redo action can be accessed from menu > Playlist
 
-        plBar->addAction( Amarok::actionCollection()->action( "show_active_track" ) );
-        plBar->addSeparator();
+        playlistActions->addAction( Amarok::actionCollection()->action( "show_active_track" ) );
 
-        NavigatorConfigAction * navigatorConfig = new NavigatorConfigAction( m_mainWidget );
+        plBar->addCollapsibleActions( playlistActions );
+
+        NavigatorConfigAction *navigatorConfig = new NavigatorConfigAction( m_mainWidget );
         plBar->addAction( navigatorConfig );
-        plBar->addSeparator();
 
         QToolButton *toolButton =
-                qobject_cast<QToolButton*>(plBar->widgetForAction( navigatorConfig ) );
+                qobject_cast<QToolButton *>(plBar->widgetForAction( navigatorConfig ) );
         if( toolButton )
             toolButton->setPopupMode( QToolButton::InstantPopup );
 
-        QAction *queueEditAction = Amarok::actionCollection()->action( "playlist_edit_queue" );
-        plBar->addAction( queueEditAction );
         plBar->addAction( new KToolBarSpacerAction( m_mainWidget ) );
-        connect( queueEditAction, SIGNAL( triggered( bool ) ),
-                 SLOT( slotEditQueue() ) );
+
+        // label widget
+        new PlaylistInfoWidget( m_barBox );
     } // END Playlist Toolbar
 
+    //set correct colors
+    paletteChanged( QApplication::palette() );
+
     // If it is active, clear the search filter before replacing the playlist. Fixes Bug #200709.
     connect( The::playlistController(), SIGNAL( replacingPlaylist() ),
              SLOT( clearFilterIfActive() ) );
@@ -229,12 +244,26 @@ Playlist::Dock::sizeHint() const
 
 
 void
-Playlist::Dock::paletteChanged( const QPalette& palette )
+Playlist::Dock::paletteChanged( const QPalette &palette )
 {
-    m_dynamicHintWidget->setStyleSheet( QString( "QLabel { background-color: %1; color: %2; } " )
-                                        .arg( PaletteHandler::highlightColor().name() )
-                                        .arg( palette.highlightedText().color().name() )
-                                        );
+    const QString backgroundColor = PaletteHandler::highlightColor().name();
+    const QString textColor = palette.color( QPalette::HighlightedText ).name();
+    const QString linkColor = palette.color( QPalette::Link ).name();
+    const QString ridgeColor = palette.color( QPalette::Window ).name();
+
+    QString hintStyle( "QLabel { background-color: %1; color: %2; border-radius: 3px; } "
+                       "a { color: %3; }" );
+    hintStyle = hintStyle.arg( backgroundColor, textColor, linkColor );
+
+    QString barStyle( "QFrame#PlaylistBarBox { border: 1px ridge %1; background-color: %2; "
+                                             " color: %3; border-radius: 3px; } "
+                      "QLabel { color: %3; }" );
+    barStyle = barStyle.arg( ridgeColor, backgroundColor, textColor );
+
+    m_dynamicHintWidget->setStyleSheet( hintStyle );
+    if( m_barBox )
+        m_barBox->setStyleSheet( barStyle );
+
 }
 
 void
@@ -243,7 +272,7 @@ Playlist::Dock::playlistProviderAdded( Playlists::PlaylistProvider *provider, in
     if( category != PlaylistManager::UserPlaylist )
         return;
 
-    debug() << "Adding provider: " << provider->objectName();
+    debug() << "Adding provider: " << provider->prettyName();
     Playlists::UserPlaylistProvider *userProvider =
             dynamic_cast<Playlists::UserPlaylistProvider *>(provider);
     if( userProvider == 0 )
@@ -253,7 +282,7 @@ Playlist::Dock::playlistProviderAdded( Playlists::PlaylistProvider *provider, in
                                    this );
     action->setData( QVariant::fromValue(
             QWeakPointer<Playlists::UserPlaylistProvider>( userProvider ) ) );
-    m_saveActions->addAction( provider->objectName(), action );
+    m_saveActions->addAction( QString::number( (qlonglong) userProvider ), action );
 
     m_savePlaylistMenu->addAction( action );
     connect( action, SIGNAL( triggered(bool) ), SLOT( slotSaveCurrentPlaylist() ) );
@@ -265,9 +294,11 @@ Playlist::Dock::playlistProviderRemoved( Playlists::PlaylistProvider *provider,
     if( category != PlaylistManager::UserPlaylist )
         return;
 
-    QAction *action = m_saveActions->action( provider->objectName() );
+    QAction *action = m_saveActions->action( QString::number( (qlonglong) provider ) );
     if( action )
         m_savePlaylistMenu->removeAction( action );
+    else
+        warning() << __PRETTY_FUNCTION__ << ": no save action for provider" << provider->prettyName();
 }
 
 void
@@ -283,9 +314,8 @@ Playlist::Dock::slotSaveCurrentPlaylist()
             action->data().value< QWeakPointer<Playlists::UserPlaylistProvider> >();
     Playlists::UserPlaylistProvider* provider = pointer.data();
 
-    The::playlistManager()->save( The::playlist()->tracks(),
-                                  Playlist::ModelStack::instance()->bottom()->generatePlaylistName(),
-                                  provider );
+    const Meta::TrackList tracks = The::playlist()->tracks();
+    The::playlistManager()->save( tracks, Amarok::generatePlaylistName( tracks ), provider );
 }
 
 void
@@ -308,11 +338,17 @@ Playlist::Dock::showActiveTrack()
 }
 
 void
-Playlist::Dock::showDynamicHint( bool enabled ) // slot
+Playlist::Dock::editTrackInfo()
+{
+    m_playlistView->editTrackInformation();
+}
+
+void
+Playlist::Dock::showDynamicHint() // slot
 {
     DEBUG_BLOCK
 
-    if( enabled )
+    if( AmarokConfig::dynamicMode() )
         m_dynamicHintWidget->show();
     else
         m_dynamicHintWidget->hide();
@@ -328,3 +364,14 @@ Playlist::Dock::clearFilterIfActive() // slot
     if( filterActive )
         m_searchWidget->slotFilterClear();
 }
+
+void
+Playlist::Dock::slotDynamicHintLinkActivated( const QString &href )
+{
+    if( href == s_dynMode )
+        AmarokUrl( "amarok://navigate/playlists/dynamic category" ).run();
+    else if( href == s_repopulate )
+        The::playlistActions()->repopulateDynamicPlaylist();
+    else if( href == s_turnOff )
+        The::playlistActions()->enableDynamicMode( false );
+}
