@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2008-2010 Soren Harward <stharward@gmail.com>                          *
+ * Copyright (c) 2008-2011 Soren Harward <stharward@gmail.com>                          *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -34,16 +34,17 @@ namespace APG {
         Q_OBJECT
 
         public:
+            typedef QHash< Meta::TrackList*, double > Population;
+            
             static const int QUALITY_RANGE; // allows used to adjust speed/quality tradeoff
 
             ConstraintSolver( ConstraintNode*, int );
             ~ConstraintSolver();
 
             Meta::TrackList getSolution() const;
-            double finalSatisfaction() const;
-            double satisfactionThreshold() const;
+            bool satisfied() const;
             int serial() const { return m_serialNumber; }
-            int iterationCount() const { return m_maxCoolingIterations; }
+            int iterationCount() const { return m_maxGenerations; }
 
             // overloaded ThreadWeaver::Job functions
             bool canBeExecuted();
@@ -55,23 +56,25 @@ namespace APG {
         signals:
             void readyToRun();
             void incrementProgress();
+            void totalSteps( int );
+            void endProgressOperation( QObject* );
 
         protected:
             void run(); // from ThreadWeaver::Job
 
         private slots:
-            void receiveQueryMakerData( QString, Meta::TrackList );
+            void receiveQueryMakerData( Meta::TrackList );
             void receiveQueryMakerDone();
 
         private:
             int m_serialNumber;                 // a randomly-generated serial number to help with debugging
-            int m_minPlaylistSize;
-            int m_maxPlaylistSize;
+            double m_satisfactionThreshold;
+            double m_finalSatisfaction;
 
             ConstraintNode* const m_constraintTreeRoot;
             Collections::MetaQueryMaker* m_qm;
 
-            Meta::TrackList m_domain;           // tracks available to solver
+            mutable Meta::TrackList m_domain;   // tracks available to solver
             QMutex m_domainMutex;
             bool m_domainReductionFailed;
             Meta::TrackList m_solvedPlaylist;   // playlist generated by solver
@@ -80,20 +83,32 @@ namespace APG {
             bool m_abortRequested;
 
             // internal mathematical functions
-            Meta::TrackPtr randomTrackFromDomain() const;
-            double mutateRandom( const double );
-            double mutateByVote( const double );
-            double improveBySwapping();
+            void fill_population( Population& );
+            Meta::TrackList* find_best( const Population& ) const;
+            void select_population( Population&, Meta::TrackList* );
+            void mutate_population( Population& );
 
-            // internal mathematical state data
-            double m_satisfactionThreshold;
-            double m_qualityFactor;
-            int    m_maxCoolingIterations;
-            int    m_maxMutationIterations;
-            int    m_maxSwapIterations;
+            Meta::TrackList* crossover( Meta::TrackList*, Meta::TrackList* ) const;
+            static bool pop_comp( double, double );
+            Meta::TrackPtr random_track_from_domain() const;
+            Meta::TrackList sample( Meta::TrackList, const int ) const;
 
-            double m_playlistEntropy;
-            double m_finalSatisfaction;
+            double rng_gaussian( const double, const double ) const;
+            quint32 rng_poisson( const double ) const;
+            double rng_uniform() const;
+
+            quint32 playlist_size() const;
+            bool select( const double ) const;
+
+            // convenience function
+            void dump_population( const Population& ) const;
+
+            // internal mathematical parameters
+            quint32 m_maxGenerations;
+            quint32 m_populationSize;
+            quint32 m_minPlaylistSize;
+            quint32 m_suggestedPlaylistSize;
+            quint32 m_maxPlaylistSize;
     };
 } // namespace APG
 
