@@ -17,10 +17,10 @@
 #ifndef PLAYLISTFILEPROVIDER_H
 #define PLAYLISTFILEPROVIDER_H
 
-#include "core-impl/playlists/providers/user/UserPlaylistProvider.h"
 #include "core/playlists/PlaylistFormat.h"
-#include "core-impl/playlists/types/file/PlaylistFileSupport.h"
 #include "core/playlists/PlaylistProvider.h"
+#include "core-impl/playlists/providers/user/UserPlaylistProvider.h"
+#include "core-impl/playlists/types/file/PlaylistFileSupport.h"
 
 #include <kicon.h>
 
@@ -28,9 +28,9 @@ class KConfigGroup;
 class KUrl;
 
 class QAction;
+class QTimer;
 
 namespace Playlists {
-
 /**
     @author Bart Cerneels <bart.cerneels@kde.org>
 */
@@ -50,12 +50,12 @@ class PlaylistFileProvider : public Playlists::UserPlaylistProvider
         virtual int playlistCount() const;
         virtual Playlists::PlaylistList playlists();
 
-        virtual QList<QAction *> playlistActions( Playlists::PlaylistPtr playlist );
-        virtual QList<QAction *> trackActions( Playlists::PlaylistPtr playlist,
-                                                  int trackIndex );
-
-        virtual bool canSavePlaylists() { return true; }
-
+        /**
+        * Returns a Playlists::PlaylistPtr to the new playlist, NULL if something failed.
+        * @param tracks Tracks being added to that new playlist.
+        * @param name File name of the new playlist. If no extension is being given we
+        *             default to xspf. '/' and '\' are being replaced by '-'.
+        */
         virtual Playlists::PlaylistPtr save( const Meta::TrackList &tracks,
                                              const QString &name = QString() );
 
@@ -65,25 +65,27 @@ class PlaylistFileProvider : public Playlists::UserPlaylistProvider
         virtual void rename( Playlists::PlaylistPtr playlist, const QString &newName );
         virtual bool deletePlaylists( Playlists::PlaylistList playlistList );
 
-        virtual void loadPlaylists();
+        /* PlaylistFileProvider methods */
+        /** Schedules a PlaylistFile to be saved on the next iteration of the mainloop.
+          * Each playlist will be scheduled and saved only once.
+          */
+        void saveLater( Playlists::PlaylistFilePtr playlist );
 
     private slots:
-        void slotDelete();
-        void slotRename();
-        void slotRemove();
+        void loadPlaylists();
+        void slotSaveLater();
 
     private:
         bool deletePlaylistFiles( Playlists::PlaylistFileList playlistFiles );
         KConfigGroup loadedPlaylistsConfig() const;
 
         bool m_playlistsLoaded;
+        QList<KUrl> m_urlsToLoad;
         Playlists::PlaylistFileList m_playlists;
-        Playlists::PlaylistFormat m_defaultFormat;
         QMultiMap<QString, Playlists::PlaylistPtr> m_groupMap;
 
-        QAction *m_renameAction;
-        QAction *m_deleteAction;
-        QAction *m_removeTrackAction;
+        QTimer *m_saveLaterTimer;
+        QList<Playlists::PlaylistFilePtr> m_saveLaterPlaylists;
 };
 
 } //namespace Playlists
