@@ -16,10 +16,11 @@
 
 #include "SqlPlaylist.h"
 
-#include "core-impl/collections/support/CollectionManager.h"
-#include "core/support/Debug.h"
-#include "core-impl/meta/stream/Stream.h"
 #include "core/collections/support/SqlStorage.h"
+#include "core/support/Debug.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "core-impl/meta/proxy/MetaProxy.h"
+#include "core-impl/meta/stream/Stream.h"
 #include "core-impl/meta/timecode/TimecodeMeta.h"
 #include "playlistmanager/PlaylistManager.h"
 #include "playlistmanager/sql/SqlPlaylistGroup.h"
@@ -62,12 +63,6 @@ SqlPlaylist::~SqlPlaylist()
 {
 }
 
-SqlPlaylistGroupPtr
-SqlPlaylist::parent() const
-{
-    return m_parent;
-}
-
 KUrl
 SqlPlaylist::uidUrl() const
 {
@@ -210,7 +205,7 @@ int
 SqlPlaylist::trackCount() const
 {
     if( m_tracksLoaded )
-      return m_tracks.count();
+        return m_tracks.count();
 
     QString query = "SELECT COUNT(id) FROM playlist_tracks WHERE playlist_id=%1;";
 
@@ -225,14 +220,16 @@ SqlPlaylist::trackCount() const
         return -1;
     }
     int trackCount = results.first().toInt();
-    debug() << trackCount;
     return trackCount;
 }
 
 Meta::TrackList
 SqlPlaylist::tracks()
 {
-    //only returns the already loaded tracks.
+    //If you do not load before, m_tracks
+    //can be empty before usage.
+    triggerTrackLoad();
+
     return m_tracks;
 }
 
@@ -246,9 +243,14 @@ SqlPlaylist::triggerTrackLoad()
 void
 SqlPlaylist::addTrack( Meta::TrackPtr track, int position )
 {
-    int insertAt = (position == -1) ? m_tracks.count() : position;
-    subscribeTo( track ); //keep track of metadata changes.
-    m_tracks.insert( insertAt, track );
+    if( !m_tracksLoaded )
+        loadTracks();
+
+    if( position < 0 )
+        position = m_tracks.count();
+    else
+        position = qMin( position, m_tracks.count() );
+    m_tracks.insert( position, track );
     saveToDb( true );
     notifyObserversTrackAdded( track, position );
 }
@@ -256,28 +258,16 @@ SqlPlaylist::addTrack( Meta::TrackPtr track, int position )
 void
 SqlPlaylist::removeTrack( int position )
 {
+    if( !m_tracksLoaded )
+        loadTracks();
+
     if( position < 0 || position >= m_tracks.size() )
         return;
     Meta::TrackPtr track = m_tracks.takeAt( position );
-    unsubscribeFrom( track );
     saveToDb( true );
     notifyObserversTrackRemoved( position );
 }
 
-void
-SqlPlaylist::metadataChanged( Meta::TrackPtr track )
-{
-    //When AFT detects a moved file it will update the track and make it signal it's observers.
-    if( !m_tracks.contains( track ) )
-    {
-        error() << "Got a metadataChanged for a track that is not in the playlist.";
-        return;
-    }
-
-    //force update of tracks in database
-    saveToDb();
-}
-
 void
 SqlPlaylist::loadTracks()
 {
@@ -294,43 +284,13 @@ SqlPlaylist::loadTracks()
         QStringList row = result.mid( i*7, 7 );
         KUrl url = KUrl( row[2] );
 
-        Meta::TrackPtr trackPtr = CollectionManager::instance()->trackForUrl( url );
+        MetaProxy::Track *proxyTrack = new MetaProxy::Track( url );
 
-        if( trackPtr )
-        {
-            if( typeid( * trackPtr.data() ) == typeid( MetaStream::Track ) )
-            {
-
-                debug() << "got stream from trackForUrl, setting album to " << row[4];
-
-                MetaStream::Track * streamTrack =
-                        dynamic_cast<MetaStream::Track *> ( trackPtr.data() );
-                
-                if( streamTrack )
-                {
-                    streamTrack->setTitle( row[3] );
-                    streamTrack->setAlbum( row[4] );
-                    streamTrack->setArtist( row[5] );
-                }
-            }
-            else if( typeid( * trackPtr.data() ) == typeid( Meta::TimecodeTrack ) )
-            {
-                Meta::TimecodeTrack * timecodeTrack =
-                        dynamic_cast<Meta::TimecodeTrack *> ( trackPtr.data() );
-
-                if( timecodeTrack )
-                {
-                    timecodeTrack->beginMetaDataUpdate();
-                    timecodeTrack->setTitle( row[3] );
-                    timecodeTrack->setAlbum( row[4] );
-                    timecodeTrack->setArtist( row[5] );
-                    timecodeTrack->endMetaDataUpdate();
-                }
-            }
-
-            subscribeTo( trackPtr );
-            m_tracks << trackPtr;
-        }
+        proxyTrack->setName( row[3] );
+        proxyTrack->setAlbum( row[4] );
+        proxyTrack->setArtist( row[5] );
+        Meta::TrackPtr trackPtr = Meta::TrackPtr( proxyTrack );
+        m_tracks << trackPtr;
     }
 
     m_tracksLoaded = true;
@@ -355,11 +315,4 @@ SqlPlaylist::removeFromDb()
     CollectionManager::instance()->sqlStorage()->query( query );
 }
 
-int
-SqlPlaylist::id()
-{
-    return m_dbId;
-}
-
 } //namespace Playlists
-
