@@ -16,40 +16,42 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
  ****************************************************************************************/
 
+#define DEBUG_PREFIX "AmpacheServiceQueryMaker"
+
 #include "AmpacheServiceQueryMaker.h"
 
-#include "Amarok.h"
-#include "Debug.h"
 #include "AmpacheMeta.h"
-#include "collection/support/MemoryMatcher.h"
-
-#include <threadweaver/Job.h>
-#include <threadweaver/ThreadWeaver.h>
+#include "core/support/Amarok.h"
+#include "core/support/Debug.h"
+#include "core-impl/collections/support/MemoryMatcher.h"
 
 #include <QDomDocument>
 
-using namespace Meta;
+using namespace Collections;
 
 struct AmpacheServiceQueryMaker::Private
 {
     enum QueryType { NONE, TRACK, ARTIST, ALBUM, COMPOSER, YEAR, GENRE, CUSTOM };
     QueryType type;
     int maxsize;
-    bool returnDataPtrs;
+    QHash<QLatin1String, KUrl> urls;
 };
 
 AmpacheServiceQueryMaker::AmpacheServiceQueryMaker( AmpacheServiceCollection * collection, const QString &server, const QString &sessionId  )
     : DynamicServiceQueryMaker()
     , m_collection( collection )
-    , m_storedTransferJob( 0 )
     , d( new Private )
     , m_server( server )
     , m_sessionId( sessionId )
     , m_dateFilter( -1 )
 {
-    DEBUG_BLOCK
-    m_collection = collection;
-    reset();
+    // DEBUG_BLOCK
+
+    d->type = Private::NONE;
+    d->maxsize = 0;
+    d->urls.clear();
+    m_dateFilter = 0;
+    //m_lastArtistFilter = QString(); this one really should survive a reset....
 }
 
 AmpacheServiceQueryMaker::~AmpacheServiceQueryMaker()
@@ -57,34 +59,12 @@ AmpacheServiceQueryMaker::~AmpacheServiceQueryMaker()
     delete d;
 }
 
-QueryMaker *
-AmpacheServiceQueryMaker::reset()
-{
-    d->type = Private::NONE;
-    d->maxsize = 0;
-    d->returnDataPtrs = false;
-    m_parentArtistId.clear();
-    m_parentAlbumId.clear();
-    m_artistFilter.clear();
-    m_dateFilter = 0;
-    //m_lastArtistFilter = QString(); this one really should survive a reset....
-
-    return this;
-}
-
-QueryMaker*
-AmpacheServiceQueryMaker::setReturnResultAsDataPtrs( bool resultAsDataPtrs )
-{
-    d->returnDataPtrs = resultAsDataPtrs;
-    return this;
-}
-
 void
 AmpacheServiceQueryMaker::run()
 {
     DEBUG_BLOCK
 
-    if( m_storedTransferJob != 0 )
+    if( !d->urls.isEmpty() )
         return;
 
     //naive implementation, fix this
@@ -118,6 +98,7 @@ AmpacheServiceQueryMaker::setQueryType( QueryType type )
     switch( type ) {
 
     case QueryMaker::Artist:
+    case QueryMaker::AlbumArtist:
         d->type = Private::ARTIST;
         return this;
 
@@ -133,6 +114,7 @@ AmpacheServiceQueryMaker::setQueryType( QueryType type )
     case QueryMaker::Composer:
     case QueryMaker::Year:
     case QueryMaker::Custom:
+    case QueryMaker::Label:
     case QueryMaker::None:
         //TODO: Implement.
         return this;
@@ -142,12 +124,12 @@ AmpacheServiceQueryMaker::setQueryType( QueryType type )
 }
 
 QueryMaker *
-AmpacheServiceQueryMaker::addMatch( const ArtistPtr & artist )
+AmpacheServiceQueryMaker::addMatch( const Meta::ArtistPtr & artist )
 {
     DEBUG_BLOCK
     if( m_parentAlbumId.isEmpty() )
     {
-        const ServiceArtist * serviceArtist = dynamic_cast< const ServiceArtist * >( artist.data() );
+        const Meta::ServiceArtist * serviceArtist = dynamic_cast< const Meta::ServiceArtist * >( artist.data() );
         if( serviceArtist )
         {
             m_parentArtistId = QString::number( serviceArtist->id() );
@@ -156,7 +138,7 @@ AmpacheServiceQueryMaker::addMatch( const ArtistPtr & artist )
         {
             if( m_collection->artistMap().contains( artist->name() ) )
             {
-                serviceArtist = static_cast< const ServiceArtist* >( m_collection->artistMap().value( artist->name() ).data() );
+                serviceArtist = static_cast< const Meta::ServiceArtist* >( m_collection->artistMap().value( artist->name() ).data() );
                 m_parentArtistId = QString::number( serviceArtist->id() );
             }
             else
@@ -173,7 +155,7 @@ QueryMaker *
 AmpacheServiceQueryMaker::addMatch( const Meta::AlbumPtr & album )
 {
     DEBUG_BLOCK
-    const ServiceAlbum * serviceAlbum = dynamic_cast< const ServiceAlbum * >( album.data() );
+    const Meta::ServiceAlbum * serviceAlbum = dynamic_cast< const Meta::ServiceAlbum * >( album.data() );
     if( serviceAlbum )
     {
         m_parentAlbumId = QString::number( serviceAlbum->id() );
@@ -182,9 +164,9 @@ AmpacheServiceQueryMaker::addMatch( const Meta::AlbumPtr & album )
     }
     else
     {
-        if( m_collection->albumMap().contains( album->name() ) )
+        if( m_collection->albumMap().contains( album ) )  // compares albums by value
         {
-            serviceAlbum = static_cast< const ServiceAlbum* >( m_collection->albumMap().value( album->name() ).data() );
+            serviceAlbum = static_cast< const Meta::ServiceAlbum* >( m_collection->albumMap().value( album ).data() );
             m_parentAlbumId = QString::number( serviceAlbum->id() );
         }
         else
@@ -196,48 +178,6 @@ AmpacheServiceQueryMaker::addMatch( const Meta::AlbumPtr & album )
     return this;
 }
 
-template<class PointerType, class ListType>
-void AmpacheServiceQueryMaker::emitProperResult( const ListType& list )
-{
-    if ( d->returnDataPtrs ) {
-        DataList data;
-        foreach( PointerType p, list )
-            data << DataPtr::staticCast( p );
-
-        emit newResultReady( m_collection->collectionId(), data );
-    }
-    else
-        emit newResultReady( m_collection->collectionId(), list );
-}
-
-void AmpacheServiceQueryMaker::handleResult()
-{
-    DEBUG_BLOCK
-}
-
-void AmpacheServiceQueryMaker::handleResult( const ArtistList & artists )
-{
-    DEBUG_BLOCK
-
-    emitProperResult<ArtistPtr, ArtistList>( artists );
-}
-
-void
-AmpacheServiceQueryMaker::handleResult( const AlbumList &albums )
-{
-    DEBUG_BLOCK
-
-    emitProperResult<AlbumPtr, AlbumList>( albums );
-}
-
-void
-AmpacheServiceQueryMaker::handleResult( const TrackList & tracks )
-{
-    DEBUG_BLOCK
-
-    emitProperResult<TrackPtr, TrackList>( tracks );
-}
-
 void
 AmpacheServiceQueryMaker::fetchArtists()
 {
@@ -246,7 +186,7 @@ AmpacheServiceQueryMaker::fetchArtists()
     //this stuff causes crashes and "whiteouts" and will need to change anyway if the Ampache API is updated to
     // allow multilevel filtering. Hence it is commented out but left in for future reference
     /*if ( ( m_collection->artistMap().values().count() != 0 ) && ( m_artistFilter == m_lastArtistFilter ) ) {
-        handleResult( m_collection->artistMap().values() );
+        emit newResultReady( m_collection->artistMap().values() );
         debug() << "no need to fetch artists again! ";
         debug() << "    filter: " << m_artistFilter;
         debug() << "    last filter: " << m_lastArtistFilter;
@@ -273,10 +213,9 @@ AmpacheServiceQueryMaker::fetchArtists()
         request.addQueryItem( "limit", QString::number( d->maxsize ) ); // set to 0 in reset() so fine to use uncondiationally
         debug() << "Artist url: " << request.url();
 
-
-        m_storedTransferJob =  KIO::storedGet(  request, KIO::NoReload, KIO::HideProgressInfo );
-        connect( m_storedTransferJob, SIGNAL( result( KJob * ) )
-            , this, SLOT( artistDownloadComplete( KJob *) ) );
+        d->urls[QLatin1String("artists")] = request;
+        The::networkAccessManager()->getData( request, this,
+             SLOT(artistDownloadComplete(KUrl,QByteArray,NetworkAccessManagerProxy::Error)) );
     //}
 
     m_lastArtistFilter = m_artistFilter;
@@ -287,7 +226,7 @@ AmpacheServiceQueryMaker::fetchAlbums()
 {
     DEBUG_BLOCK
 
-    AlbumList albums;
+    Meta::AlbumList albums;
 
     if( !m_parentArtistId.isEmpty() )
     {
@@ -296,7 +235,7 @@ AmpacheServiceQueryMaker::fetchAlbums()
 
     if ( albums.count() > 0 )
     {
-        handleResult( albums );
+        emit newResultReady( albums );
         emit queryDone();
     }
     else
@@ -318,9 +257,9 @@ AmpacheServiceQueryMaker::fetchAlbums()
         request.addQueryItem( "limit", QString::number( d->maxsize ) ); // set to 0 in reset() so fine to use uncondiationally
         debug() << "request url: " << request.url();
 
-        m_storedTransferJob =  KIO::storedGet(  request, KIO::NoReload, KIO::HideProgressInfo );
-        connect( m_storedTransferJob, SIGNAL( result( KJob * ) )
-            , this, SLOT( albumDownloadComplete( KJob *) ) );
+        d->urls[QLatin1String("albums")] = request;
+        The::networkAccessManager()->getData( request, this,
+             SLOT(albumDownloadComplete(KUrl,QByteArray,NetworkAccessManagerProxy::Error)) );
     }
 }
 
@@ -329,7 +268,7 @@ AmpacheServiceQueryMaker::fetchTracks()
 {
     DEBUG_BLOCK
 
-    TrackList tracks;
+    Meta::TrackList tracks;
 
     //debug() << "parent album id: " << m_parentAlbumId;
 
@@ -346,7 +285,7 @@ AmpacheServiceQueryMaker::fetchTracks()
 
     if( tracks.count() > 0 )
     {
-        handleResult( tracks );
+        emit newResultReady( tracks );
         emit queryDone();
     }
     else
@@ -376,46 +315,45 @@ AmpacheServiceQueryMaker::fetchTracks()
 
         request.addQueryItem( "limit", QString::number( d->maxsize ) );// set to 0 in reset() so fine to use uncondiationally
 
-        m_storedTransferJob =  KIO::storedGet(  request, KIO::NoReload, KIO::HideProgressInfo );
-        connect( m_storedTransferJob, SIGNAL( result( KJob * ) )
-            , this, SLOT( trackDownloadComplete( KJob *) ) );
+        d->urls[QLatin1String("tracks")] = request;
+        The::networkAccessManager()->getData( request, this,
+             SLOT(trackDownloadComplete(KUrl,QByteArray,NetworkAccessManagerProxy::Error)) );
     }
 }
 
 void
-AmpacheServiceQueryMaker::artistDownloadComplete( KJob * job )
+AmpacheServiceQueryMaker::artistDownloadComplete( const KUrl &url, QByteArray data, NetworkAccessManagerProxy::Error e )
 {
-    DEBUG_BLOCK
+    if( d->urls.value(QLatin1String("artists")) != url )
+        return;
 
-    if( job->error() )
+    d->urls.remove( QLatin1String("artists") );
+    if( e.code != QNetworkReply::NoError )
     {
-        error() << job->error();
-        m_storedTransferJob->deleteLater();
+        debug() << "Artist download error:" << e.description;
         return;
     }
 
-    ArtistList artists;
+    // DEBUG_BLOCK
+
+    Meta::ArtistList artists;
 
      //so lets figure out what we got here:
     QDomDocument doc( "reply" );
-    doc.setContent( m_storedTransferJob->data() );
+    doc.setContent( data );
     QDomElement root = doc.firstChildElement( "root" );
 
     // Is this an error, if so we need to 'un-ready' the service and re-authenticate before contiuning
-    QDomElement error = root.firstChildElement( "error" );
+    QDomElement domError = root.firstChildElement( "error" );
 
-    if ( !error.isNull() )
+    if ( !domError.isNull() )
     {
-        debug () << "Error getting Artist List" << error.text();
+        debug () << "Error getting Artist List" << domError.text();
         AmpacheService *m_parentService = dynamic_cast< AmpacheService * >( m_collection->service() );
         if ( m_parentService == 0 )
-        {
-                return;
-        }
+            return;
         else
-        {
             m_parentService->reauthenticate();
-        }
     }
 
     QDomNode n = root.firstChild();
@@ -426,7 +364,7 @@ AmpacheServiceQueryMaker::artistDownloadComplete( KJob * job )
         //    break;
 
         QDomElement element = n.firstChildElement( "name" );
-        ServiceArtist * artist = new AmpacheArtist( element.text(), m_collection->service() );
+        Meta::ServiceArtist * artist = new Meta::AmpacheArtist( element.text(), m_collection->service() );
 
         int artistId = e.attribute( "id", "0").toInt();
 
@@ -434,7 +372,7 @@ AmpacheServiceQueryMaker::artistDownloadComplete( KJob * job )
 
         artist->setId( artistId );
 
-        ArtistPtr artistPtr( artist );
+        Meta::ArtistPtr artistPtr( artist );
 
         artists.push_back( artistPtr );
 
@@ -445,50 +383,43 @@ AmpacheServiceQueryMaker::artistDownloadComplete( KJob * job )
         n = n.nextSibling();
     }
 
-   m_storedTransferJob->deleteLater();
-
-   handleResult( artists );
-   emit queryDone();
+    emit newResultReady( artists );
+    emit queryDone();
 }
 
 void
-AmpacheServiceQueryMaker::albumDownloadComplete( KJob * job )
+AmpacheServiceQueryMaker::albumDownloadComplete( const KUrl &url, QByteArray data, NetworkAccessManagerProxy::Error e )
 {
-    DEBUG_BLOCK
+    if( d->urls.value(QLatin1String("albums")) != url )
+        return;
 
-    if( job->error() )
+    d->urls.remove( QLatin1String("albums") );
+    if( e.code != QNetworkReply::NoError )
     {
-        error() << job->error();
-        m_storedTransferJob->deleteLater();
+        debug() << "Album download error:" << e.description;
         return;
     }
 
-    //debug() << "Received response: " << m_storedTransferJob->data();
+    // DEBUG_BLOCK
 
-    AlbumList albums;
-
-    //debug() << "received artists: " <<  m_storedTransferJob->data();
+    Meta::AlbumList albums;
 
      //so lets figure out what we got here:
     QDomDocument doc( "reply" );
-    doc.setContent( m_storedTransferJob->data() );
+    doc.setContent( data );
     QDomElement root = doc.firstChildElement( "root" );
 
     // Is this an error, if so we need to 'un-ready' the service and re-authenticate before contiuning
-    QDomElement error = root.firstChildElement( "error" );
+    QDomElement domError = root.firstChildElement( "error" );
 
-    if ( !error.isNull() )
+    if( !domError.isNull() )
     {
-        debug () << "Error getting Album List" << error.text();
+        debug () << "Error getting Album List" << domError.text();
         AmpacheService *m_parentService = dynamic_cast< AmpacheService * >(m_collection->service());
         if ( m_parentService == 0 )
-        {
-                return;
-        }
+            return;
         else
-        {
             m_parentService->reauthenticate();
-        }
     }
 
     QDomNode n = root.firstChild();
@@ -506,7 +437,7 @@ AmpacheServiceQueryMaker::albumDownloadComplete( KJob * job )
 
         int albumId = e.attribute( "id", "0" ).toInt();
 
-        AmpacheAlbum * album = new AmpacheAlbum( title );
+        Meta::AmpacheAlbum * album = new Meta::AmpacheAlbum( title );
         album->setId( albumId );
 
 
@@ -515,7 +446,7 @@ AmpacheServiceQueryMaker::albumDownloadComplete( KJob * job )
         QString coverUrl = element.text();
         album->setCoverUrl( coverUrl );
 
-        AlbumPtr albumPtr( album );
+        Meta::AlbumPtr albumPtr( album );
 
         //debug() << "Adding album: " <<  title;
         //debug() << "   Id: " <<  albumId;
@@ -527,11 +458,11 @@ AmpacheServiceQueryMaker::albumDownloadComplete( KJob * job )
 
         element = n.firstChildElement( "artist" );
 
-        ArtistPtr artistPtr = m_collection->artistById( m_parentArtistId.toInt() );
+        Meta::ArtistPtr artistPtr = m_collection->artistById( m_parentArtistId.toInt() );
         if ( artistPtr.data() != 0 )
         {
            //debug() << "Found parent artist";
-           //ServiceArtist *artist = dynamic_cast< ServiceArtist * > ( artistPtr.data() );
+           //Meta::ServiceArtist *artist = dynamic_cast< Meta::ServiceArtist * > ( artistPtr.data() );
            album->setAlbumArtist( artistPtr );
         }
 
@@ -540,48 +471,43 @@ AmpacheServiceQueryMaker::albumDownloadComplete( KJob * job )
         n = n.nextSibling();
     }
 
-   m_storedTransferJob->deleteLater();
-
-   handleResult( albums );
-   emit queryDone();
+    emit newResultReady( albums );
+    emit queryDone();
 }
 
 void
-AmpacheServiceQueryMaker::trackDownloadComplete( KJob * job )
+AmpacheServiceQueryMaker::trackDownloadComplete( const KUrl &url, QByteArray data, NetworkAccessManagerProxy::Error e )
 {
-    DEBUG_BLOCK
+    if( d->urls.value(QLatin1String("tracks")) != url )
+        return;
 
-    if( job->error() )
+    d->urls.remove( QLatin1String("tracks") );
+    if( e.code != QNetworkReply::NoError )
     {
-        error() << job->error();
-        m_storedTransferJob->deleteLater();
+        debug() << "Track download error:" << e.description;
         return;
     }
 
-    //debug() << "Received response: " << m_storedTransferJob->data();
+    // DEBUG_BLOCK
 
-    TrackList tracks;
+    Meta::TrackList tracks;
 
      //so lets figure out what we got here:
     QDomDocument doc( "reply" );
-    doc.setContent( m_storedTransferJob->data() );
+    doc.setContent( data );
     QDomElement root = doc.firstChildElement( "root" );
 
     // Is this an error, if so we need to 'un-ready' the service and re-authenticate before contiuning
-    QDomElement error = root.firstChildElement( "error" );
+    QDomElement domError = root.firstChildElement( "error" );
 
-    if ( !error.isNull() )
+    if( !domError.isNull() )
     {
-        debug () << "Error getting Track Download " << error.text();
+        debug () << "Error getting Track Download " << domError.text();
         AmpacheService *m_parentService = dynamic_cast< AmpacheService * >( m_collection->service() );
         if ( m_parentService == 0 )
-        {
-                return;
-        }
+            return;
         else
-        {
             m_parentService->reauthenticate();
-        }
     }
 
     QDomNode n = root.firstChild();
@@ -598,8 +524,8 @@ AmpacheServiceQueryMaker::trackDownloadComplete( KJob * job )
         if ( title.isEmpty() )
             title = "Unknown";
         element = n.firstChildElement( "url" );
-        AmpacheTrack * track = new AmpacheTrack( title, m_collection->service() );
-        TrackPtr trackPtr( track );
+        Meta::AmpacheTrack * track = new Meta::AmpacheTrack( title, m_collection->service() );
+        Meta::TrackPtr trackPtr( track );
 
         //debug() << "Adding track: " <<  title;
 
@@ -622,32 +548,31 @@ AmpacheServiceQueryMaker::trackDownloadComplete( KJob * job )
         QDomElement artistElement = n.firstChildElement( "artist" );
         int artistId = artistElement.attribute( "id", "0").toInt();
 
-        ArtistPtr artistPtr = m_collection->artistById( artistId );
+        Meta::ArtistPtr artistPtr = m_collection->artistById( artistId );
         if ( artistPtr.data() != 0 )
         {
             //debug() << "Found parent artist " << artistPtr->name();
-           ServiceArtist *artist = dynamic_cast< ServiceArtist * > ( artistPtr.data() );
-           track->setArtist( artistPtr );
-           artist->addTrack( trackPtr );
+            Meta::ServiceArtist *artist = dynamic_cast< Meta::ServiceArtist * > ( artistPtr.data() );
+            track->setArtist( artistPtr );
+            artist->addTrack( trackPtr );
         }
 
-        AlbumPtr albumPtr = m_collection->albumById( albumId );
+        Meta::AlbumPtr albumPtr = m_collection->albumById( albumId );
         if ( albumPtr.data() != 0 )
         {
-           //debug() << "Found parent album " << albumPtr->name() ;
-           ServiceAlbum *album = dynamic_cast< ServiceAlbum * > ( albumPtr.data() );
-           track->setAlbumPtr( albumPtr );
-           album->addTrack( trackPtr );
+            //debug() << "Found parent album " << albumPtr->name() ;
+            Meta::ServiceAlbum *album = dynamic_cast< Meta::ServiceAlbum * > ( albumPtr.data() );
+            track->setAlbumPtr( albumPtr );
+            album->addTrack( trackPtr );
         }
 
         tracks.push_back( trackPtr );
 
         n = n.nextSibling();
-   }
-   m_storedTransferJob->deleteLater();
+    }
 
-   handleResult( tracks );
-   emit queryDone();
+    emit newResultReady( tracks );
+    emit queryDone();
 }
 
 QueryMaker *
@@ -659,7 +584,7 @@ AmpacheServiceQueryMaker::addFilter( qint64 value, const QString & filter, bool
 
     //debug() << "value: " << value;
     //for now, only accept artist filters
-    if ( value == valArtist )
+    if ( value == Meta::valArtist )
     {
         //debug() << "Filter: " << filter;
         m_artistFilter = filter;
@@ -672,7 +597,7 @@ AmpacheServiceQueryMaker::addNumberFilter( qint64 value, qint64 filter, QueryMak
 {
     DEBUG_BLOCK
 
-    if( value == valCreateDate && compare == QueryMaker::GreaterThan )
+    if( value == Meta::valCreateDate && compare == QueryMaker::GreaterThan )
     {
         debug() << "asking to filter based on added date";
         m_dateFilter = filter;
@@ -681,7 +606,6 @@ AmpacheServiceQueryMaker::addNumberFilter( qint64 value, qint64 filter, QueryMak
     return this;
 }
 
-
 int
 AmpacheServiceQueryMaker::validFilterMask()
 {
