@@ -15,8 +15,8 @@
  ****************************************************************************************/
 
 #include "SimilarArtistsAction.h"
-#include "playlist/PlaylistModelStack.h"
-#include "collection/CollectionManager.h"
+#include "playlist/PlaylistController.h"
+#include "core-impl/collections/support/CollectionManager.h"
 #include "SvgHandler.h"
 
 #include <KIcon>
