@@ -16,12 +16,12 @@
 
 #include "MagnatuneStore.h"
 
-#include "Amarok.h"
+#include "core/support/Amarok.h"
+#include "core/support/Components.h"
+#include "core/interfaces/Logger.h"
 #include "amarokurls/AmarokUrlHandler.h"
 #include "browsers/CollectionTreeItem.h"
 #include "browsers/SingleCollectionTreeItemModel.h"
-#include "statusbar/StatusBar.h"
-#include "statusbar/ProgressBar.h"
 #include "EngineController.h"
 #include "MagnatuneConfig.h"
 #include "MagnatuneDatabaseWorker.h"
@@ -29,9 +29,11 @@
 #include "browsers/InfoProxy.h"
 #include "MagnatuneUrlRunner.h"
 
+#include "ui_MagnatuneSignupDialogBase.h"
+
 #include "../ServiceSqlRegistry.h"
-#include "CollectionManager.h"
-#include "Debug.h"
+#include "core-impl/collections/support/CollectionManager.h"
+#include "core/support/Debug.h"
 #include "playlist/PlaylistModelStack.h"
 #include "widgets/SearchWidget.h"
 
@@ -42,18 +44,27 @@
 #include <KUrl>
 #include <threadweaver/ThreadWeaver.h>
 
-#include <QAction>
 #include <QDateTime>
 #include <QMenu>
+#include <QToolBar>
+#include <QToolButton>
 
 #include <typeinfo>
 
-AMAROK_EXPORT_PLUGIN( MagnatuneServiceFactory )
+AMAROK_EXPORT_SERVICE_PLUGIN( magnatunestore, MagnatuneServiceFactory )
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 // class MagnatuneServiceFactory
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
+MagnatuneServiceFactory::MagnatuneServiceFactory( QObject *parent, const QVariantList &args )
+    : ServiceFactory( parent, args )
+{
+    KPluginInfo pluginInfo( "amarok_service_magnatunestore.desktop", "services" );
+    pluginInfo.setConfig( config() );
+    m_info = pluginInfo;
+}
+
 void MagnatuneServiceFactory::init()
 {
     DEBUG_BLOCK
@@ -68,13 +79,6 @@ QString MagnatuneServiceFactory::name()
     return "Magnatune.com";
 }
 
-KPluginInfo MagnatuneServiceFactory::info()
-{
-    KPluginInfo pluginInfo( "amarok_service_magnatunestore.desktop", "services" );
-    pluginInfo.setConfig( config() );
-    return pluginInfo;
-}
-
 KConfigGroup MagnatuneServiceFactory::config()
 {
     return Amarok::config( "Service_Magnatune" );
@@ -86,19 +90,20 @@ KConfigGroup MagnatuneServiceFactory::config()
 
 MagnatuneStore::MagnatuneStore( MagnatuneServiceFactory* parent, const char *name )
         : ServiceBase( name, parent )
-        , m_purchaseHandler( 0 )
+        , m_downloadHandler( 0 )
         , m_redownloadHandler( 0 )
-        , m_purchaseInProgress( 0 )
+        , m_downloadInProgress( 0 )
         , m_currentAlbum( 0 )
         , m_streamType( MagnatuneMetaFactory::OGG )
         , m_magnatuneTimestamp( 0 )
         , m_registry( 0 )
+        , m_signupInfoWidget( 0 )
 {
     setObjectName(name);
     DEBUG_BLOCK
     //initTopPanel( );
 
-    setShortDescription( i18n( "\"Fair trade\" online music store." ) );
+    setShortDescription( i18n( "\"Fair trade\" online music store" ) );
     setIcon( KIcon( "view-services-magnatune-amarok" ) );
 
     // xgettext: no-c-format
@@ -127,7 +132,7 @@ MagnatuneStore::MagnatuneStore( MagnatuneServiceFactory* parent, const char *nam
 
     metaFactory->setStreamType( m_streamType );
     m_registry = new ServiceSqlRegistry( metaFactory );
-    m_collection = new MagnatuneSqlCollection( "magnatune", "Magnatune.com", metaFactory, m_registry );
+    m_collection = new Collections::MagnatuneSqlCollection( "magnatune", "Magnatune.com", metaFactory, m_registry );
     m_serviceready = true;
     CollectionManager::instance()->addUnmanagedCollection( m_collection, CollectionManager::CollectionDisabled );
     emit( ready() );
@@ -136,14 +141,15 @@ MagnatuneStore::MagnatuneStore( MagnatuneServiceFactory* parent, const char *nam
 MagnatuneStore::~MagnatuneStore()
 {
     CollectionManager::instance()->removeUnmanagedCollection( m_collection );
-    //hm, memory handling?
+    delete m_registry;
+    delete m_collection;
 }
 
 
-void MagnatuneStore::purchase( )
+void MagnatuneStore::download( )
 {
     DEBUG_BLOCK
-    if ( m_purchaseInProgress )
+    if ( m_downloadInProgress )
         return;
 
     if ( !m_polished )
@@ -151,49 +157,56 @@ void MagnatuneStore::purchase( )
 
     debug() << "here";
 
-    m_purchaseInProgress = true;
-    m_purchaseAlbumButton->setEnabled( false );
-
-    if ( !m_purchaseHandler )
+    //check if we need to start a download or show the signup dialog
+    if( !m_isMember || m_membershipType != MagnatuneConfig::DOWNLOAD )
     {
-        m_purchaseHandler = new MagnatunePurchaseHandler();
-        m_purchaseHandler->setParent( this );
-        connect( m_purchaseHandler, SIGNAL( purchaseCompleted( bool ) ), this, SLOT( purchaseCompleted( bool ) ) );
+        showSignupDialog();
+        return;
+    }
+
+    m_downloadInProgress = true;
+    m_downloadAlbumButton->setEnabled( false );
+
+    if ( !m_downloadHandler )
+    {
+        m_downloadHandler = new MagnatuneDownloadHandler();
+        m_downloadHandler->setParent( this );
+        connect( m_downloadHandler, SIGNAL( downloadCompleted( bool ) ), this, SLOT( downloadCompleted( bool ) ) );
     }
 
     if ( m_currentAlbum != 0 )
-        m_purchaseHandler->purchaseAlbum( m_currentAlbum );
+        m_downloadHandler->downloadAlbum( m_currentAlbum );
 }
 
 
-void MagnatuneStore::purchase( Meta::MagnatuneTrack * track )
+void MagnatuneStore::download( Meta::MagnatuneTrack * track )
 {
     Meta::MagnatuneAlbum * album = dynamic_cast<Meta::MagnatuneAlbum *>( track->album().data() );
     if ( album )
-        purchase( album );
+        download( album );
 }
 
-void MagnatuneStore::purchase( Meta::MagnatuneAlbum * album )
+void MagnatuneStore::download( Meta::MagnatuneAlbum * album )
 {
 
     DEBUG_BLOCK
-    if ( m_purchaseInProgress )
+    if ( m_downloadInProgress )
         return;
 
     if ( !m_polished )
         polish();
 
-    m_purchaseInProgress = true;
-    m_purchaseAlbumButton->setEnabled( false );
+    m_downloadInProgress = true;
+    m_downloadAlbumButton->setEnabled( false );
 
-    if ( !m_purchaseHandler )
+    if ( !m_downloadHandler )
     {
-        m_purchaseHandler = new MagnatunePurchaseHandler();
-        m_purchaseHandler->setParent( this );
-        connect( m_purchaseHandler, SIGNAL( purchaseCompleted( bool ) ), this, SLOT( purchaseCompleted( bool ) ) );
+        m_downloadHandler = new MagnatuneDownloadHandler();
+        m_downloadHandler->setParent( this );
+        connect( m_downloadHandler, SIGNAL( downloadCompleted( bool ) ), this, SLOT( downloadCompleted( bool ) ) );
     }
 
-    m_purchaseHandler->purchaseAlbum( album );
+    m_downloadHandler->downloadAlbum( album );
 }
 
 
@@ -250,20 +263,28 @@ void MagnatuneStore::initBottomPanel()
 {
     //m_bottomPanel->setMaximumHeight( 24 );
 
-    m_purchaseAlbumButton = new QPushButton;
-    m_purchaseAlbumButton->setParent( m_bottomPanel );
+    m_downloadAlbumButton = new QPushButton;
+    m_downloadAlbumButton->setParent( m_bottomPanel );
 
     MagnatuneConfig config;
     if ( config.isMember() && config.membershipType() == MagnatuneConfig::DOWNLOAD )
-        m_purchaseAlbumButton->setText( i18n( "Download Album" ) );
+    {
+        m_downloadAlbumButton->setText( i18n( "Download Album" ) );
+        m_downloadAlbumButton->setEnabled( false );
+    }
+    else if ( config.isMember() )
+        m_downloadAlbumButton->hide();
     else
-        m_purchaseAlbumButton->setText( i18n( "Purchase Album" ) );
+    {
+        m_downloadAlbumButton->setText( i18n( "Signup" ) );
+        m_downloadAlbumButton->setEnabled( true );
+    }
 
-    m_purchaseAlbumButton->setObjectName( "purchaseButton" );
-    m_purchaseAlbumButton->setIcon( KIcon( "download-amarok" ) );
-    m_purchaseAlbumButton->setEnabled( false );
+    m_downloadAlbumButton->setObjectName( "downloadButton" );
+    m_downloadAlbumButton->setIcon( KIcon( "download-amarok" ) );
+    
 
-    connect( m_purchaseAlbumButton, SIGNAL( clicked() ) , this, SLOT( purchase() ) );
+    connect( m_downloadAlbumButton, SIGNAL( clicked() ) , this, SLOT( download() ) );
 }
 
 
@@ -294,9 +315,7 @@ bool MagnatuneStore::updateMagnatuneList()
     m_tempFileName = tempFile.fileName();
 
     m_listDownloadJob = KIO::file_copy( KUrl( "http://magnatune.com/info/album_info_xml.bz2" ),  KUrl( m_tempFileName ), 0700 , KIO::HideProgressInfo | KIO::Overwrite );
-    The::statusBar()->newProgressOperation( m_listDownloadJob, i18n( "Downloading Magnatune.com Database" ) )
-    ->setAbortSlot( this, SLOT( listDownloadCancelled() ) );
-
+    Amarok::Components::logger()->newProgressOperation( m_listDownloadJob, i18n( "Downloading Magnatune.com database..." ), this, SLOT( listDownloadCancelled() ) );
 
     connect( m_listDownloadJob, SIGNAL( result( KJob * ) ),
             this, SLOT( listDownloadComplete( KJob * ) ) );
@@ -325,8 +344,7 @@ void MagnatuneStore::listDownloadComplete( KJob * downLoadJob )
     }
 
 
-    The::statusBar()->shortMessage( i18n( "Updating the local Magnatune database."  ) );
-    debug() << "MagnatuneStore: create xml parser";
+    Amarok::Components::logger()->shortMessage( i18n( "Updating the local Magnatune database."  ) );
     MagnatuneXmlParser * parser = new MagnatuneXmlParser( m_tempFileName );
     parser->setDbHandler( new MagnatuneDatabaseHandler() );
     connect( parser, SIGNAL( doneParsing() ), SLOT( doneParsing() ) );
@@ -339,7 +357,6 @@ void MagnatuneStore::listDownloadCancelled( )
 {
     DEBUG_BLOCK
 
-    //The::statusBar()->endProgressOperation( m_listDownloadJob );
     m_listDownloadJob->kill();
     m_listDownloadJob = 0;
     debug() << "Aborted xml download";
@@ -378,13 +395,13 @@ void MagnatuneStore::processRedownload( )
 }
 
 
-void MagnatuneStore::purchaseCompleted( bool )
+void MagnatuneStore::downloadCompleted( bool )
 {
-    delete m_purchaseHandler;
-    m_purchaseHandler = 0;
+    delete m_downloadHandler;
+    m_downloadHandler = 0;
 
-    m_purchaseAlbumButton->setEnabled( true );
-    m_purchaseInProgress = false;
+    m_downloadAlbumButton->setEnabled( true );
+    m_downloadInProgress = false;
 
     debug() << "Purchase operation complete";
 
@@ -396,6 +413,10 @@ void MagnatuneStore::itemSelected( CollectionTreeItem * selectedItem )
 {
     DEBUG_BLOCK
 
+    //only care if the user has a download membership
+    if( !m_isMember || m_membershipType != MagnatuneConfig::DOWNLOAD )
+        return;
+
     //we only enable the purchase button if there is only one item selected and it happens to
     //be an album or a track
     Meta::DataPtr dataPtr = selectedItem->data();
@@ -405,18 +426,18 @@ void MagnatuneStore::itemSelected( CollectionTreeItem * selectedItem )
         debug() << "is right type (track)";
         Meta::MagnatuneTrack * track = static_cast<Meta::MagnatuneTrack *> ( dataPtr.data() );
         m_currentAlbum = static_cast<Meta::MagnatuneAlbum *> ( track->album().data() );
-        m_purchaseAlbumButton->setEnabled( true );
+        m_downloadAlbumButton->setEnabled( true );
 
     } else if ( typeid( * dataPtr.data() ) == typeid( Meta::MagnatuneAlbum ) ) {
 
         m_currentAlbum = static_cast<Meta::MagnatuneAlbum *> ( dataPtr.data() );
         debug() << "is right type (album) named " << m_currentAlbum->name();
 
-        m_purchaseAlbumButton->setEnabled( true );
+        m_downloadAlbumButton->setEnabled( true );
     } else {
 
         debug() << "is wrong type";
-        m_purchaseAlbumButton->setEnabled( false );
+        m_downloadAlbumButton->setEnabled( false );
 
     }
 }
@@ -442,7 +463,7 @@ void MagnatuneStore::polish()
         initTopPanel( );
         initBottomPanel();
 
-        QList<int> levels;
+        QList<CategoryId::CatMenuId> levels;
         levels << CategoryId::Genre << CategoryId::Artist << CategoryId::Album;
 
         m_magnatuneInfoParser = new MagnatuneInfoParser();
@@ -459,10 +480,10 @@ void MagnatuneStore::polish()
         connect( runner, SIGNAL( showFavorites() ), this, SLOT( showFavoritesPage() ) );
         connect( runner, SIGNAL( showHome() ), this, SLOT( showHomePage() ) );
         connect( runner, SIGNAL( showRecommendations() ), this, SLOT( showRecommendationsPage() ) );
-        connect( runner, SIGNAL( buyOrDownload( const QString & ) ), this, SLOT( purchase( const QString & ) ) );
+        connect( runner, SIGNAL( buyOrDownload( const QString & ) ), this, SLOT( download( const QString & ) ) );
         connect( runner, SIGNAL( removeFromFavorites( const QString & ) ), this, SLOT( removeFromFavorites( const QString & ) ) );
 
-        The::amarokUrlHandler()->registerRunner( runner, "service_magnatune" );
+        The::amarokUrlHandler()->registerRunner( runner, runner->command() );
     }
 
     const KUrl url( KStandardDirs::locate( "data", "amarok/data/" ) );
@@ -510,7 +531,7 @@ void MagnatuneStore::moodMapReady(QMap< QString, int > map)
         QVariantMap action;
         action["component"]  = "/ServicePluginManager";
         action["function"] = "sendMessage";
-        action["arg1"] = QString( "Magnatune.com").arg( escapedKey );
+        action["arg1"] = QString( "Magnatune.com");
         action["arg2"] = QString( "addMoodyTracks %1 10").arg( escapedKey );
 
         dbusActions[key] = action;
@@ -532,13 +553,13 @@ void MagnatuneStore::setStreamType( int type )
 }
 
 
-void MagnatuneStore::purchaseCurrentTrackAlbum()
+void MagnatuneStore::downloadCurrentTrackAlbum()
 {
     //get current track
     Meta::TrackPtr track = The::engineController()->currentTrack();
 
     //check if this is indeed a magnatune track
-    Meta::SourceInfoCapability *sic = track->create<Meta::SourceInfoCapability>();
+    Capabilities::SourceInfoCapability *sic = track->create<Capabilities::SourceInfoCapability>();
     if( sic )
     {
         //is the source defined
@@ -565,14 +586,14 @@ void MagnatuneStore::purchaseCurrentTrackAlbum()
     if ( !magnatuneAlbum )
         return;
 
-    if ( !m_purchaseHandler )
+    if ( !m_downloadHandler )
     {
-        m_purchaseHandler = new MagnatunePurchaseHandler();
-        m_purchaseHandler->setParent( this );
-        connect( m_purchaseHandler, SIGNAL( purchaseCompleted( bool ) ), this, SLOT( purchaseCompleted( bool ) ) );
+        m_downloadHandler = new MagnatuneDownloadHandler();
+        m_downloadHandler->setParent( this );
+        connect( m_downloadHandler, SIGNAL( downloadCompleted( bool ) ), this, SLOT( downloadCompleted( bool ) ) );
     }
 
-    m_purchaseHandler->purchaseAlbum( magnatuneAlbum );
+    m_downloadHandler->downloadAlbum( magnatuneAlbum );
 }
 
 
@@ -677,13 +698,13 @@ void MagnatuneStore::showRecommendationsPage()
     m_magnatuneInfoParser->getRecommendationsPage();
 }
 
-void MagnatuneStore::purchase( const QString &sku )
+void MagnatuneStore::download( const QString &sku )
 {
     DEBUG_BLOCK
     debug() << "sku: " << sku;
     MagnatuneDatabaseWorker * databaseWorker = new MagnatuneDatabaseWorker();
     databaseWorker->fetchAlbumBySku( sku, m_registry );
-    connect( databaseWorker, SIGNAL( gotAlbumBySku( Meta::MagnatuneAlbum * ) ), this, SLOT( purchase( Meta::MagnatuneAlbum * ) ) );
+    connect( databaseWorker, SIGNAL( gotAlbumBySku( Meta::MagnatuneAlbum * ) ), this, SLOT( download( Meta::MagnatuneAlbum * ) ) );
 
     ThreadWeaver::Weaver::instance()->enqueue( databaseWorker );
 }
@@ -729,12 +750,26 @@ void MagnatuneStore::favoritesResult( KJob* addToFavoritesJob )
 
     QString result = m_favoritesJob->data();
 
-    The::statusBar()->longMessage( result );
+    Amarok::Components::logger()->longMessage( result );
 
     //show the favorites page
     showFavoritesPage();
 }
 
+void
+MagnatuneStore::showSignupDialog()
+{
+
+    if ( m_signupInfoWidget== 0 )
+    {
+        m_signupInfoWidget = new QDialog;
+        Ui::SignupDialog ui;
+        ui.setupUi( m_signupInfoWidget );
+    }
+
+     m_signupInfoWidget->show();
+}
+
 
 
 #include "MagnatuneStore.moc"
