@@ -18,11 +18,10 @@
 #include "BookmarkPopup.h"
 #include "BookmarkTriangle.h"
 #include "SvgHandler.h"
-#include "Debug.h"
+#include "core/support/Debug.h"
 
 
 #include <KLocale>
-#include <KPixmapCache>
 
 #include <QPainter>
 #include <QHBoxLayout>
@@ -105,7 +104,7 @@ void BookmarkPopup::paintEvent ( QPaintEvent* event )
     p.drawPixmap ( m_deleteIconRect.x(), m_deleteIconRect.y(), m_deleteIcon.pixmap ( 16 ) );
 
     p.setOpacity ( 1 );
-    p.drawPixmap ( 5, 1, The::svgHandler()->renderSvg ( "bookmark", 6, 20, "bookmark" ) );
+    p.drawPixmap ( 5, 1, The::svgHandler()->renderSvg ( "bookmarks", 6, 20, "bookmarks" ) );
 
     p.setPen ( Qt::gray );
     rect = QRect ( 15, 3, m_width, m_lineHeight );
