@@ -29,8 +29,10 @@
 #include "SvgHandler.h"
 #include "amarokconfig.h"
 #include "core/meta/support/MetaUtility.h"
+#include "KNotificationBackend.h"
 
 #include <KApplication>
+#include <KDebug>
 #include <KIcon>
 #include <KLocale>
 #include <KWindowSystem>
@@ -61,9 +63,9 @@ OSDWidget::OSDWidget( QWidget *parent, const char *name )
         , m_y( MARGIN )
         , m_drawShadow( true )
         , m_rating( 0 )
-        , m_volume( 0 )
+        , m_volume( The::engineController()->volume() )
         , m_showVolume( false )
-        , m_fontScale( 1.15 )
+        , m_hideWhenFullscreenWindowIsActive( false )
 {
     Qt::WindowFlags flags;
     flags = Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint;
@@ -81,6 +83,8 @@ OSDWidget::OSDWidget( QWidget *parent, const char *name )
     setFocusPolicy( Qt::NoFocus );
     unsetColors();
 
+    setFont(QFont("sans-serif"));
+
     #ifdef Q_WS_X11
     KWindowSystem::setType( winId(), NET::Notification );
     #endif
@@ -90,8 +94,6 @@ OSDWidget::OSDWidget( QWidget *parent, const char *name )
 
     //or crashes, KWindowSystem bug I think, crashes in QWidget::icon()
     kapp->setTopWidget( this );
-
-    m_volume = The::engineController()->volume();
 }
 
 OSDWidget::~OSDWidget()
@@ -100,7 +102,7 @@ OSDWidget::~OSDWidget()
 }
 
 void
-OSDWidget::show( const QString &text, QImage newImage )
+OSDWidget::show( const QString &text, const QImage &newImage )
 {
     DEBUG_BLOCK
     m_showVolume = false;
@@ -114,14 +116,31 @@ OSDWidget::show( const QString &text, QImage newImage )
     else
         m_cover = Amarok::icon();
 
-    QFont f = QFont( "sans-serif" );
-    f.setPointSizeF( f.pointSizeF() * m_fontScale );
-    setFont( f );
-
     m_text = text;
     show();
 }
 
+void
+OSDWidget::show()
+{
+    if ( !isTemporaryDisabled() )
+        QWidget::show();
+}
+
+bool
+OSDWidget::isTemporaryDisabled()
+{
+    // Check if the OSD should not be shown,
+    // if a fullscreen window is focused.
+    if ( m_hideWhenFullscreenWindowIsActive )
+    {
+        return Amarok::KNotificationBackend::instance()->isFullscreenWindowActive();
+    }
+
+    return false;
+}
+
+
 void
 OSDWidget::ratingChanged( const QString& path, int rating )
 {
@@ -148,9 +167,7 @@ OSDWidget::volumeChanged( int volume )
 
     if ( isEnabled() )
     {
-        QString muteState = "";
         m_showVolume = true;
-
         m_text = i18n("Volume: %1% %2", m_volume, ( The::engineController()->isMuted() ? i18n("(muted)") : "" ) );
 
         show();
@@ -167,7 +184,6 @@ OSDWidget::setVisible( bool visible )
 
         const uint M = fontMetrics().width( 'x' );
 
-        const QRect oldGeometry = QRect( pos(), size() );
         const QRect newGeometry = determineMetrics( M );
 
         if( newGeometry.width() > 0 && newGeometry.height() > 0 )
@@ -299,8 +315,8 @@ OSDWidget::determineMetrics( const int M )
 void
 OSDWidget::paintEvent( QPaintEvent *e )
 {
-    int M = m_m;
-    QSize size = m_size;
+    const int& M = m_m;
+    const QSize& size = m_size;
 
     QPoint point;
     QRect rect( point, size );
@@ -425,6 +441,14 @@ OSDWidget::unsetColors()
     setPalette( newPal );
 }
 
+void
+OSDWidget::setTextColor(const QColor& color)
+{
+    QPalette palette = this->palette();
+    palette.setColor( QPalette::Active, QPalette::WindowText, color );
+    setPalette(palette);
+}
+
 void
 OSDWidget::setScreen( int screen )
 {
@@ -432,6 +456,23 @@ OSDWidget::setScreen( int screen )
     m_screen = ( screen >= n ) ? n - 1 : screen;
 }
 
+void
+OSDWidget::setFontScale( int scale )
+{
+    double fontScale = static_cast<double>( scale ) / 100.0;
+
+    // update font, reuse old one
+    QFont newFont( font() );
+    newFont.setPointSizeF( defaultPointSize() * fontScale );
+    setFont( newFont );
+}
+
+void
+OSDWidget::setHideWhenFullscreenWindowIsActive( bool hide )
+{
+    m_hideWhenFullscreenWindowIsActive = hide;
+}
+
 
 /////////////////////////////////////////////////////////////////////////////////////////
 // Class OSDPreviewWidget
@@ -442,14 +483,10 @@ OSDPreviewWidget::OSDPreviewWidget( QWidget *parent )
         , m_dragging( false )
 {
     setObjectName( "osdpreview" );
-    m_text = i18n( "On-Screen-Display preview\nDrag to reposition" );
-    m_duration = 0;
-    m_alignment = static_cast<Alignment>( AmarokConfig::osdAlignment() );
-    m_y = AmarokConfig::osdYOffset();
-    m_cover = Amarok::icon();
+    setDuration( 0 );
+    setImage( Amarok::icon() );
     setTranslucent( AmarokConfig::osdUseTranslucency() );
-    setText( m_text );
-    setImage( m_cover );
+    setText( i18n( "On-Screen-Display preview\nDrag to reposition" ) );
 }
 
 void
@@ -464,6 +501,15 @@ OSDPreviewWidget::mousePressEvent( QMouseEvent *event )
     }
 }
 
+void
+OSDPreviewWidget::setUseCustomColors(const bool use, const QColor& fg)
+{
+    if( use )
+        setTextColor( fg );
+    else
+        unsetColors();
+}
+
 void
 OSDPreviewWidget::mouseReleaseEvent( QMouseEvent * /*event*/ )
 {
@@ -479,8 +525,8 @@ OSDPreviewWidget::mouseReleaseEvent( QMouseEvent * /*event*/ )
         if( currentScreen != -1 )
         {
             // set new data
-            m_screen = currentScreen;
-            m_y      = QWidget::y();
+            setScreen( currentScreen );
+            setOffset( QWidget::y() );
 
             emit positionChanged();
         }
@@ -494,13 +540,13 @@ OSDPreviewWidget::mouseMoveEvent( QMouseEvent *e )
     {
         // Here we implement a "snap-to-grid" like positioning system for the preview widget
 
-        const QRect screen      = QApplication::desktop()->screenGeometry( m_screen );
-        const uint  hcenter     = screen.width() / 2;
-        const uint  eGlobalPosX = e->globalPos().x() - screen.left();
-        const uint  snapZone    = screen.width() / 24;
+        const QRect screenRect  = QApplication::desktop()->screenGeometry( screen() );
+        const uint  hcenter     = screenRect.width() / 2;
+        const uint  eGlobalPosX = e->globalPos().x() - screenRect.left();
+        const uint  snapZone    = screenRect.width() / 24;
 
-        QPoint destination = e->globalPos() - m_dragOffset - screen.topLeft();
-        int maxY = screen.height() - height() - MARGIN;
+        QPoint destination = e->globalPos() - m_dragOffset - screenRect.topLeft();
+        int maxY = screenRect.height() - height() - MARGIN;
         if( destination.y() < MARGIN )
             destination.ry() = MARGIN;
         if( destination.y() > maxY )
@@ -508,29 +554,30 @@ OSDPreviewWidget::mouseMoveEvent( QMouseEvent *e )
 
         if( eGlobalPosX < ( hcenter - snapZone ) )
         {
-            m_alignment = Left;
+            setAlignment(Left);
             destination.rx() = MARGIN;
         }
         else if( eGlobalPosX > ( hcenter + snapZone ) )
         {
-            m_alignment = Right;
-            destination.rx() = screen.width() - MARGIN - width();
+            setAlignment(Right);
+            destination.rx() = screenRect.width() - MARGIN - width();
         }
         else {
-            const uint eGlobalPosY = e->globalPos().y() - screen.top();
-            const uint vcenter     = screen.height() / 2;
+            const uint eGlobalPosY = e->globalPos().y() - screenRect.top();
+            const uint vcenter     = screenRect.height() / 2;
 
             destination.rx() = hcenter - width() / 2;
 
             if( eGlobalPosY >= ( vcenter - snapZone ) && eGlobalPosY <= ( vcenter + snapZone ) )
             {
-                m_alignment = Center;
+                setAlignment(Center);
                 destination.ry() = vcenter - height() / 2;
             }
-            else m_alignment = Middle;
+            else
+                setAlignment(Middle);
         }
 
-        destination += screen.topLeft();
+        destination += screenRect.topLeft();
 
         move( destination );
     }
@@ -645,6 +692,7 @@ Amarok::OSD::applySettings()
     setOffset( AmarokConfig::osdYOffset() );
     setScreen( AmarokConfig::osdScreen() );
     setFontScale( AmarokConfig::osdFontScaling() );
+    setHideWhenFullscreenWindowIsActive( AmarokConfig::osdHideOnFullscreen() );
 
     if( AmarokConfig::osdUseCustomColors() )
         setTextColor( AmarokConfig::osdTextColor() );
@@ -673,7 +721,7 @@ Amarok::OSD::muteStateChanged( bool mute )
 {
     Q_UNUSED( mute )
 
-    volumeChanged( m_volume );
+    volumeChanged( The::engineController()->volume() );
 }
 
 void
@@ -681,7 +729,7 @@ Amarok::OSD::trackPlaying( Meta::TrackPtr track )
 {
     m_currentTrack = track;
 
-    m_paused = false;
+    setPaused(false);
     show( m_currentTrack );
 }
 
@@ -691,7 +739,7 @@ Amarok::OSD::stopped()
     setImage( QImage( KIconLoader::global()->iconPath( "amarok", -KIconLoader::SizeHuge ) ) );
     setRating( 0 ); // otherwise stars from last rating change are visible
     OSDWidget::show( i18n( "Stopped" ) );
-    m_paused = false;
+    setPaused(false);
 }
 
 void
@@ -700,7 +748,7 @@ Amarok::OSD::paused()
     setImage( QImage( KIconLoader::global()->iconPath( "amarok", -KIconLoader::SizeHuge ) ) );
     setRating( 0 ); // otherwise stars from last rating change are visible
     OSDWidget::show( i18n( "Paused" ) );
-    m_paused = true;
+    setPaused(true);
 }
 
 void
