@@ -41,7 +41,7 @@ using ::testing::Return;
 using ::testing::AnyNumber;
 using ::testing::_;
 
-QTEST_KDEMAIN_CORE( TestSingleCollectionTreeItemModel )
+QTEST_KDEMAIN( TestSingleCollectionTreeItemModel, GUI )
 
 void addMockTrack( Collections::CollectionTestImpl *coll, const QString &trackName, const QString &artistName, const QString &albumName )
 {
@@ -53,9 +53,12 @@ void addMockTrack( Collections::CollectionTestImpl *coll, const QString &trackNa
     EXPECT_CALL( *track, uidUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( trackName + "_" + artistName + "_" + albumName ) );
     EXPECT_CALL( *track, isPlayable() ).Times( AnyNumber() ).WillRepeatedly( Return( true ) );
     EXPECT_CALL( *track, playableUrl() ).Times( AnyNumber() ).WillRepeatedly( Return( KUrl( '/' + track->uidUrl() ) ) );
+    EXPECT_CALL( *track, composer() ).Times( AnyNumber() ).WillRepeatedly( Return( Meta::ComposerPtr() ) );
+    EXPECT_CALL( *track, genre() ).Times( AnyNumber() ).WillRepeatedly( Return( Meta::GenrePtr() ) );
+    EXPECT_CALL( *track, year() ).Times( AnyNumber() ).WillRepeatedly( Return( Meta::YearPtr() ) );
     coll->mc->addTrack( trackPtr );
 
-    Meta::AlbumPtr albumPtr = coll->mc->albumMap().value( albumName );
+    Meta::AlbumPtr albumPtr = coll->mc->albumMap().value( albumName, QString() /* no album artist */ );
     Meta::MockAlbum *album;
     Meta::TrackList albumTracks;
     if( albumPtr )
@@ -109,6 +112,7 @@ void addMockTrack( Collections::CollectionTestImpl *coll, const QString &trackNa
     artistTracks << trackPtr;
     EXPECT_CALL( *artist, tracks() ).Times( AnyNumber() ).WillRepeatedly( Return( artistTracks ) );
     EXPECT_CALL( *track, artist() ).Times( AnyNumber() ).WillRepeatedly( Return( artistPtr ) );
+    EXPECT_CALL( *album, albumArtist() ).Times( AnyNumber() ).WillRepeatedly( Return( artistPtr ) );
 }
 
 TestSingleCollectionTreeItemModel::TestSingleCollectionTreeItemModel() : QObject()
@@ -119,13 +123,10 @@ TestSingleCollectionTreeItemModel::TestSingleCollectionTreeItemModel() : QObject
     qRegisterMetaType<Meta::AlbumList>();
     qRegisterMetaType<Meta::ArtistList>();
     qRegisterMetaType<Meta::DataList>();
-
-    m_app = new QApplication( 0, 0 ); // need an application for pixmaps
 }
 
 TestSingleCollectionTreeItemModel::~TestSingleCollectionTreeItemModel()
 {
-    delete m_app;
 }
 
 void
@@ -147,15 +148,13 @@ TestSingleCollectionTreeItemModel::testAddNewArtist()
     Collections::CollectionTestImpl *coll = new Collections::CollectionTestImpl( "test" );
     addMockTrack( coll, "track1", "artist1", "album1" );
 
-    QList<int> levels;
+    QList<CategoryId::CatMenuId> levels;
     levels<< CategoryId::Artist << CategoryId::Album;
 
     SingleCollectionTreeItemModel *model = new SingleCollectionTreeItemModel( coll, levels );
 
     loadChildren( model, QModelIndex() );
 
-    QTest::qWait( 30 );
-
     QCOMPARE( model->rowCount( QModelIndex() ), 1 );
 
     {
@@ -169,8 +168,6 @@ TestSingleCollectionTreeItemModel::testAddNewArtist()
 
     loadChildren( model, QModelIndex() );
 
-    QTest::qWait( 30 );
-
     QCOMPARE( model->rowCount( QModelIndex() ), 2 );
 
     QSet<QString> artists;
@@ -198,15 +195,13 @@ TestSingleCollectionTreeItemModel::testRemoveArtist()
     addMockTrack( coll, "track1", "artist1", "album1" );
     addMockTrack( coll, "track2", "artist2", "album2" );
 
-    QList<int> levels;
+    QList<CategoryId::CatMenuId> levels;
     levels<< CategoryId::Artist << CategoryId::Album;
 
     SingleCollectionTreeItemModel *model = new SingleCollectionTreeItemModel( coll, levels );
 
     loadChildren( model, QModelIndex() );
 
-    QTest::qWait( 30 );
-
     QCOMPARE( model->rowCount( QModelIndex() ), 2 );
 
     {
@@ -231,8 +226,6 @@ TestSingleCollectionTreeItemModel::testRemoveArtist()
 
     loadChildren( model, QModelIndex() );
 
-    QTest::qWait( 30 );
-
     QCOMPARE( model->rowCount( QModelIndex() ), 1 );
 
     {
@@ -251,13 +244,12 @@ TestSingleCollectionTreeItemModel::testAddTrack()
     addMockTrack( coll, "track1", "artist1", "album1" );
     addMockTrack( coll, "track2", "artist2", "album2" );
 
-    QList<int> levels;
+    QList<CategoryId::CatMenuId> levels;
     levels<< CategoryId::Artist << CategoryId::Album;
 
     SingleCollectionTreeItemModel *model = new SingleCollectionTreeItemModel( coll, levels );
 
     loadChildren( model, QModelIndex() );
-    QTest::qWait( 30 );
 
     QCOMPARE( model->rowCount( QModelIndex() ), 2 );
 
@@ -326,15 +318,12 @@ TestSingleCollectionTreeItemModel::testAddTrackWithFilter()
     Collections::CollectionTestImpl *coll = new Collections::CollectionTestImpl( "test" );
     addMockTrack( coll, "track1", "artist1", "album1" );
 
-    QList<int> levels;
-    levels<< CategoryId::Artist << CategoryId::Album;
+    QList<CategoryId::CatMenuId> levels;
+    levels << CategoryId::Artist << CategoryId::Album;
 
     SingleCollectionTreeItemModel *model = new SingleCollectionTreeItemModel( coll, levels );
 
     loadChildren( model, QModelIndex() );
-
-    QTest::qWait( 100 ); // note: increasing this count makes the test less likely to fail. Somehow it still fails sometimes.
-
     QCOMPARE( model->rowCount( QModelIndex() ), 1 );
 
     {
@@ -343,17 +332,16 @@ TestSingleCollectionTreeItemModel::testAddTrackWithFilter()
     }
 
     addMockTrack( coll, "track2", "artist2", "album2" );
-
     model->setCurrentFilter( "track2" );
-
     model->slotFilter();
-
-    QTest::qWait( 30 );
-
     loadChildren( model, QModelIndex() );
-
     QCOMPARE( model->rowCount( QModelIndex() ), 1 );
 
+    model->setCurrentFilter( QString() );
+    model->slotFilter();
+    loadChildren( model, QModelIndex() );
+    QCOMPARE( model->rowCount( QModelIndex() ), 2 );
+
     QModelIndex idx1 = model->index( 0, 0, QModelIndex() );
     QCOMPARE( model->data( idx1, Qt::DisplayRole ).toString(), QString( "artist2" ) );
 
