@@ -27,7 +27,6 @@
 
 #include <QFile>
 #include <QImage>
-#include <QApplication>
 
 #include <qtest_kde.h>
 
@@ -39,13 +38,10 @@ TestSqlAlbum::TestSqlAlbum()
     , m_storage( 0 )
     , m_tmpDir( 0 )
 {
-    m_app = new QApplication( 0, 0 ); // need an application for pixmaps
 }
 
 TestSqlAlbum::~TestSqlAlbum()
-{
-    delete m_app;
-}
+{ }
 
 void
 TestSqlAlbum::initTestCase()
@@ -203,8 +199,6 @@ TestSqlAlbum::testCapabilities()
     QVERIFY( btc );
     QVERIFY( btc->bookmarkAction() );
 
-    QVERIFY( !album->hasCapabilityInterface( Capabilities::Capability::Updatable ) );
-
     // need to delete the actions. They hold pointers that prevent the registry from cleaning
     // it's state.
     foreach( QAction *action, ac->actions() )
